 /* ++                                                                */
 /*                                                                   */
 /* Copyright (c) 1996 Microsoft Corporation                          */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   icm.cpy                                                         */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   Public header file for Image Color Management                   */
 /*                                                                   */
 /* Revision History:                                                 */
 /*                                                                   */
 /* --                                                                */

 %dcl #ICM_H_ char ext;
 %if #ICM_H_ = '' %then %do;
 %#ICM_H_ = 'Y';

 %dcl #WINGDI_ char ext;
 %if #WINGDI_ = '' %then %do;
 %include wingdi;
 %end;
 %dcl #WINUSER_ char ext;
 %if #WINUSER_ = '' %then %do;
 %include winuser;
 %end;

 /* Support for named color profiles */

 define alias   COLOR_NAME char(32-1) varz;
 define alias  PCOLOR_NAME pointer;
 define alias LPCOLOR_NAME pointer;

 define structure
   1 NAMED_PROFILE_INFO,
     2 dwFlags               type DWORD,
     2 dwCount               type DWORD,
     2 dwCountDevCoordinates type DWORD,
     2 szPrefix              type COLOR_NAME,
     2 szSuffix              type COLOR_NAME;
 define alias  PNAMED_PROFILE_INFO handle NAMED_PROFILE_INFO;
 define alias LPNAMED_PROFILE_INFO handle NAMED_PROFILE_INFO;


 /* Color spaces                                                      */
 /*                                                                   */
 /* The following color spaces are supported.                         */
 /* Gray, RGB, CMYK, XYZ, Yxy, Lab, generic 3 channel color spaces    */
 /* where the profiles defines how to interpret the 3 channels,       */
 /* named color spaces which can either be indices into the space     */
 /* or have color names, and multichannel spaces with 1 byte per      */
 /* channel upto MAX_COLOR_CHANNELS.                                  */

 dcl MAX_COLOR_CHANNELS fixed bin(31) value(8);
     /* maximum number of HiFi color channels */

 define structure
   1 GRAYCOLOR,
     2 gray    type WORD;

 define structure
   1 RGBCOLOR,
     2 red     type WORD,
     2 green   type WORD,
     2 blue    type WORD;

 define structure
   1 CMYKCOLOR,
     2 cyan    type WORD,
     2 magenta type WORD,
     2 yellow  type WORD,
     2 black   type WORD;

 define structure
   1 XYZCOLOR,
     2 X       type WORD,
     2 Y       type WORD,
     2 Z       type WORD;

 define structure
   1 YxyCOLOR,
     2 Y$      type WORD, /* Y in C, Y$ in PL/I */
     2 x       type WORD,
     2 y       type WORD;

 define structure
   1 LabCOLOR,
     2 L       type WORD,
     2 a       type WORD,
     2 b       type WORD;

 define structure
   1 GENERIC3CHANNEL,
     2 ch1     type WORD,
     2 ch2     type WORD,
     2 ch3     type WORD;

 define structure
   1 NAMEDCOLOR,
     2 dwIndex type DWORD;

 define structure
   1 HiFiCOLOR,
     2 channel (MAX_COLOR_CHANNELS) type BYTE;


 define structure
   1 COLOR union,
     2 gray   type GRAYCOLOR,
     2 rgb    type RGBCOLOR,
     2 cmyk   type CMYKCOLOR,
     2 XYZ    type XYZCOLOR,
     2 Yxy    type YxyCOLOR,
     2 Lab    type LabCOLOR,
     2 gen3ch type GENERIC3CHANNEL,
     2 named  type NAMEDCOLOR,
     2 hifi   type HiFiCOLOR;
 define alias  PCOLOR handle COLOR;
 define alias LPCOLOR handle COLOR;

 define ordinal COLORTYPE
   ( COLOR_GRAY value(1),
     COLOR_RGB,
     COLOR_XYZ,
     COLOR_Yxy,
     COLOR_Lab,
     COLOR_3_CHANNEL, /* WORD per channel */
     COLOR_CMYK,
     COLOR_5_CHANNEL, /* BYTE per channel */
     COLOR_6_CHANNEL, /*      - do - */
     COLOR_7_CHANNEL, /*      - do - */
     COLOR_8_CHANNEL, /*      - do - */
     COLOR_NAMED
   ) precision(31);

 define alias  PCOLORTYPE pointer;
 define alias LPCOLORTYPE pointer;


 /* Bitmap formats supported */

 /* PL/I note: order changed as PL/I demands increasing ordinal value */
 define ordinal BMFORMAT
   ( BM_x555RGB       value('0000'xn),
     /* 16 bpp - 5 bits for Red & Blue, 6 bits for Green */
     BM_565RGB        value('0001'xn),
     BM_RGBTRIPLETS   value('0002'xn),
     BM_BGRTRIPLETS   value('0004'xn),
     BM_xRGBQUADS     value('0008'xn),
     BM_10b_RGB       value('0009'xn),
     BM_16b_RGB       value('000A'xn),
     BM_xBGRQUADS     value('0010'xn),
     BM_CMYKQUADS     value('0020'xn),

     BM_x555XYZ       value('0101'xn),
     BM_x555Yxy,
     BM_x555Lab,
     BM_x555G3CH,

     /* Packed 8 bits per channel => 8bpp for GRAY and         */
     /* 24bpp for the 3 channel colors, more for hifi channels */

     BM_XYZTRIPLETS   value('0201'xn),
     BM_YxyTRIPLETS,
     BM_LabTRIPLETS,
     BM_G3CHTRIPLETS,
     BM_5CHANNEL,
     BM_6CHANNEL,
     BM_7CHANNEL,
     BM_8CHANNEL,
     BM_GRAY,

     /* 32bpp - 8 bits per channel. The most significant byte is */
     /* ignored for the 3 channel colors.                         */

     BM_xG3CHQUADS    value('0304'xn),
     BM_KYMCQUADS,

     /* 32bpp - 10 bits per channel. The 2 most significant bits */
     /* are ignored.                                             */

     BM_10b_XYZ       value('0401'xn),
     BM_10b_Yxy,
     BM_10b_Lab,
     BM_10b_G3CH,

     /* 32bpp - named color indices (1-based) */

     BM_NAMED_INDEX,

     /* Packed 16 bits per channel => 16bpp for GRAY and */
     /* 48bpp for the 3 channel colors.                  */

     BM_16b_XYZ       value('0501'xn),
     BM_16b_Yxy,
     BM_16b_Lab,
     BM_16b_G3CH,
     BM_16b_GRAY
   ) precision(31);
 define alias  PBMFORMAT pointer;
 define alias LPBMFORMAT pointer;

 /* Callback function definition */

 define alias PBMCALLBACKFN
        entry (type ULONG,
               type ULONG,
               type ULONG)
     returns (type BOOL byvalue optional)
        limited WINAPI;
 define alias LPBMCALLBACKFN type PBMCALLBACKFN;


 /* ICC profile header */

 define structure
   1 PROFILEHEADER,
     2 phSize            type DWORD,/* profile size in bytes */
     2 phCMMType         type DWORD,/* CMM for this profile */
     2 phVersion         type DWORD,/* profile format version number */
     2 phClass           type DWORD,/* type of profile */
     2 phDataColorSpace  type DWORD,/* color space of data */
     2 phConnectionSpace type DWORD,/* PCS */
     2 phDateTime (2)    type DWORD,/* date profile was created */
     2 phSignature       type DWORD,/* magic number */
     2 phPlatform        type DWORD,/* primary platform */
     2 phProfileFlags    type DWORD,/* various bit settings */
     2 phManufacturer    type DWORD,/* device manufacturer */
     2 phModel           type DWORD,/* device model number */
     2 phAttributes (2)  type DWORD,/* device attributes */
     2 phRenderingIntent type DWORD,/* rendering intent */
     2 phIlluminant      type CIEXYZ,/* profile illuminant */
     2 phCreator         type DWORD,/* profile creator */
     2 phReserved (44)   type BYTE;/* reserved for future use */
 define alias  PPROFILEHEADER handle PROFILEHEADER;
 define alias LPPROFILEHEADER handle PROFILEHEADER;

 /* Profile class values */

 dcl CLASS_MONITOR    char value('mntr');
 dcl CLASS_PRINTER    char value('prtr');
 dcl CLASS_SCANNER    char value('scnr');
 dcl CLASS_LINK       char value('link');
 dcl CLASS_ABSTRACT   char value('abst');
 dcl CLASS_COLORSPACE char value('spac');
 dcl CLASS_NAMED      char value('nmcl');

 /* Color space values */

 dcl SPACE_XYZ        char value('XYZ ');
 dcl SPACE_Lab        char value('Lab ');
 dcl SPACE_Luv        char value('Luv ');
 dcl SPACE_YCbCr      char value('YCbr');
 dcl SPACE_Yxy        char value('Yxy ');
 dcl SPACE_RGB        char value('RGB ');
 dcl SPACE_GRAY       char value('GRAY');
 dcl SPACE_HSV        char value('HSV ');
 dcl SPACE_HLS        char value('HLS ');
 dcl SPACE_CMYK       char value('CMYK');
 dcl SPACE_CMY        char value('CMY ');
 dcl SPACE_2_CHANNEL  char value('2CLR');
 dcl SPACE_3_CHANNEL  char value('3CLR');
 dcl SPACE_4_CHANNEL  char value('4CLR');
 dcl SPACE_5_CHANNEL  char value('5CLR');
 dcl SPACE_6_CHANNEL  char value('6CLR');
 dcl SPACE_7_CHANNEL  char value('7CLR');
 dcl SPACE_8_CHANNEL  char value('8CLR');

 /* Profile flag bitfield values */

 dcl FLAG_EMBEDDEDPROFILE type DWORD value('00000001'xu);
 dcl FLAG_DEPENDENTONDATA type DWORD value('00000002'xu);

 /* Profile attributes bitfield values */

 dcl ATTRIB_TRANSPARENCY  type DWORD value('00000001'xu);
 dcl ATTRIB_MATTE         type DWORD value('00000002'xu);

 /* Rendering Intents */

 dcl INTENT_PERCEPTUAL            fixed bin(31) value(0);
 dcl INTENT_RELATIVE_COLORIMETRIC fixed bin(31) value(1);
 dcl INTENT_SATURATION            fixed bin(31) value(2);
 dcl INTENT_ABSOLUTE_COLORIMETRIC fixed bin(31) value(3);

 /* Profile data structure */

 define structure
   1 PROFILE,
     2 dwType       type DWORD,/* profile type */
     2 pProfileData type PVOID,
       /* filename or buffer containing profile */
     2 cbDataSize   type DWORD;/* size of profile data */
 define alias  PPROFILE handle PROFILE;
 define alias LPPROFILE handle PROFILE;


 /* Profile types to be used in the PROFILE structure */
 dcl PROFILE_FILENAME  fixed bin(31) value(1);
     /* profile data is NULL terminated filename */
 dcl PROFILE_MEMBUFFER fixed bin(31) value(2);
     /* profile data is a buffer containing the profile */

 /* Desired access mode for opening profiles */
 dcl PROFILE_READ      fixed bin(31) value(1);
     /* opened for read access */
 dcl PROFILE_READWRITE fixed bin(31) value(2);
     /* opened for read and write access */

 /* Handles returned to applications */

 define alias HPROFILE type HANDLE; /* handle to profile object */
 define alias PHPROFILE pointer;    /* better use HPROFILE byaddr! */
 define alias HTRANSFORM type HANDLE;
 /* handle to color transform object */

 /* Use built-in CMM selection in CreateMultiProfileTransform */

 dcl INDEX_DONT_CARE fixed bin(31) value(0);

 /* Tags found in ICC profiles */

 define alias   TAGTYPE type DWORD;
 define alias  PTAGTYPE pointer;   /* better use TAGTYPE byaddr! */
 define alias LPTAGTYPE pointer;   /* better use TAGTYPE byaddr! */

 /* Profile enumeration data structure */

 dcl ENUM_TYPE_VERSION type DWORD value('0300'xu);

 define structure
   1 ENUMTYPEA,
     2 dwSize            type DWORD,/* structure size */
     2 dwVersion         type DWORD,/* structure version */
     2 dwFields          type DWORD,/* bit fields */
     2 pDeviceName       type PCSTR,/* device friendly name */
     2 dwMediaType       type DWORD,/* media type */
     2 dwDitheringMode   type DWORD,/* dithering mode */
     2 dwResolution (2)  type DWORD,/* x and y resolutions */
     2 dwCMMType         type DWORD,/* cmm ID */
     2 dwClass           type DWORD,/* profile class */
     2 dwDataColorSpace  type DWORD,/* color space of data */
     2 dwConnectionSpace type DWORD,/* pcs */
     2 dwSignature       type DWORD,/* magic number */
     2 dwPlatform        type DWORD,/* primary platform */
     2 dwProfileFlags    type DWORD,/* various bit settings in profile*/
     2 dwManufacturer    type DWORD,/* manufacturer ID */
     2 dwModel           type DWORD,/* model ID */
     2 dwAttributes (2)  type DWORD,/* device attributes */
     2 dwRenderingIntent type DWORD,/* rendering intent */
     2 dwCreator         type DWORD,/* profile creator */
     2 dwDeviceClass     type DWORD;/* device class */
 define alias  PENUMTYPEA handle ENUMTYPEA;
 define alias LPENUMTYPEA handle ENUMTYPEA;

 define structure
   1 ENUMTYPEW,
     2 dwSize            type DWORD,/* structure size */
     2 dwVersion         type DWORD,/* structure version */
     2 dwFields          type DWORD,/* bit fields */
     2 pDeviceName       type PCWSTR,/* device friendly name */
     2 dwMediaType       type DWORD,/* media type */
     2 dwDitheringMode   type DWORD,/* dithering mode */
     2 dwResolution (2)  type DWORD,/* x and y resolutions */
     2 dwCMMType         type DWORD,/* cmm ID */
     2 dwClass           type DWORD,/* profile class */
     2 dwDataColorSpace  type DWORD,/* color space of data */
     2 dwConnectionSpace type DWORD,/* pcs */
     2 dwSignature       type DWORD,/* magic number */
     2 dwPlatform        type DWORD,/* primary platform */
     2 dwProfileFlags    type DWORD,/* various bit settings in profile*/
     2 dwManufacturer    type DWORD,/* manufacturer ID */
     2 dwModel           type DWORD,/* model ID */
     2 dwAttributes (2)  type DWORD,/* device attributes */
     2 dwRenderingIntent type DWORD,/* rendering intent */
     2 dwCreator         type DWORD,/* profile creator */
     2 dwDeviceClass     type DWORD;/* device class */
 define alias  PENUMTYPEW handle ENUMTYPEW;
 define alias LPENUMTYPEW handle ENUMTYPEW;

 /* Bitfields for enumeration record above */

 dcl ET_DEVICENAME             type DWORD value('00000001'xu);
 dcl ET_MEDIATYPE              type DWORD value('00000002'xu);
 dcl ET_DITHERMODE             type DWORD value('00000004'xu);
 dcl ET_RESOLUTION             type DWORD value('00000008'xu);
 dcl ET_CMMTYPE                type DWORD value('00000010'xu);
 dcl ET_CLASS                  type DWORD value('00000020'xu);
 dcl ET_DATACOLORSPACE         type DWORD value('00000040'xu);
 dcl ET_CONNECTIONSPACE        type DWORD value('00000080'xu);
 dcl ET_SIGNATURE              type DWORD value('00000100'xu);
 dcl ET_PLATFORM               type DWORD value('00000200'xu);
 dcl ET_PROFILEFLAGS           type DWORD value('00000400'xu);
 dcl ET_MANUFACTURER           type DWORD value('00000800'xu);
 dcl ET_MODEL                  type DWORD value('00001000'xu);
 dcl ET_ATTRIBUTES             type DWORD value('00002000'xu);
 dcl ET_RENDERINGINTENT        type DWORD value('00004000'xu);
 dcl ET_CREATOR                type DWORD value('00008000'xu);
 dcl ET_DEVICECLASS            type DWORD value('00010000'xn);

 /* Flags for creating color transforms */

 dcl PROOF_MODE                type DWORD value('00000001'xu);
 dcl NORMAL_MODE               type DWORD value('00000002'xu);
 dcl BEST_MODE                 type DWORD value('00000003'xu);
 dcl ENABLE_GAMUT_CHECKING     type DWORD value('00010000'xu);
 dcl USE_RELATIVE_COLORIMETRIC type DWORD value('00020000'xu);
 dcl FAST_TRANSLATE            type DWORD value('00040000'xu);
 dcl RESERVED                  type DWORD value('80000000'xu);

 /* Paremeter for GetPS2ColorSpaceArray */

 dcl CSA_A    fixed bin(31) value(1);
 dcl CSA_ABC  fixed bin(31) value(2);
 dcl CSA_DEF  fixed bin(31) value(3);
 dcl CSA_DEFG fixed bin(31) value(4);
 dcl CSA_GRAY fixed bin(31) value(5);
 dcl CSA_RGB  fixed bin(31) value(6);
 dcl CSA_CMYK fixed bin(31) value(7);
 dcl CSA_Lab  fixed bin(31) value(8);

 /* Parameter for CMGetInfo() */

 dcl CMM_WIN_VERSION     fixed bin(31) value(0);
 dcl CMM_IDENT           fixed bin(31) value(1);
 dcl CMM_DRIVER_VERSION  fixed bin(31) value(2);
 dcl CMM_DLL_VERSION     fixed bin(31) value(3);
 dcl CMM_VERSION         fixed bin(31) value(4);
 dcl CMM_DESCRIPTION     fixed bin(31) value(5);
 dcl CMM_LOGOICON        fixed bin(31) value(6);

 /* Parameter for CMTranslateRGBs() */

 dcl CMS_FORWARD         fixed bin(31) value(0);
 dcl CMS_BACKWARD        fixed bin(31) value(1);

 /* Constants for SetupColorMatching() */

 dcl COLOR_MATCH_VERSION fixed bin(15) value('0200'xn);

 /*  Constants for flags */

 dcl CMS_DISABLEICM        fixed bin(31) value(1);
     /* Disable color matching */
 dcl CMS_ENABLEPROOFING    fixed bin(31) value(2);
     /* Enable proofing */

 dcl CMS_SETRENDERINTENT   fixed bin(31) value(4);
     /* Use passed in value */
 dcl CMS_SETPROOFINTENT    fixed bin(31) value(8);
 dcl CMS_SETMONITORPROFILE fixed bin(15) value('10'xn);
     /* Use passed in profile name initially */
 dcl CMS_SETPRINTERPROFILE fixed bin(15) value('20'xn);
 dcl CMS_SETTARGETPROFILE  fixed bin(15) value('40'xn);

 dcl CMS_USEHOOK           fixed bin(15) value('80'xn);
     /* Use hook procedure in lpfnHook */
 dcl CMS_USEAPPLYCALLBACK  fixed bin(15) value('100'xn);
     /* Use callback procedure when applied */

 /*  Used to denote too-small buffers (output only) */

 dcl CMS_MONITOROVERFLOW   type DWORD value('80000000'xu);
 dcl CMS_PRINTEROVERFLOW   type DWORD value('40000000'xu);
 dcl CMS_TARGETOVERFLOW    type DWORD value('20000000'xu);


 /*  Structures (both ANSI and Unicode) */

 /*  Can't use type COLORMATCHSETUPW/A byaddr as alias used in       */
 /*  COLORMATCHSETUPW/A, and PL/I doesn't allow forward declarations */
 define alias PCMSCALLBACKW
        entry (handle COLORMATCHSETUPW,
               type LPARAM)
     returns (type BOOL byvalue optional)
        limited WINAPI;

 define alias PCMSCALLBACKA
        entry (handle COLORMATCHSETUPA,
               type LPARAM)
     returns (type BOOL byvalue optional)
        limited WINAPI;

 define structure
   1 COLORMATCHSETUPW,
     2 dwSize              type DWORD,/* Size of structure in bytes */
     2 dwVersion           type DWORD,/* Set to COLOR_MATCH_VERSION */
     2 dwFlags             type DWORD,
       /* See constants listed previously */
     2 hwndOwner           type HWND, /* Window handle of owner */
     2 pSourceName         type PCWSTR,
       /* Name of Image Source, defaults to "sRGB Color Space" */
     2 pDisplayName        type PCWSTR,
       /* If sysnull, defaults to first enumerated monitor */
     2 pPrinterName        type PCWSTR,
       /* If sysnull, defaults to default printer. */
     2 dwRenderIntent      type DWORD,/* Rendering Intent */
     2 dwProofingIntent    type DWORD,/* Rendering Intent for Proofing*/
     2 pMonitorProfile     type PWSTR,/* Monitor profile name */
     2 ccMonitorProfile    type DWORD,/* Size of above in characters */
     2 pPrinterProfile     type PWSTR,/* Printer profile name */
     2 ccPrinterProfile    type DWORD,/* Size of above in characters */
     2 pTargetProfile      type PWSTR,/* Target profile name */
     2 ccTargetProfile     type DWORD,/* Size of above in characters */
     2 lpfnHook            type DLGPROC,/* Hook Procedure address */
     2 lParam              type LPARAM,
       /* Given to hook procedure at WM_INITDIALOG */
     2 lpfnApplyCallback   type PCMSCALLBACKW,
       /* Callback Procedure address when apply is pushed */
     2 lParamApplyCallback type LPARAM;
       /* Given to callback Procedure for apply */
 define alias  PCOLORMATCHSETUPW handle COLORMATCHSETUPW;
 define alias LPCOLORMATCHSETUPW handle COLORMATCHSETUPW;

 define structure
   1 COLORMATCHSETUPA,
     2 dwSize              type DWORD,/* Size of structure in bytes */
     2 dwVersion           type DWORD,/* Set to COLOR_MATCH_VERSION */
     2 dwFlags             type DWORD,
       /* See constants listed previously */
     2 hwndOwner           type HWND, /* Window handle of owner */
     2 pSourceName         type PCSTR,
       /* Name of Image Source, defaults to "sRGB Color Space" */
     2 pDisplayName        type PCSTR,
       /* If sysnull, defaults to first enumerated monitor */
     2 pPrinterName        type PCSTR,
       /* If sysnull, defaults to default printer. */
     2 dwRenderIntent      type DWORD,/* Rendering Intent */
     2 dwProofingIntent    type DWORD,/* Rendering Intent for Proofing*/
     2 pMonitorProfile     type PSTR,/* Monitor profile name */
     2 ccMonitorProfile    type DWORD,/* Size of above in characters */
     2 pPrinterProfile     type PSTR,/* Printer profile name */
     2 ccPrinterProfile    type DWORD,/* Size of above in characters */
     2 pTargetProfile      type PSTR,/* Target profile name */
     2 ccTargetProfile     type DWORD,/* Size of above in characters */
     2 lpfnHook            type DLGPROC,/* Hook Procedure address */
     2 lParam              type LPARAM,
       /* Given to hook procedure at WM_INITDIALOG */
     2 lpfnApplyCallback   type PCMSCALLBACKA,
       /* Callback Procedure address when apply is pushed */
     2 lParamApplyCallback type LPARAM;
       /* Given to callback Procedure for apply */
 define alias  PCOLORMATCHSETUPA handle COLORMATCHSETUPA;
 define alias LPCOLORMATCHSETUPA handle COLORMATCHSETUPA;

 /* Windows API definitions */

 dcl OpenColorProfileA
     entry (type PROFILE byaddr,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type HPROFILE byvalue optional)
     WINAPI external('OpenColorProfileA');

 dcl OpenColorProfileW
     entry (type PROFILE byaddr,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type HPROFILE byvalue optional)
     WINAPI external('OpenColorProfileW');

 dcl CloseColorProfile
     entry (type HPROFILE)
     returns (type BOOL byvalue optional)
     WINAPI external('CloseColorProfile');

 dcl GetColorProfileFromHandle
     entry (type HPROFILE,
            type PBYTE,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetColorProfileFromHandle');

 dcl IsColorProfileValid
     entry (type HPROFILE,
            type BOOL byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('IsColorProfileValid');

 dcl CreateProfileFromLogColorSpaceA
     entry (type LOGCOLORSPACEA byaddr,
            type PBYTE byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('CreateProfileFromLogColorSpaceA');

 dcl CreateProfileFromLogColorSpaceW
     entry (type LOGCOLORSPACEW byaddr,
            type PBYTE byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('CreateProfileFromLogColorSpaceW');

 dcl GetCountColorProfileElements
     entry (type HPROFILE,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetCountColorProfileElements');

 dcl GetColorProfileHeader
     entry (type HPROFILE,
            type PROFILEHEADER byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetColorProfileHeader');

 dcl GetColorProfileElementTag
     entry (type HPROFILE,
            type DWORD,
            type TAGTYPE byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetColorProfileElementTag');

 dcl IsColorProfileTagPresent
     entry (type HPROFILE,
            type TAGTYPE,
            type BOOL byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('IsColorProfileTagPresent');

 dcl GetColorProfileElement
     entry (type HPROFILE,
            type TAGTYPE,
            type DWORD,
            type DWORD byaddr,
            type PVOID,
            type BOOL byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetColorProfileElement');

 dcl SetColorProfileHeader
     entry (type HPROFILE,
            type PROFILEHEADER byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('SetColorProfileHeader');

 dcl SetColorProfileElementSize
     entry (type HPROFILE,
            type TAGTYPE,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('SetColorProfileElementSize');

 dcl SetColorProfileElement
     entry (type HPROFILE,
            type TAGTYPE,
            type DWORD,
            type DWORD byaddr,
            type PVOID)
     returns (type BOOL byvalue optional)
     WINAPI external('SetColorProfileElement');

 dcl SetColorProfileElementReference
     entry (type HPROFILE,
            type TAGTYPE,
            type TAGTYPE)
     returns (type BOOL byvalue optional)
     WINAPI external('SetColorProfileElementReference');

 dcl GetPS2ColorSpaceArray
     entry (type HPROFILE,
            type DWORD,
            type DWORD,
            type PBYTE,
            type DWORD byaddr,
            type BOOL byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetPS2ColorSpaceArray');

 dcl GetPS2ColorRenderingIntent
     entry (type HPROFILE,
            type DWORD,
            type PBYTE,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetPS2ColorRenderingIntent');

 dcl GetPS2ColorRenderingDictionary
     entry (type HPROFILE,
            type DWORD,
            type PBYTE,
            type DWORD byaddr,
            type BOOL byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetPS2ColorRenderingDictionary');

 dcl GetNamedProfileInfo
     entry (type HPROFILE,
            type NAMED_PROFILE_INFO byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetNamedProfileInfo');

 dcl ConvertColorNameToIndex
     entry (type HPROFILE,
            (*) type COLOR_NAME byaddr,
            (*) type DWORD byaddr,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('ConvertColorNameToIndex');

 dcl ConvertIndexToColorName
     entry (type HPROFILE,
            (*) type DWORD byaddr,
            (*) type COLOR_NAME byaddr,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('ConvertIndexToColorName');

 dcl CreateDeviceLinkProfile
     entry ((*) type HPROFILE byaddr,
            type DWORD,
            (*) type DWORD byaddr,
            type DWORD,
            type DWORD,
            type PBYTE byaddr,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('CreateDeviceLinkProfile');

 dcl CreateColorTransformA
     entry (type LOGCOLORSPACEA byaddr,
            type HPROFILE,
            type HPROFILE,
            type DWORD)
     returns (type HTRANSFORM byvalue optional)
     WINAPI external('CreateColorTransformA');

 dcl CreateColorTransformW
     entry (type LOGCOLORSPACEW byaddr,
            type HPROFILE,
            type HPROFILE,
            type DWORD)
     returns (type HTRANSFORM byvalue optional)
     WINAPI external('CreateColorTransformW');

 dcl CreateMultiProfileTransform
     entry ((*) type HPROFILE byaddr,
            type DWORD,
            (*) type DWORD BYADDR,
            type DWORD,
            type DWORD,
            type DWORD)
     returns (type HTRANSFORM byvalue optional)
     WINAPI external('CreateMultiProfileTransform');

 dcl DeleteColorTransform
     entry (type HTRANSFORM)
     returns (type BOOL byvalue optional)
     WINAPI external('DeleteColorTransform');

 dcl TranslateBitmapBits
     entry (type HTRANSFORM,
            type PVOID,
            type BMFORMAT,
            type DWORD,
            type DWORD,
            type DWORD,
            type PVOID,
            type BMFORMAT,
            type DWORD,
            type PBMCALLBACKFN,
            type LPARAM)
     returns (type BOOL byvalue optional)
     WINAPI external('TranslateBitmapBits');

 dcl CheckBitmapBits
     entry (type HTRANSFORM,
            type PVOID,
            type BMFORMAT,
            type DWORD,
            type DWORD,
            type DWORD,
            type PBYTE,
            type PBMCALLBACKFN,
            type LPARAM)
     returns (type BOOL byvalue optional)
     WINAPI external('CheckBitmapBits');

 dcl TranslateColors
     entry (type HTRANSFORM,
            (*) type COLOR byaddr,
            type DWORD,
            type COLORTYPE,
            (*) type COLOR byaddr,
            type COLORTYPE)
     returns (type BOOL byvalue optional)
     WINAPI external('TranslateColors');

 dcl CheckColors
     entry (type HTRANSFORM,
            type COLOR byaddr,
            type DWORD,
            type COLORTYPE,
            type PBYTE)
     returns (type BOOL byvalue optional)
     WINAPI external('CheckColors');

 dcl GetCMMInfo
     entry (type HTRANSFORM,
            type DWORD)
     returns (type DWORD byvalue optional)
     WINAPI external('GetCMMInfo');

 dcl RegisterCMMA
     entry (char (*) varz byaddr nonasgn optional,
            type DWORD,
            char (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('RegisterCMMA');

 dcl RegisterCMMW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type DWORD,
            WCHAR (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('RegisterCMMW');

 dcl UnregisterCMMA
     entry (char (*) varz byaddr nonasgn optional,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('UnregisterCMMA');

 dcl UnregisterCMMW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('UnregisterCMMW');

 dcl SelectCMM
     entry (type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('SelectCMM');

 dcl GetColorDirectoryA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr,
            type PDWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('GetColorDirectoryA');

 dcl GetColorDirectoryW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr,
            type PDWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('GetColorDirectoryW');

 dcl InstallColorProfileA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('InstallColorProfileA');

 dcl InstallColorProfileW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('InstallColorProfileW');

 dcl UninstallColorProfileA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn,
            type BOOL)
     returns (type BOOL byvalue optional)
     WINAPI external('UninstallColorProfileA');

 dcl UninstallColorProfileW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn,
            type BOOL)
     returns (type BOOL byvalue optional)
     WINAPI external('UninstallColorProfileW');

 dcl EnumColorProfilesA
     entry (char (*) varz byaddr nonasgn optional,
            type ENUMTYPEA byaddr,
            type PBYTE,
            type DWORD byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('EnumColorProfilesA');

 dcl EnumColorProfilesW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type ENUMTYPEW byaddr,
            type PBYTE,
            type DWORD byaddr,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('EnumColorProfilesW');

 dcl SetStandardColorSpaceProfileA
     entry (char (*) varz byaddr nonasgn optional,
            type DWORD,
            char (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('SetStandardColorSpaceProfileA');

 dcl SetStandardColorSpaceProfileW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type DWORD,
            WCHAR (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('SetStandardColorSpaceProfileW');

 dcl GetStandardColorSpaceProfileA
     entry (char (*) varz byaddr nonasgn optional,
            type DWORD,
            char (*) varz byaddr optional,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetStandardColorSpaceProfileA');

 dcl GetStandardColorSpaceProfileW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            type DWORD,
            WCHAR (*) varz byaddr optional,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('GetStandardColorSpaceProfileW');

 dcl AssociateColorProfileWithDeviceA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn,
            char (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('AssociateColorProfileWithDeviceA');

 dcl AssociateColorProfileWithDeviceW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn,
            WCHAR (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('AssociateColorProfileWithDeviceW');

 dcl DisassociateColorProfileFromDeviceA
     entry (char (*) varz byaddr nonasgn optional,
            char (*) varz byaddr nonasgn,
            char (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('DisassociateColorProfileFromDeviceA');

 dcl DisassociateColorProfileFromDeviceW
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn,
            WCHAR (*) varz byaddr nonasgn)
     returns (type BOOL byvalue optional)
     WINAPI external('DisassociateColorProfileFromDeviceW');

 dcl SetupColorMatchingW
     entry (type COLORMATCHSETUPW byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('SetupColorMatchingW');

 dcl SetupColorMatchingA
     entry (type COLORMATCHSETUPA byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('SetupColorMatchingA');


 %dcl CreateColorTransform               char ext;
 %dcl OpenColorProfile                   char ext;
 %dcl CreateProfileFromLogColorSpace     char ext;
 %dcl RegisterCMM                        char ext;
 %dcl UnregisterCMM                      char ext;
 %dcl GetColorDirectory                  char ext;
 %dcl InstallColorProfile                char ext;
 %dcl UninstallColorProfile              char ext;
 %dcl AssociateColorProfileWithDevice    char ext;
 %dcl DisassociateColorProfileFromDevice char ext;
 %dcl EnumColorProfiles                  char ext;
 %dcl SetStandardColorSpaceProfile       char ext;
 %dcl GetStandardColorSpaceProfile       char ext;
 %dcl SetupColorMatching                 char ext;

 %dcl UNICODE char ext;
 %if UNICODE ^= '' %then %do;
 define alias  ENUMTYPE         type  ENUMTYPEW;
 define alias PENUMTYPE         type PENUMTYPEW;
 define alias   COLORMATCHSETUP type   COLORMATCHSETUPW;
 define alias  PCOLORMATCHSETUP type  PCOLORMATCHSETUPW;
 define alias LPCOLORMATCHSETUP type LPCOLORMATCHSETUPW;
 define alias PCMSCALLBACK      type PCMSCALLBACKW;

 %CreateColorTransform            = 'CreateColorTransformW';
 %OpenColorProfile                = 'OpenColorProfileW';
 %CreateProfileFromLogColorSpace  = 'CreateProfileFromLogColorSpaceW';
 %RegisterCMM                     = 'RegisterCMMW';
 %UnregisterCMM                   = 'UnregisterCMMW';
 %GetColorDirectory               = 'GetColorDirectoryW';
 %InstallColorProfile             = 'InstallColorProfileW';
 %UninstallColorProfile           = 'UninstallColorProfileW';
 %AssociateColorProfileWithDevice = 'AssociateColorProfileWithDeviceW';
 %DisassociateColorProfileFromDevice
    ='DisassociateColorProfileFromDeviceW';
 %EnumColorProfiles               = 'EnumColorProfilesW';
 %SetStandardColorSpaceProfile    = 'SetStandardColorSpaceProfileW';
 %GetStandardColorSpaceProfile    = 'GetStandardColorSpaceProfileW';
 %SetupColorMatching              = 'SetupColorMatchingW';

 %end; %else %do;

 define alias  ENUMTYPE         type  ENUMTYPEA;
 define alias PENUMTYPE         type PENUMTYPEA;
 define alias   COLORMATCHSETUP type   COLORMATCHSETUPA;
 define alias  PCOLORMATCHSETUP type  PCOLORMATCHSETUPA;
 define alias LPCOLORMATCHSETUP type LPCOLORMATCHSETUPA;
 define alias PCMSCALLBACK      type PCMSCALLBACKA;

 %CreateColorTransform            = 'CreateColorTransformA';
 %OpenColorProfile                = 'OpenColorProfileA';
 %CreateProfileFromLogColorSpace  = 'CreateProfileFromLogColorSpaceA';
 %RegisterCMM                     = 'RegisterCMMA';
 %UnregisterCMM                   = 'UnregisterCMMA';
 %GetColorDirectory               = 'GetColorDirectoryA';
 %InstallColorProfile             = 'InstallColorProfileA';
 %UninstallColorProfile           = 'UninstallColorProfileA';
 %AssociateColorProfileWithDevice = 'AssociateColorProfileWithDeviceA';
 %DisassociateColorProfileFromDevice
    ='DisassociateColorProfileFromDeviceA';
 %EnumColorProfiles               = 'EnumColorProfilesA';
 %SetStandardColorSpaceProfile    = 'SetStandardColorSpaceProfileA';
 %GetStandardColorSpaceProfile    = 'GetStandardColorSpaceProfileA';
 %SetupColorMatching              = 'SetupColorMatchingA';

 %end; /* !UNICODE */

 /* Transform returned by CMM */

 define alias HCMTRANSFORM type HANDLE;

 /* Pointer to ICC color profile data. */

 define alias LPDEVCHARACTER type PVOID;

 /* CMM API definition */

 dcl CMCheckColors
     entry (type HCMTRANSFORM,         /* transform handle */
            (*) type COLOR byaddr,     /* array of COLORs */
            type DWORD,                /* COLOR array size */
            type COLORTYPE,            /* input color type */
            type LPBYTE)               /* buffer for results */
     returns (type BOOL byvalue optional)
     WINAPI external('CMCheckColors');

 dcl CMCheckColorsInGamut
     entry (type HCMTRANSFORM,         /* transform handle */
            (*) type RGBTRIPLE byaddr, /* RGB triple array */
            type LPBYTE,               /* buffer for results */
            type UINT)                 /* result buffer size */
     returns (type BOOL byvalue optional)
     WINAPI external('CMCheckColorsInGamut');

 dcl CMCheckRGBs
     entry (type HCMTRANSFORM,         /* transform handle */
            type LPVOID,               /* source bitmap bits */
            type BMFORMAT,             /* source bitmap format */
            type DWORD,                /* source bitmap width */
            type DWORD,                /* source bitmap hight */
            type DWORD,                /* source bitmap delta */
            type LPBYTE,               /* buffer for results */
            type PBMCALLBACKFN optional,/*pointer to callback function*/
            type LPARAM)      /* caller-defined parameter to callback */
     returns (type BOOL byvalue optional)
     WINAPI external('CMCheckRGBs');

 dcl CMConvertColorNameToIndex
     entry (type HPROFILE,
            (*) type COLOR_NAME byaddr,
            (*) type DWORD byaddr,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('CMConvertColorNameToIndex');

 dcl CMConvertIndexToColorName
     entry (type HPROFILE,
            (*) type DWORD byaddr,
            (*) type COLOR_NAME byaddr,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('CMConvertColorNameToIndex');

 dcl CMCreateDeviceLinkProfile
     entry ((*) type HPROFILE byaddr,  /* array of profile handles */
            type DWORD,                /* profile handle array size */
            (*) type DWORD byaddr,     /* array of rendering intents */
            type DWORD,                /* intent array size */
            type DWORD,                /* transform creation flags */
            type LPBYTE byaddr)        /* pointer to pointer to buffer*/
     returns (type BOOL byvalue optional)
     WINAPI external('CMCreateDeviceLinkProfile');

 dcl CMCreateMultiProfileTransform
     entry ((*) type HPROFILE byaddr,  /* array of profile handles */
            type DWORD,                /* profile handle array size */
            (*) type DWORD byaddr,     /* array of rendering intents */
            type DWORD,                /* intent array size */
            type DWORD)                /* transform creation flags */
     returns (type HCMTRANSFORM byvalue optional)
     WINAPI external('CMCreateMultiProfileTransform');

 dcl CMCreateProfile
     entry (type LOGCOLORSPACEA byaddr,
                                  /* pointer to a logical color space */
            type LPDEVCHARACTER byaddr)
                                  /* pointer to pointer to buffer */
     returns (type BOOL byvalue optional)
     WINAPI external('CMCreateProfile');

 dcl CMCreateProfileW
     entry (type LOGCOLORSPACEW byaddr,
                                  /* pointer to a logical color space */
            type LPDEVCHARACTER byaddr)
                                  /* pointer to pointer to buffer */
     returns (type BOOL byvalue optional)
     WINAPI external('CMCreateProfileW');

 dcl CMCreateTransform
     entry (type LOGCOLORSPACEA byaddr,
                                  /* pointer to logical color space */
            type LPDEVCHARACTER,       /* profile data */
            type LPDEVCHARACTER)       /* target profile data */
     returns (type HCMTRANSFORM byvalue optional)
     WINAPI external('CMCreateTransform');

 dcl CMCreateTransformW
     entry (type LOGCOLORSPACEW byaddr,
                                  /* pointer to logical color space */
            type LPDEVCHARACTER,       /* profile data */
            type LPDEVCHARACTER)       /* target profile data */
     returns (type HCMTRANSFORM byvalue optional)
     WINAPI external('CMCreateTransformW');

 dcl CMCreateTransformExt
     entry (type LOGCOLORSPACEA byaddr,
                                  /* pointer to logical color space */
            type LPDEVCHARACTER,       /* profile data */
            type LPDEVCHARACTER,       /* target profile data */
            type DWORD)                /* creation flags */
     returns (type HCMTRANSFORM byvalue optional)
     WINAPI external('CMCreateTransformExt');

 dcl CMCreateTransformExtW
     entry (type LOGCOLORSPACEW byaddr,
                                  /* pointer to logical color space */
            type LPDEVCHARACTER,       /* profile data */
            type LPDEVCHARACTER,       /* target profile data */
            type DWORD)                /* creation flags */
     returns (type HCMTRANSFORM byvalue optional)
     WINAPI external('CMCreateTransformExtW');

 dcl CMDeleteTransform
     entry (type HCMTRANSFORM)     /* transform handle to be deleted. */
     returns (type BOOL byvalue optional)
     WINAPI external('CMDeleteTransform');

 dcl CMGetInfo
     entry (type DWORD)
     returns (type DWORD byvalue optional)
     WINAPI external('CMGetInfo');

 dcl CMGetNamedProfileInfo
     entry (type HPROFILE,             /* profile handle */
            type NAMED_PROFILE_INFO byaddr)
                                   /* pointer to named profile info */
     returns (type BOOL byvalue optional)
     WINAPI external('CMGetNamedProfileInfo');

 dcl CMGetPS2ColorRenderingDictionary
     entry (type HPROFILE,
            type DWORD,
            type LPBYTE,
            type DWORD byaddr,
            type BOOL byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('CMGetPS2ColorRenderingDictionary');

 dcl CMGetPS2ColorRenderingIntent
     entry (type HPROFILE,
            type DWORD,
            type LPBYTE,
            type DWORD byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('CMGetPS2ColorRenderingIntent');

 dcl CMGetPS2ColorSpaceArray
     entry (type HPROFILE,
            type DWORD,
            type DWORD,
            type LPBYTE,
            type DWORD byaddr,
            type BOOL byaddr)
     returns (type BOOL byvalue optional)
     WINAPI external('CMGetPS2ColorSpaceArray');

 dcl CMIsProfileValid
     entry (type HPROFILE,             /* proflle handle */
            type BOOL byaddr)          /* buffer for result. */
     returns (type BOOL byvalue optional)
     WINAPI external('CMIsProfileValid');

 dcl CMTranslateColors
     entry (type HCMTRANSFORM,     /* transform handle */
            (*) type COLOR byaddr, /* pointer to input color array */
            type DWORD,            /* number of colors in color array */
            type COLORTYPE,        /* input color type */
            (*) type COLOR byaddr, /* pointer to output color array */
            type COLORTYPE)        /* output color type */
     returns (type BOOL byvalue optional)
     WINAPI external('CMTranslateColors');

 dcl CMTranslateRGB
     entry (type HCMTRANSFORM,
            type COLORREF,
            type COLORREF byaddr,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('CMTranslateRGB');

 dcl CMTranslateRGBs
     entry (type HCMTRANSFORM,
            type LPVOID,
            type BMFORMAT,
            type DWORD,
            type DWORD,
            type DWORD,
            type LPVOID,
            type BMFORMAT,
            type DWORD)
     returns (type BOOL byvalue optional)
     WINAPI external('CMTranslateRGBs');

 dcl CMTranslateRGBsExt
     entry (type HCMTRANSFORM,
            type LPVOID,
            type BMFORMAT,
            type DWORD,
            type DWORD,
            type DWORD,
            type LPVOID,
            type BMFORMAT,
            type DWORD,
            type LPBMCALLBACKFN optional,
            type LPARAM)
     returns (type BOOL byvalue optional)
     WINAPI external('CMTranslateRGBsExt');

 %end; /* ifndef _ICM_H_ */

