 /*++ BUILD Version: 0002                                             */
 /*   -- Increment this if a change has global effects                */
 /*                                                                   */
 /* Copyright 1991-1998 Microsoft Corporation                         */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   lmerrlog.cpy                                                    */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   This module defines the API function prototypes and data        */
 /*   structures for the following groups of NT API functions:        */
 /*       NetErrorLog                                                 */
 /*                                                                   */
 /* Environment:                                                      */
 /*                                                                   */
 /*   User Mode - Win32                                               */
 /*                                                                   */
 /* Notes:                                                            */
 /*                                                                   */
 /*   You must include NETCONS.H before this file, since this file    */
 /*   depends on values defined in NETCONS.H.                         */
 /*--                                                                 */

 %dcl #LMERRLOG_ char ext;
 %if #LMERRLOG_ = '' %then %do;
 %#LMERRLOG_ = 'Y';

 /* Data Structures - Config */

 define structure
   1 ERROR_LOG,
     2 el_len       type DWORD,
     2 el_reserved  type DWORD,
     2 el_time      type DWORD,
     2 el_error     type DWORD,
     2 el_name      type LPWSTR,/* pointer to service name */
     2 el_text      type LPWSTR,/* pointer to string array */
     2 el_data      type LPBYTE,/* pointer to BYTE array */
     2 el_data_size type DWORD,/* byte count of el_data area */
     2 el_nstrings  type DWORD;/* number of strings in el_text. */

 define alias  PERROR_LOG handle ERROR_LOG;
 define alias LPERROR_LOG handle ERROR_LOG;


 /* Temporary to let users %if on this struct layout. */
 %dcl REVISED_ERROR_LOG_STRUCT char ext;
 %REVISED_ERROR_LOG_STRUCT = 'Y';


 %dcl #LMHLOGDEFINED_ char ext;
 %if #LMHLOGDEFINED_ = '' %then %do;
 %#LMHLOGDEFINED_ = 'Y';

 /* Warning: Microsoft call this HLOG,         */
 /* but this conflicts with HLOG in pdh.cpy.   */
 /* Accordingly this is renamed to LMHLOG here */

 define structure
   1 LMHLOG,
     2 time       type DWORD,
     2 last_flags type DWORD,
     2 offset     type DWORD,
     2 rec_offset type DWORD;

 define alias  PLMHLOG handle LMHLOG;
 define alias LPLMHLOG handle LMHLOG;

 dcl LOGFLAGS_FORWARD  fixed bin(31) value(0);
 dcl LOGFLAGS_BACKWARD fixed bin(15) value('1'xn);
 dcl LOGFLAGS_SEEK     fixed bin(15) value('2'xn);

 %end;

 /* Function Prototypes - ErrorLog */

 dcl NetErrorLogClear /* onbsolete */
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr nonasgn optional,
            type LPBYTE)
     returns (type NET_API_STATUS byvalue optional)
     NET_API_FUNCTION external('NetErrorLogClear');

 dcl NetErrorLogRead
     entry (WCHAR (*) varz byaddr nonasgn optional,
            WCHAR (*) varz byaddr,
            type LMHLOG byaddr,
            type DWORD,
            type DWORD byaddr,
            type DWORD,
            type DWORD,
            type LPBYTE byaddr,
            type DWORD,
            type DWORD byaddr,
            type DWORD byaddr)
     returns (type NET_API_STATUS byvalue optional)
     NET_API_FUNCTION external('NetErrorLogRead');

 dcl NetErrorLogWrite
     entry (type LPBYTE,
            type DWORD,
            WCHAR (*) varz byaddr nonasgn,
            type LPBYTE,
            type DWORD,
            type LPBYTE,
            type DWORD,
            type LPBYTE)
     returns (type NET_API_STATUS byvalue optional)
     NET_API_FUNCTION external('NetErrorLogWrite');

 /* Special Values and Constants */


 /* Generic (could be used by more than one service) */
 /* error log messages from 0 to 25                  */

 dcl ERRLOG_BASE fixed bin(31) value(3100);
     /* NELOG errors start here */

 dcl NELOG_Internal_Error         fixed bin(31) value(ERRLOG_BASE + 0);
     /* The operation failed because a network software error */
     /* occurred.                                             */

 dcl NELOG_Resource_Shortage      fixed bin(31) value(ERRLOG_BASE + 1);
     /* The system ran out of a resource controlled by the %1 option. */

 dcl NELOG_Unable_To_Lock_Segment fixed bin(31) value(ERRLOG_BASE + 2);
     /* The service failed to obtain a long-term lock on the */
     /* segment for network control blocks (NCBs).           */
     /* The error code is the data.                          */

 dcl NELOG_Unable_To_Unlock_Segment fixed bin(31) value(ERRLOG_BASE +3);
     /* The service failed to release the long-term lock on the */
     /*  segment for network control blocks (NCBs).             */
     /* The error code is the data.                          */

 dcl NELOG_Uninstall_Service      fixed bin(31) value(ERRLOG_BASE + 4);
     /* There was an error stopping service %1.             */
     /*  The error code from NetServiceControl is the data. */

 dcl NELOG_Init_Exec_Fail         fixed bin(31) value(ERRLOG_BASE + 5);
     /* Initialization failed because of a system execution failure */
     /* on path %1. The system error code is the data.              */

 dcl NELOG_Ncb_Error              fixed bin(31) value(ERRLOG_BASE + 6);
     /* An unexpected network control block (NCB) was received. */
     /* The NCB is the data.                                    */

 dcl NELOG_Net_Not_Started        fixed bin(31) value(ERRLOG_BASE + 7);
     /* The network is not started. */

 dcl NELOG_Ioctl_Error            fixed bin(31) value(ERRLOG_BASE + 8);
     /* A DosDevIoctl or DosFsCtl to NETWKSTA.SYS failed. */
     /* The data shown is in this format:                 */
     /*     DWORD  approx CS:IP of call to ioctl or fsctl */
     /*     WORD   error code                             */
     /*     WORD   ioctl or fsctl number                  */

 dcl NELOG_System_Semaphore       fixed bin(31) value(ERRLOG_BASE + 9);
     /* Unable to create or open system semaphore %1. */
     /*  The error code is the data.                  */

 dcl NELOG_Init_OpenCreate_Err    fixed bin(31) value(ERRLOG_BASE + 10);
     /* Initialization failed because of an open/create error on the */
     /*  file %1. The system error code is the data.                 */

 dcl NELOG_NetBios                fixed bin(31) value(ERRLOG_BASE + 11);

     /* An unexpected NetBIOS error occurred. */
     /*  The error code is the data.          */

 dcl NELOG_SMB_Illegal            fixed bin(31) value(ERRLOG_BASE + 12);
     /* An illegal server message block (SMB) was received. */
     /*  The SMB is the data.                               */

 dcl NELOG_Service_Fail           fixed bin(31) value(ERRLOG_BASE + 13);
     /* Initialization failed because the requested service %1 */
     /*  could not be started.                                 */

 dcl NELOG_Entries_Lost           fixed bin(31) value(ERRLOG_BASE + 14);
     /* Some entries in the error log were lost because of a buffer */
     /* overflow.                                                   */


 /* Server specific error log messages from 20 to 40 */

 dcl NELOG_Init_Seg_Overflow      fixed bin(31) value(ERRLOG_BASE + 20);
     /* Initialization parameters controlling resource usage other    */
     /* than net buffers are sized so that too much memory is needed. */

 dcl NELOG_Srv_No_Mem_Grow        fixed bin(31) value(ERRLOG_BASE + 21);
     /* The server cannot increase the size of a memory segment. */

 dcl NELOG_Access_File_Bad        fixed bin(31) value(ERRLOG_BASE + 22);
     /* Initialization failed because account file %1 is either */
     /* incorrect or not present.                               */

 dcl NELOG_Srvnet_Not_Started     fixed bin(31) value(ERRLOG_BASE + 23);
     /* Initialization failed because network %1 was not started. */

 dcl NELOG_Init_Chardev_Err       fixed bin(31) value(ERRLOG_BASE + 24);
     /* The server failed to start. Either all three chdev     */
     /*  parameters must be zero or all three must be nonzero. */

 dcl NELOG_Remote_API             fixed bin(31) value(ERRLOG_BASE + 25);
     /* A remote API request was halted due to the following */
     /* invalid description string: %1.                      */

 dcl NELOG_Ncb_TooManyErr         fixed bin(31) value(ERRLOG_BASE + 26);
     /* The network %1 ran out of network control blocks (NCBs). */
     /* You may need to increase NCBs for this network.          */
     /* The following information includes the number of NCBs    */
     /* submitted by the server when this error occurred:        */

 dcl NELOG_Mailslot_err           fixed bin(31) value(ERRLOG_BASE + 27);
     /* The server cannot create the %1 mailslot needed to send  */
     /* the ReleaseMemory alert message. The error received is:  */

 dcl NELOG_ReleaseMem_Alert       fixed bin(31) value(ERRLOG_BASE + 28);
     /* The server failed to register for the ReleaseMemory alert, */
     /* with recipient %1. The error code from NetAlertStart is    */
     /* the data.                                                  */

 dcl NELOG_AT_cannot_write        fixed bin(31) value(ERRLOG_BASE + 29);
     /* The server cannot update the AT schedule file. */
     /* The file is corrupted.                         */

 dcl NELOG_Cant_Make_Msg_File     fixed bin(31) value(ERRLOG_BASE + 30);
     /* The server encountered an error when calling    */
     /* NetIMakeLMFileName. The error code is the data. */

 dcl NELOG_Exec_Netservr_NoMem    fixed bin(31) value(ERRLOG_BASE + 31);
     /* Initialization failed because of a system execution failure  */
     /* on path %1. There is not enough memory to start the process. */
     /* The system error code is the data.                           */

 dcl NELOG_Server_Lock_Failure    fixed bin(31) value(ERRLOG_BASE + 32);
     /* Longterm lock of the server buffers failed. Check swap disk's */
     /* free space and restart the system to start the server.        */


 /* Message service and POPUP specific error log messages */
 /* from 40 to 55                                         */

 dcl NELOG_Msg_Shutdown           fixed bin(31) value(ERRLOG_BASE + 40);
     /* The service has stopped due to repeated consecutive */
     /* occurrences of a network control block (NCB) error. */
     /* The last bad NCB follows in raw data.               */

 dcl NELOG_Msg_Sem_Shutdown       fixed bin(31) value(ERRLOG_BASE + 41);
     /* The Message server has stopped due to a lock on the */
     /* Message server shared data segment.                 */

 dcl NELOG_Msg_Log_Err            fixed bin(31) value(ERRLOG_BASE + 50);
     /* A file system error occurred while opening or writing to the */
     /* system message log file %1. Message logging has been         */
     /* switched off due to the error. The error code is the data.   */


 dcl NELOG_VIO_POPUP_ERR          fixed bin(31) value(ERRLOG_BASE + 51);
     /* Unable to display message POPUP due to system VIO call error. */
     /* The error code is the data.                                   */

 dcl NELOG_Msg_Unexpected_SMB_Type fixed bin(31) value(ERRLOG_BASE +52);
     /* An illegal server message block (SMB) was received. */
     /*  The SMB is the data.                               */


 /* Workstation specific error log messages from 60 to 75 */

 dcl NELOG_Wksta_Infoseg          fixed bin(31) value(ERRLOG_BASE + 60);
     /* The workstation information segment is bigger than 64K. */
     /*  The size follows, in DWORD format:                     */

 dcl NELOG_Wksta_Compname         fixed bin(31) value(ERRLOG_BASE + 61);
     /* The workstation was unable to get the name-number of the */
     /* computer.                                                */

 dcl NELOG_Wksta_BiosThreadFailure fixed bin(31) value(ERRLOG_BASE +62);
     /* The workstation could not initialize the Async NetBIOS */
     /* Thread. The error code is the data.                    */

 dcl NELOG_Wksta_IniSeg           fixed bin(31) value(ERRLOG_BASE + 63);
     /* The workstation could not open the initial shared segment. */
     /* The error code is the data.                                */

 dcl NELOG_Wksta_HostTab_Full     fixed bin(31) value(ERRLOG_BASE + 64);
 /*
      * The workstation host table is full.
      */

 dcl NELOG_Wksta_Bad_Mailslot_SMB fixed bin(31) value(ERRLOG_BASE + 65);
     /* A bad mailslot server message block (SMB) was received. */
     /* The SMB is the data.                                    */

 dcl NELOG_Wksta_UASInit          fixed bin(31) value(ERRLOG_BASE + 66);
     /* The workstation encountered an error while trying to start */
     /* the user accounts database. The error code is the data.    */

 dcl NELOG_Wksta_SSIRelogon       fixed bin(31) value(ERRLOG_BASE + 67);
     /* The workstation encountered an error while responding to an */
     /* SSI revalidation request.                                   */
     /* The function code and the error codes are the data.         */


 /* Alerter service specific error log messages from 70 to 79 */

 dcl NELOG_Build_Name             fixed bin(31) value(ERRLOG_BASE + 70);
     /* The Alerter service had a problem creating the list of */
     /* alert recipients. The error code is %1.                */

 dcl NELOG_Name_Expansion         fixed bin(31) value(ERRLOG_BASE + 71);
     /* There was an error expanding %1 as a group name. Try  */
     /*  splitting the group into two or more smaller groups. */

 dcl NELOG_Message_Send           fixed bin(31) value(ERRLOG_BASE + 72);
     /* There was an error sending %2 the alert message - (%3) */
     /* The error code is %1.                                  */

 dcl NELOG_Mail_Slt_Err           fixed bin(31) value(ERRLOG_BASE + 73);
     /* There was an error in creating or reading the alerter */
     /* mailslot.                                             */
     /* The error code is %1.                                 */

 dcl NELOG_AT_cannot_read         fixed bin(31) value(ERRLOG_BASE + 74);
     /* The server could not read the AT schedule file. */

 dcl NELOG_AT_sched_err           fixed bin(31) value(ERRLOG_BASE + 75);
     /* The server found an invalid AT schedule record. */

 dcl NELOG_AT_schedule_file_created
                                  fixed bin(31) value(ERRLOG_BASE + 76);
     /* The server could not find an AT schedule file */
     /* so it created one.                            */

 dcl NELOG_Srvnet_NB_Open         fixed bin(31) value(ERRLOG_BASE + 77);
     /* The server could not access the %1 network with NetBiosOpen. */

 dcl NELOG_AT_Exec_Err            fixed bin(31) value(ERRLOG_BASE + 78);
     /* The AT command processor could not run %1. */


 /* Cache Lazy Write and HPFS386 specific error log messages */
 /* from 80 to 89                                            */

 dcl NELOG_Lazy_Write_Err         fixed bin(31) value(ERRLOG_BASE + 80);
     /* WARNING:  Because of a lazy-write error, drive %1 now */
     /* contains some corrupted data.  The cache is stopped.  */

 dcl NELOG_HotFix                 fixed bin(31) value(ERRLOG_BASE + 81);
     /* A defective sector on drive %1 has been replaced (hotfixed). */
     /* No data was lost. You should run CHKDSK soon to restore full */
     /* performance and replenish the volume's spare sector pool.    */
     /* The hotfix occurred while processing a remote request.       */

 dcl NELOG_HardErr_From_Server    fixed bin(31) value(ERRLOG_BASE + 82);
     /* A disk error occurred on the HPFS volume in drive %1. */
     /* The error occurred while processing a remote request. */

 dcl NELOG_LocalSecFail1          fixed bin(31) value(ERRLOG_BASE + 83);
     /* The user accounts database (NET.ACC) is corrupted.           */
     /* The local security system is replacing the corrupted NET.ACC */
     /* with the backup made at %1.                                  */
     /* Any updates made to the database after this time are lost.   */

 dcl NELOG_LocalSecFail2          fixed bin(31) value(ERRLOG_BASE + 84);
     /* The user accounts database (NET.ACC) is missing. The local   */
     /* security system is restoring the backup database made at %1. */
     /* Any updates made to the database made after this time are    */
     /* lost.                                                        */

 dcl NELOG_LocalSecFail3          fixed bin(31) value(ERRLOG_BASE + 85);
     /* Local security could not be started because the user accounts */
     /* database (NET.ACC) was missing or corrupted, and no usable    */
     /* backup database was present.                                  */
     /* THE SYSTEM IS NOT SECURE.                                     */

 dcl NELOG_LocalSecGeneralFail    fixed bin(31) value(ERRLOG_BASE + 86);
     /* Local security could not be started because an error          */
     /* occurred during initialization. The error code returned is %1.*/
     /* THE SYSTEM IS NOT SECURE.                                     */


 /* NETWKSTA.SYS specific error log messages from 90 to 99 */

 dcl NELOG_NetWkSta_Internal_Error fixed bin(31) value(ERRLOG_BASE +90);
     /* A NetWksta internal error has occurred: %1 */

 dcl NELOG_NetWkSta_No_Resource   fixed bin(31) value(ERRLOG_BASE + 91);
     /* The redirector is out of a resource: %1. */

 dcl NELOG_NetWkSta_SMB_Err       fixed bin(31) value(ERRLOG_BASE + 92);
     /* A server message block (SMB) error occurred on the connection */
     /* to %1. The SMB header is the data.                            */

 dcl NELOG_NetWkSta_VC_Err        fixed bin(31) value(ERRLOG_BASE + 93);
     /* A virtual circuit error occurred on the session to %1.  */
     /* The network control block (NCB) command and return code */
     /* is the data.                                            */

 dcl NELOG_NetWkSta_Stuck_VC_Err  fixed bin(31) value(ERRLOG_BASE + 94);
     /* Hanging up a stuck session to %1. */

 dcl NELOG_NetWkSta_NCB_Err       fixed bin(31) value(ERRLOG_BASE + 95);
     /* A network control block (NCB) error occurred (%1). */
     /* The NCB is the data.                               */

 dcl NELOG_NetWkSta_Write_Behind_Err
                                  fixed bin(31) value(ERRLOG_BASE + 96);
     /* A write operation to %1 failed. Data may have been lost. */

 dcl NELOG_NetWkSta_Reset_Err     fixed bin(31) value(ERRLOG_BASE + 97);
     /* Reset of driver %1 failed to complete the network control */
     /* block (NCB). The NCB is the data.                         */

 dcl NELOG_NetWkSta_Too_Many      fixed bin(31) value(ERRLOG_BASE + 98);
     /* The amount of resource %1 requested was more         */
     /*  than the maximum. The maximum amount was allocated. */


 /*  Spooler specific error log messages from 100 to 103 */

 dcl NELOG_Srv_Thread_Failure    fixed bin(31) value(ERRLOG_BASE + 104);
     /* The server could not create a thread. The THREADS parameter */
     /* in the CONFIG.SYS file should be increased.                 */

 dcl NELOG_Srv_Close_Failure     fixed bin(31) value(ERRLOG_BASE + 105);
     /* The server could not close %1. The file is probably corrupted.*/

 dcl NELOG_ReplUserCurDir        fixed bin(31) value(ERRLOG_BASE + 106);
     /* The replicator cannot update directory %1. It has tree   */
     /* integrity and is the current directory for some process. */

 dcl NELOG_ReplCannotMasterDir   fixed bin(31) value(ERRLOG_BASE + 107);
     /* The server cannot export directory %1 to client %2. */
     /* It is exported from another server.                 */

 dcl NELOG_ReplUpdateError       fixed bin(31) value(ERRLOG_BASE + 108);
     /* The replication server could not update directory %2 from */
     /* the source on %3 due to error %1.                         */

 dcl NELOG_ReplLostMaster        fixed bin(31) value(ERRLOG_BASE + 109);
     /* Master %1 did not send an update notice for directory %2 */
     /* at the expected time.                                    */

 dcl NELOG_NetlogonAuthDCFail    fixed bin(31) value(ERRLOG_BASE + 110);
     /* Failed to authenticate with %2, a Windows NT domain */
     /* controller for domain %1.                           */

 dcl NELOG_ReplLogonFailed       fixed bin(31) value(ERRLOG_BASE + 111);
     /* The replicator attempted to log on at %2 as %1 and failed. */

 dcl NELOG_ReplNetErr            fixed bin(31) value(ERRLOG_BASE + 112);
     /* Network error %1 occurred. */

 dcl NELOG_ReplMaxFiles          fixed bin(31) value(ERRLOG_BASE + 113);
     /* Replicator limit for files in a directory has been exceeded. */


 dcl NELOG_ReplMaxTreeDepth      fixed bin(31) value(ERRLOG_BASE + 114);
     /* Replicator limit for tree depth has been exceeded. */

 dcl NELOG_ReplBadMsg            fixed bin(31) value(ERRLOG_BASE + 115);
     /* Unrecognized message received in mailslot. */

 dcl NELOG_ReplSysErr            fixed bin(31) value(ERRLOG_BASE + 116);
     /* System error %1 occurred. */

 dcl NELOG_ReplUserLoged         fixed bin(31) value(ERRLOG_BASE + 117);
     /* Cannot log on. User is currently logged on and argument */
     /* TRYUSER is set to NO.                                   */

 dcl NELOG_ReplBadImport         fixed bin(31) value(ERRLOG_BASE + 118);
     /* IMPORT path %1 cannot be found. */

 dcl NELOG_ReplBadExport         fixed bin(31) value(ERRLOG_BASE + 119);
     /* EXPORT path %1 cannot be found. */

 dcl NELOG_ReplSignalFileErr     fixed bin(31) value(ERRLOG_BASE + 120);
     /* Replicator failed to update signal file in directory %2 due */
     /* to %1 system error.                                         */

 dcl NELOG_DiskFT                fixed bin(31) value(ERRLOG_BASE+121);
     /* Disk Fault Tolerance Error %1 */

 dcl NELOG_ReplAccessDenied      fixed bin(31) value(ERRLOG_BASE + 122);
     /* Replicator could not access %2 on %3 due to system error %1. */

 dcl NELOG_NetlogonFailedPrimary fixed bin(31) value(ERRLOG_BASE + 123);
     /* The primary domain controller for domain %1 has apparently */
     /* failed.                                                    */

 dcl NELOG_NetlogonPasswdSetFailed
                                 fixed bin(31) value(ERRLOG_BASE + 124);
     /* Changing machine account password for account %1 failed with */
     /* the following error: %n%2                                    */

 dcl NELOG_NetlogonTrackingError fixed bin(31) value(ERRLOG_BASE + 125);
     /* An error occurred while updating the logon or logoff */
     /* information for %1.                                  */

 dcl NELOG_NetlogonSyncError     fixed bin(31) value(ERRLOG_BASE + 126);
     /* An error occurred while synchronizing with primary domain */
     /* controller %1                                             */


 /* UPS service specific error log messages from 130 to 136 */

 dcl NELOG_UPS_PowerOut          fixed bin(31) value(ERRLOG_BASE + 130);
     /* A power failure was detected at the server. */

 dcl NELOG_UPS_Shutdown          fixed bin(31) value(ERRLOG_BASE + 131);
     /* The UPS service performed server shut down. */

 dcl NELOG_UPS_CmdFileError      fixed bin(31) value(ERRLOG_BASE + 132);
     /* The UPS service did not complete execution of the */
     /* user specified shut down command file.            */

 dcl NELOG_UPS_CannotOpenDriver  fixed bin(31) value(ERRLOG_BASE + 133);
     /* The UPS driver could not be opened. The error code is */
     /* the data.                                             */

 dcl NELOG_UPS_PowerBack         fixed bin(31) value(ERRLOG_BASE + 134);
     /* Power has been restored. */

 dcl NELOG_UPS_CmdFileConfig     fixed bin(31) value(ERRLOG_BASE + 135);
     /* There is a problem with a configuration of user specified */
     /* shut down command file.                                   */

 dcl NELOG_UPS_CmdFileExec       fixed bin(31) value(ERRLOG_BASE + 136);
     /* The UPS service failed to execute a user specified shutdown */
     /* command file %1. The error code is the data.                */


 /* Remoteboot server specific error log messages are from 150 to 159 */

 dcl NELOG_Missing_Parameter     fixed bin(31) value(ERRLOG_BASE + 150);
     /* Initialization failed because of an invalid or missing */
     /* parameter in the configuration file %1.                */

 dcl NELOG_Invalid_Config_Line   fixed bin(31) value(ERRLOG_BASE + 151);
     /* Initialization failed because of an invalid line in the */
     /*  configuration file %1. The invalid line is the data.   */

 dcl NELOG_Invalid_Config_File   fixed bin(31) value(ERRLOG_BASE + 152);
     /* Initialization failed because of an error in the */
     /* configuration file %1.                           */

 dcl NELOG_File_Changed          fixed bin(31) value(ERRLOG_BASE + 153);
     /* The file %1 has been changed after initialization. */
     /* The boot-block loading was temporarily terminated. */

 dcl NELOG_Files_Dont_Fit        fixed bin(31) value(ERRLOG_BASE + 154);
     /* The files do not fit to the boot-block configuration file %1. */
     /* Change the BASE and ORG definitions or the order of the files.*/

 dcl NELOG_Wrong_DLL_Version     fixed bin(31) value(ERRLOG_BASE + 155);
     /* Initialization failed because the dynamic-link library %1 */
     /* returned an incorrect version number.                     */

 dcl NELOG_Error_in_DLL          fixed bin(31) value(ERRLOG_BASE + 156);
     /* There was an unrecoverable error in the dynamic-Link library */
     /* of the service.                                              */

 dcl NELOG_System_Error          fixed bin(31) value(ERRLOG_BASE + 157);
     /* The system returned an unexpected error code.*/
     /* The error code is the data.                  */

 dcl NELOG_FT_ErrLog_Too_Large   fixed bin(31) value(ERRLOG_BASE + 158);
     /* The fault-tolerance error log file, LANROOT\LOGS\FT.LOG, */
     /* is more than 64K.                                        */

 dcl NELOG_FT_Update_In_Progress fixed bin(31) value(ERRLOG_BASE + 159);
     /* The fault-tolerance error-log file, LANROOT\LOGS\FT.LOG, had  */
     /* the update in progress bit set upon opening, which means that */
     /* the system crashed while working on the error log.            */


 /* Microsoft has created a generic error log entry for OEMs to use to*/
 /* log errors from OEM value added services.  The code, which is the */
 /* 2nd arg to NetErrorLogWrite, is 3299.  This value is manifest in  */
 /* NET/H/ERRLOG.H as NELOG_OEM_Code.  The text for error log entry   */
 /* NELOG_OEM_Code is:  "%1 %2 %3 %4 %5 %6 %7 %8 %9.".                */
 /*                                                                   */
 /* Microsoft suggests that OEMs use the insertion strings as follows:*/
 /* %1:  OEM System Name  (e.g. 3+Open)                               */
 /* %2:  OEM Service Name (e.g. 3+Mail)                               */
 /* %3:  Severity level (e.g.  error, warning, etc.)                  */
 /* %4:  OEM error log entry sub-identifier  (e.g. error code #)      */
 /* %5 - % 9:  Text.                                                  */
 /*                                                                   */
 /* The call to NetErrorWrite must set nstrings = 9, and provide 9    */
 /* ASCIIZ strings.  If the caller does not have 9 insertion strings, */
 /* provide null strings for the empty insertion strings.             */

 dcl NELOG_OEM_Code fixed bin(31) value(ERRLOG_BASE + 199);
     /* %1 %2 %3 %4 %5 %6 %7 %8 %9. */


 /* another error log range defined for NT Lanman. */

 dcl ERRLOG2_BASE fixed bin(31) value(5700);
     /* New NT NELOG errors start here */

 dcl NELOG_NetlogonSSIInitError   fixed bin(31) value(ERRLOG2_BASE + 0);
     /* The Netlogon service could not initialize the replication */
     /* data structures successfully. The service was terminated. */
     /* The following error occurred: %n%1                        */

 dcl NELOG_NetlogonFailedToUpdateTrustList
                                  fixed bin(31) value(ERRLOG2_BASE + 1);
     /* The Netlogon service failed to update the domain trust list. */
     /* The following error occurred: %n%1                           */

 dcl NELOG_NetlogonFailedToAddRpcInterface
                                  fixed bin(31) value(ERRLOG2_BASE + 2);
     /* The Netlogon service could not add the RPC interface. The  */
     /* service was terminated. The following error occurred: %n%1 */

 dcl NELOG_NetlogonFailedToReadMailslot
                                  fixed bin(31) value(ERRLOG2_BASE + 3);
     /* The Netlogon service could not read a mailslot message from */
     /* %1 due to the following error: %n%2                         */

 dcl NELOG_NetlogonFailedToRegisterSC
                                  fixed bin(31) value(ERRLOG2_BASE + 4);
     /* The Netlogon service failed to register the service with */
     /* the service controller. The service was terminated.      */
     /* The following error occurred: %n%1                       */

 dcl NELOG_NetlogonChangeLogCorrupt
                                  fixed bin(31) value(ERRLOG2_BASE + 5);
     /* The change log cache maintained by the Netlogon service */
     /* for database changes is corrupted. The Netlogon service */
     /* is resetting the change log.                            */

 dcl NELOG_NetlogonFailedToCreateShare
                                  fixed bin(31) value(ERRLOG2_BASE + 6);
     /* The Netlogon service could not create server share %1. */
     /* The following error occurred: %n%2                     */

 dcl NELOG_NetlogonDownLevelLogonFailed
                                  fixed bin(31) value(ERRLOG2_BASE + 7);
     /* The down-level logon request for the user %1 from %2 failed. */

 dcl NELOG_NetlogonDownLevelLogoffFailed
                                  fixed bin(31) value(ERRLOG2_BASE + 8);
     /* The down-level logoff request for the user %1 from %2 failed. */

 dcl NELOG_NetlogonNTLogonFailed  fixed bin(31) value(ERRLOG2_BASE + 9);
     /* The Windows NT %1 logon request for the user %2\%3 from %4 */
     /* (via %5) failed.                                           */

 dcl NELOG_NetlogonNTLogoffFailed
                                 fixed bin(31) value(ERRLOG2_BASE + 10);
     /* The Windows NT %1 logoff request for the user %2\%3 from %4 */
     /* failed.                                                     */

 dcl NELOG_NetlogonPartialSyncCallSuccess
                                 fixed bin(31) value(ERRLOG2_BASE + 11);
     /* The partial synchronization request from the server %1 */
     /* completed successfully. %2 changes(s) has(have) been   */
     /* returned to the caller.                                */

 dcl NELOG_NetlogonPartialSyncCallFailed
                                 fixed bin(31) value(ERRLOG2_BASE + 12);
     /* The partial synchronization request from the server %1 */
     /* failed with the following error: %n%2                  */

 dcl NELOG_NetlogonFullSyncCallSuccess
                                 fixed bin(31) value(ERRLOG2_BASE + 13);
     /* The full synchronization request from the server %1 completed */
     /* successfully. %2 object(s) has(have) been returned to         */
     /* the caller.                                                   */

 dcl NELOG_NetlogonFullSyncCallFailed
                                 fixed bin(31) value(ERRLOG2_BASE + 14);
     /* The full synchronization request from the server %1 failed */
     /* with the following error: %n%2                             */

 dcl NELOG_NetlogonPartialSyncSuccess
                                 fixed bin(31) value(ERRLOG2_BASE + 15);
     /* The partial synchronization replication of the %1 database    */
     /* from the primary domain controller %2 completed successfully. */
     /* %3 change(s) is(are) applied to the database.                 */


 dcl NELOG_NetlogonPartialSyncFailed
                                 fixed bin(31) value(ERRLOG2_BASE + 16);
     /* The partial synchronization replication of the %1 database */
     /* from the primary domain controller %2 failed with the      */
     /* following error: %n%3                                      */

 dcl NELOG_NetlogonFullSyncSuccess
                                 fixed bin(31) value(ERRLOG2_BASE + 17);
     /* The full synchronization replication of the %1 database from */
     /* the  primary domain controller %2 completed successfully.    */


 dcl NELOG_NetlogonFullSyncFailed
                                 fixed bin(31) value(ERRLOG2_BASE + 18);
     /* The full synchronization replication of the %1 database from */
     /* the primary domain controller %2 failed with the following   */
     /* error: %n%3                                                  */

 dcl NELOG_NetlogonAuthNoDomainController
                                 fixed bin(31) value(ERRLOG2_BASE + 19);
     /* No Windows NT Domain Controller is available for domain %1.  */
     /* (This event is expected and can be ignored when booting with */
     /* the 'No Net' Hardware Profile.)  The following error         */
     /* occurred: %n%2                                               */

 dcl NELOG_NetlogonAuthNoTrustLsaSecret
                                 fixed bin(31) value(ERRLOG2_BASE + 20);
     /* The session setup to the Windows NT Domain Controller %1 for */
     /* the domain %2 failed because the computer %3 does not have a */
     /* local security database account.                             */

 dcl NELOG_NetlogonAuthNoTrustSamAccount
                                 fixed bin(31) value(ERRLOG2_BASE + 21);
     /* The session setup to the Windows NT Domain Controller %1 for  */
     /* the domain %2 failed because the Windows NT Domain Controller */
     /* does not have an account for the computer %3.                 */

 dcl NELOG_NetlogonServerAuthFailed
                                 fixed bin(31) value(ERRLOG2_BASE + 22);
     /* The session setup from the computer %1 failed to authenticate.*/
     /* The name of the account referenced in the security database   */
     /* is %2.  The following error occurred: %n%3                    */

 dcl NELOG_NetlogonServerAuthNoTrustSamAccount
                                 fixed bin(31) value(ERRLOG2_BASE + 23);
     /* The session setup from the computer %1 failed because there is*/
     /* no trust account in the security database for this computer.  */
     /* The name of the account referenced in the security database   */
     /* is %2.                                                        */


 /* General log messages for NT services. */

 dcl NELOG_FailedToRegisterSC    fixed bin(31) value(ERRLOG2_BASE + 24);
     /* Could not register control handler with service controller %1.*/

 dcl NELOG_FailedToSetServiceStatus
                                 fixed bin(31) value(ERRLOG2_BASE + 25);
     /* Could not set service status with service controller %1. */

 dcl NELOG_FailedToGetComputerName
                                 fixed bin(31) value(ERRLOG2_BASE + 26);
     /* Could not find the computer name %1. */

 dcl NELOG_DriverNotLoaded       fixed bin(31) value(ERRLOG2_BASE + 27);
     /* Could not load %1 device driver. */

 dcl NELOG_NoTranportLoaded      fixed bin(31) value(ERRLOG2_BASE + 28);
     /* Could not load any transport. */


 /* More Netlogon service events */

 dcl NELOG_NetlogonFailedDomainDelta
                                 fixed bin(31) value(ERRLOG2_BASE + 29);
     /* Replication of the %1 Domain Object "%2" from primary domain */
     /* controller %3 failed with the following error: %n%4          */

 dcl NELOG_NetlogonFailedGlobalGroupDelta
                                 fixed bin(31) value(ERRLOG2_BASE + 30);
     /* Replication of the %1 Global Group "%2" from primary domain */
     /* controller %3 failed with the following error: %n%4         */

 dcl NELOG_NetlogonFailedLocalGroupDelta
                                 fixed bin(31) value(ERRLOG2_BASE + 31);
     /* Replication of the %1 Local Group "%2" from primary domain */
     /* controller %3 failed with the following error: %n%4        */

 dcl NELOG_NetlogonFailedUserDelta
                                 fixed bin(31) value(ERRLOG2_BASE + 32);
     /* Replication of the %1 User "%2" from primary domain */
     /* controller %3 failed with the following error: %n%4 */

 dcl NELOG_NetlogonFailedPolicyDelta
                                 fixed bin(31) value(ERRLOG2_BASE + 33);
     /* Replication of the %1 Policy Object "%2" from primary domain */
     /* controller %3 failed with the following error: %n%4          */

 dcl NELOG_NetlogonFailedTrustedDomainDelta
                                 fixed bin(31) value(ERRLOG2_BASE + 34);
     /* Replication of the %1 Trusted Domain Object "%2" from primary */
     /* domain controller %3 failed with the following error: %n%4    */

 dcl NELOG_NetlogonFailedAccountDelta
                                 fixed bin(31) value(ERRLOG2_BASE + 35);
     /* Replication of the %1 Account Object "%2" from primary domain */
     /* controller %3 failed with the following error: %n%4           */

 dcl NELOG_NetlogonFailedSecretDelta
                                 fixed bin(31) value(ERRLOG2_BASE + 36);
     /* Replication of the %1 Secret "%2" from primary domain */
     /* controller %3 failed with the following error: %n%4   */

 dcl NELOG_NetlogonSystemError   fixed bin(31) value(ERRLOG2_BASE + 37);
     /* The system returned the following unexpected error code: %n%1 */

 dcl NELOG_NetlogonDuplicateMachineAccounts
                                 fixed bin(31) value(ERRLOG2_BASE + 38);
     /* Netlogon has detected two machine accounts for server "%1".   */
     /* The server can be either a Windows NT Server that is a member */
     /* of the domain or the server can be a LAN Manager server with  */
     /* an account in the SERVERS global group. It cannot be both.    */

 dcl NELOG_NetlogonTooManyGlobalGroups
                                 fixed bin(31) value(ERRLOG2_BASE + 39);
     /* This domain has more global groups than can be replicated to */
     /* a LanMan BDC. Either delete some of your global groups or    */
     /* remove the LanMan BDCs from the domain.                      */

 dcl NELOG_NetlogonBrowserDriver fixed bin(31) value(ERRLOG2_BASE + 40);
     /* The Browser driver returned the following error to Netlogon: */
     /* %n%1                                                         */

 dcl NELOG_NetlogonAddNameFailure
                                 fixed bin(31) value(ERRLOG2_BASE + 41);
     /* Netlogon could not register the %1<1B> name for the */
     /* following reason: %n%2                              */


 /* More Remoteboot service events. */

 dcl NELOG_RplMessages           fixed bin(31) value(ERRLOG2_BASE + 42);
     /* Service failed to retrieve messages needed to boot remote */
     /* boot clients.                                             */

 dcl NELOG_RplXnsBoot            fixed bin(31) value(ERRLOG2_BASE + 43);
     /* Service experienced a severe error and can no longer provide */
     /* remote boot for 3Com 3Start remote boot clients.             */

 dcl NELOG_RplSystem             fixed bin(31) value(ERRLOG2_BASE + 44);
     /* Service experienced a severe system error and will shut */
     /* itself down.                                            */

 dcl NELOG_RplWkstaTimeout       fixed bin(31) value(ERRLOG2_BASE + 45);
     /* Client with computer name %1 failed to acknowledge receipt of */
     /* the boot data.  Remote boot of this client was not completed. */

 dcl NELOG_RplWkstaFileOpen      fixed bin(31) value(ERRLOG2_BASE + 46);
     /* Client with computer name %1 was not booted due to an error */
     /* in opening file %2.                                         */

 dcl NELOG_RplWkstaFileRead      fixed bin(31) value(ERRLOG2_BASE + 47);
     /* Client with computer name %1 was not booted due to an error */
     /* in reading file %2.                                         */

 dcl NELOG_RplWkstaMemory        fixed bin(31) value(ERRLOG2_BASE + 48);
     /* Client with computer name %1 was not booted due to */
     /* insufficent memory at the remote boot server.      */

 dcl NELOG_RplWkstaFileChecksum  fixed bin(31) value(ERRLOG2_BASE + 49);
     /* Client with computer name %1 will be booted without using */
     /* checksums because checksum for file %2 could not be       */
     /* calculated.                                               */

 dcl NELOG_RplWkstaFileLineCount fixed bin(31) value(ERRLOG2_BASE + 50);
     /* Client with computer name %1 was not booted due to too many */
     /* lines in file %2.                                           */

 dcl NELOG_RplWkstaBbcFile       fixed bin(31) value(ERRLOG2_BASE + 51);
     /* Client with computer name %1 was not booted because the boot */
     /* block configuration file %2 for this client does not contain */
     /* boot block line and/or loader line.                          */

 dcl NELOG_RplWkstaFileSize      fixed bin(31) value(ERRLOG2_BASE + 52);
     /* Client with computer name %1 was not booted due to a bad size */
     /* of file %2.                                                   */

 dcl NELOG_RplWkstaInternal      fixed bin(31) value(ERRLOG2_BASE + 53);
     /* Client with computer name %1 was not booted due to remote */
     /* boot service internal error.                              */

 dcl NELOG_RplWkstaWrongVersion  fixed bin(31) value(ERRLOG2_BASE + 54);
     /* Client with computer name %1 was not booted because file %2 */
     /* has an invalid boot header.                                 */

 dcl NELOG_RplWkstaNetwork       fixed bin(31) value(ERRLOG2_BASE + 55);
     /* Client with computer name %1 was not booted due to */
     /* network error.                                     */

 dcl NELOG_RplAdapterResource    fixed bin(31) value(ERRLOG2_BASE + 56);
     /* Client with adapter id %1 was not booted due to */
     /* lack of resources.                              */

 dcl NELOG_RplFileCopy           fixed bin(31) value(ERRLOG2_BASE + 57);
     /* Service experienced error copying file or directory %1. */

 dcl NELOG_RplFileDelete         fixed bin(31) value(ERRLOG2_BASE + 58);
     /* Service experienced error deleting file or directory %1. */

 dcl NELOG_RplFilePerms          fixed bin(31) value(ERRLOG2_BASE + 59);
     /* Service experienced error setting permissions on file */
     /* or directory %1.                                      */

 dcl NELOG_RplCheckConfigs       fixed bin(31) value(ERRLOG2_BASE + 60);
     /* Service experienced error evaluating RPL configurations. */

 dcl NELOG_RplCreateProfiles     fixed bin(31) value(ERRLOG2_BASE + 61);
     /* Service experienced error creating RPL profiles for all */
     /* configurations.                                         */

 dcl NELOG_RplRegistry           fixed bin(31) value(ERRLOG2_BASE + 62);
     /* Service experienced error accessing registry. */

 dcl NELOG_RplReplaceRPLDISK     fixed bin(31) value(ERRLOG2_BASE + 63);
     /* Service experienced error replacing possibly outdated */
     /* RPLDISK.SYS.                                          */

 dcl NELOG_RplCheckSecurity      fixed bin(31) value(ERRLOG2_BASE + 64);
     /* Service experienced error adding security accounts or setting */
     /* file permissions.  These accounts are the RPLUSER local group */
     /* and the user accounts for the individual RPL workstations.    */

 dcl NELOG_RplBackupDatabase     fixed bin(31) value(ERRLOG2_BASE + 65);
     /* Service failed to back up its database. */

 dcl NELOG_RplInitDatabase       fixed bin(31) value(ERRLOG2_BASE + 66);
     /* Service failed to initialize from its database.              */
     /* The database may be missing or corrupted.                    */
     /* Service will attempt restoring the database from the backup. */

 dcl NELOG_RplRestoreDatabaseFailure
                                 fixed bin(31) value(ERRLOG2_BASE + 67);
     /* Service failed to restore its database from the backup. */
     /* Service will not start.                                 */

 dcl NELOG_RplRestoreDatabaseSuccess
                                 fixed bin(31) value(ERRLOG2_BASE + 68);
     /* Service sucessfully restored its database from the backup. */

 dcl NELOG_RplInitRestoredDatabase
                                 fixed bin(31) value(ERRLOG2_BASE + 69);
     /* Service failed to initialize from its restored database. */
     /* Service will not start.                                  */


 /* More Netlogon and RPL service events */

 dcl NELOG_NetlogonSessionTypeWrong
                                 fixed bin(31) value(ERRLOG2_BASE + 70);
     /* The session setup to the Windows NT Domain Controller %1 from */
     /* computer %2 using account %4 failed. %2 is declared to be a   */
     /* BDC in domain %3. However, %2 tried to connect as either a DC */
     /* in a trusted domain, a member workstation in domain %3, or as */
     /* a server in domain %3.                                        */
     /* Use the Server Manager to remove the BDC account for %2.      */

 dcl NELOG_RplUpgradeDBTo40      fixed bin(31) value(ERRLOG2_BASE + 71);
     /* The Remoteboot database was in NT 3.5 / NT 3.51 format and NT */
     /* is attempting to convert it to NT 4.0 format. The JETCONV     */
     /* converter will write to the Application event log when it is  */
     /* finished.                                                     */

 dcl NELOG_NetlogonLanmanBdcsNotAllowed
                                 fixed bin(31) value(ERRLOG2_BASE + 72);
     /* Global group SERVERS exists in domain %1 and has members. */
     /* This group defines Lan Manager BDCs in the domain.        */
     /* Lan Manager BDCs are not permitted in NT domains.         */

 dcl NELOG_NetlogonNoDynamicDns  fixed bin(31) value(ERRLOG2_BASE + 73);
     /* The DNS server for this DC does not support dynamic DNS.      */
     /* Add the DNS records from the file                             */
     /* '%SystemRoot%\System32\Config\netlogon.dns' to the DNS server */
     /* serving the domain referenced in that file.                   */

 dcl NELOG_NetlogonDynamicDnsRegisterFailure
                                 fixed bin(31) value(ERRLOG2_BASE + 74);
     /* Registration of the DNS name %1 failed with the following */
     /* error: %n%2                                               */

 dcl NELOG_NetlogonDynamicDnsDeregisterFailure
                                 fixed bin(31) value(ERRLOG2_BASE + 75);
     /* Deregistration of the DNS name %1 failed with the following */
     /* error: %n%2                                                 */

 dcl NELOG_NetlogonFailedFileCreate
                                 fixed bin(31) value(ERRLOG2_BASE + 76);
     /* Failed to create/open file %1 with the following error: %n%2 */

 dcl NELOG_NetlogonGetSubnetToSite
                                 fixed bin(31) value(ERRLOG2_BASE + 77);
     /* Netlogon got the following error while trying to get the */
     /* subnet to site mapping information from the DS: %n%1     */

 dcl NELOG_NetlogonNoSiteForClient
                                 fixed bin(31) value(ERRLOG2_BASE + 78);
     /* '%1' tried to determine its site by looking up its IP address */
     /* ('%2') in the Configuration\Sites\Subnets container in the DS.*/
     /* No subnet matched the IP address. Consider adding a subnet    */
     /* object for this IP address.                                   */

 dcl NELOG_NetlogonBadSiteName   fixed bin(31) value(ERRLOG2_BASE + 79);
     /* The site name for this computer is '%1'. That site name is    */
     /* not a valid site name. A site name must be a valid DNS label. */
     /* Rename the site to be a valid name.                           */

 dcl NELOG_NetlogonBadSubnetName fixed bin(31) value(ERRLOG2_BASE + 80);
     /* The subnet object '%1' appears in the                        */
     /* Configuration\Sites\Subnets container in the DS.             */
     /* The name is not syntactically valid. The valid syntax is     */
     /* xx.xx.xx.xx/yy where xx.xx.xx.xx is a valid IP subnet number */
     /* and yy is the number of bits in the subnet mask.             */
     /* Correct the name of the subnet object.                       */

 %end; /* _LMERRLOG_ */
