 /*===================================================================*/
 /*                                                                   */
 /*  msacm.cpy                                                        */
 /*                                                                   */
 /*  Copyright (c) 1992-1995 Microsoft Corporation.                   */
 /*                All Rights Reserved.                               */
 /*                                                                   */
 /*  Description:                                                     */
 /*      Audio Compression Manager Public Header File                 */
 /*                                                                   */
 /*  History:                                                         */
 /*                                                                   */
 /*===================================================================*/

 %dcl #INC_ACM char ext;
 %if #INC_ACM = '' %then %do;
 %#INC_ACM = 'Y';

 %dcl UNICODE char ext;

 %dcl #INC_MMREG fixed;
 %if #INC_MMREG = 0 %then %do;
 %include mmreg;
 %end;


 %dcl #DRV_MAPPER_PREFERRED_INPUT_GET char ext;
 %if #DRV_MAPPER_PREFERRED_INPUT_GET = '' %then %do;
 %#DRV_MAPPER_PREFERRED_INPUT_GET = 'Y';
 dcl DRV_MAPPER_PREFERRED_INPUT_GET fixed bin(31) value(DRV_USER + 0);
 %end;

 %dcl #DRV_MAPPER_PREFERRED_OUTPUT_GET char ext;
 %if #DRV_MAPPER_PREFERRED_OUTPUT_GET = '' %then %do;
 %#DRV_MAPPER_PREFERRED_OUTPUT_GET = 'Y';
 dcl DRV_MAPPER_PREFERRED_OUTPUT_GET fixed bin(31) value(DRV_USER + 2);
 %end;

 %dcl #DRVM_MAPPER_STATUS char ext;
 %if #DRVM_MAPPER_STATUS = '' %then %do;
 %#DRVM_MAPPER_STATUS = 'Y';
 dcl DRVM_MAPPER_STATUS fixed bin(15) value('2000'xn);
 %end;

 %dcl #WIDM_MAPPER_STATUS char ext;
 %if #WIDM_MAPPER_STATUS = '' %then %do;
 %#WIDM_MAPPER_STATUS = 'Y';
 dcl WIDM_MAPPER_STATUS fixed bin(31) value(DRVM_MAPPER_STATUS + 0);
 dcl WAVEIN_MAPPER_STATUS_DEVICE fixed bin(31) value(0);
 dcl WAVEIN_MAPPER_STATUS_MAPPED fixed bin(31) value(1);
 dcl WAVEIN_MAPPER_STATUS_FORMAT fixed bin(31) value(2);
 %end;

 %dcl #WODM_MAPPER_STATUS char ext;
 %if #WODM_MAPPER_STATUS = '' %then %do;
 %#WODM_MAPPER_STATUS = 'Y';
 dcl WODM_MAPPER_STATUS fixed bin(31) value(DRVM_MAPPER_STATUS + 0);
 dcl WAVEOUT_MAPPER_STATUS_DEVICE fixed bin(31) value(0);
 dcl WAVEOUT_MAPPER_STATUS_MAPPED fixed bin(31) value(1);
 dcl WAVEOUT_MAPPER_STATUS_FORMAT fixed bin(31) value(2);
 %end;


 %dcl ACMAPI char ext;
 %ACMAPI=WINAPI;

 /*-------------------------------------------------------------------*/
 /*                                                                   */
 /*  ACM General API's and Defines                                    */
 /*                                                                   */
 /*-------------------------------------------------------------------*/


 /* There are four types of 'handles' used by the ACM.                */
 /* The first three are unique types that define specific objects:    */
 /*                                                                   */
 /* HACMDRIVERID: used to _identify_ an ACM driver.                   */
 /* This identifier can be used to _open_ the driver for querying     */
 /* details, etc about the driver.                                    */
 /*                                                                   */
 /* HACMDRIVER: used to manage a driver (codec, filter, etc).         */
 /* This handle is much like a handle to other media drivers          */
 /* -- you use it to send messages to the converter, query for        */
 /* capabilities, etc.                                                */
 /*                                                                   */
 /* HACMSTREAM: used to manage a 'stream' (conversion channel) with   */
 /* the ACM. you use a stream handle to convert data from one         */
 /* format/type to another -- much like dealing with a file handle.   */
 /*                                                                   */
 /* the fourth handle type is a generic type used on ACM functions    */
 /* that can accept two or more of the above handle types             */
 /* (for example the acmMetrics and acmDriverID functions).           */
 /*                                                                   */
 /* HACMOBJ: used to identify ACM objects. this handle is used on     */
 /* functions that can accept two or more ACM handle types.           */

 DECLARE_HANDLE (HACMDRIVERID);
 DECLARE_HANDLE (HACMDRIVER);
 DECLARE_HANDLE (HACMSTREAM);
 DECLARE_HANDLE (HACMOBJ);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* ACM Error Codes                                                   */
 /*                                                                   */
 /* Note that these error codes are specific errors that apply to the */
 /* ACM directly -- general errors are defined as MMSYSERR_*.         */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 %dcl #MMRESULT_ char ext;
 %if #MMRESULT_ = '' %then  %do;
 %#MMRESULT_ = 'Y';
 define alias MMRESULT type UINT;
 %end;

 dcl ACMERR_BASE        fixed bin(31) value(512);
 dcl ACMERR_NOTPOSSIBLE fixed bin(31) value(ACMERR_BASE + 0);
 dcl ACMERR_BUSY        fixed bin(31) value(ACMERR_BASE + 1);
 dcl ACMERR_UNPREPARED  fixed bin(31) value(ACMERR_BASE + 2);
 dcl ACMERR_CANCELED    fixed bin(31) value(ACMERR_BASE + 3);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* ACM Window Messages                                               */
 /*                                                                   */
 /* These window messages are sent by the ACM or ACM drivers to       */
 /* notify applications of events.                                    */
 /*                                                                   */
 /* Note that these window message numbers will also be defined in    */
 /* mmsystem.                                                         */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl MM_ACM_OPEN  fixed bin (31) value (MM_STREAM_OPEN);
 /* conversion callback messages */
 dcl MM_ACM_CLOSE fixed bin (31) value (MM_STREAM_CLOSE);
 dcl MM_ACM_DONE  fixed bin (31) value (MM_STREAM_DONE);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmGetVersion()                                                   */
 /*                                                                   */
 /* The ACM version is a 32 bit number that is broken into three      */
 /* parts as follows:                                                 */
 /*                                                                   */
 /*      bits 24 - 31:   8 bit _major_ version number                 */
 /*      bits 16 - 23:   8 bit _minor_ version number                 */
 /*      bits  0 - 15:   16 bit build number                          */
 /*                                                                   */
 /* This is then displayed as follows:                                */
 /*                                                                   */
 /*      bMajor = isrl (dwVersion, 24)                                */
 /*      bMinor = iand (isrl (dwVersion, 16), 'ff'xn)                 */
 /*      wBuild = iand (dwVersion, 'ffff'xn))                         */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmGetVersion
     entry ()
     returns (type DWORD byvalue optional)
     ACMAPI external('acmGetVersion');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmMetrics()                                                      */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmMetrics
     entry (type HACMOBJ,
            type UINT,
            type LPVOID)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmMetrics');

 dcl ACM_METRIC_COUNT_DRIVERS          fixed bin(31) value(1);
 dcl ACM_METRIC_COUNT_CODECS           fixed bin(31) value(2);
 dcl ACM_METRIC_COUNT_CONVERTERS       fixed bin(31) value(3);
 dcl ACM_METRIC_COUNT_FILTERS          fixed bin(31) value(4);
 dcl ACM_METRIC_COUNT_DISABLED         fixed bin(31) value(5);
 dcl ACM_METRIC_COUNT_HARDWARE         fixed bin(31) value(6);
 dcl ACM_METRIC_COUNT_LOCAL_DRIVERS    fixed bin(31) value(20);
 dcl ACM_METRIC_COUNT_LOCAL_CODECS     fixed bin(31) value(21);
 dcl ACM_METRIC_COUNT_LOCAL_CONVERTERS fixed bin(31) value(22);
 dcl ACM_METRIC_COUNT_LOCAL_FILTERS    fixed bin(31) value(23);
 dcl ACM_METRIC_COUNT_LOCAL_DISABLED   fixed bin(31) value(24);
 dcl ACM_METRIC_HARDWARE_WAVE_INPUT    fixed bin(31) value(30);
 dcl ACM_METRIC_HARDWARE_WAVE_OUTPUT   fixed bin(31) value(31);
 dcl ACM_METRIC_MAX_SIZE_FORMAT        fixed bin(31) value(50);
 dcl ACM_METRIC_MAX_SIZE_FILTER        fixed bin(31) value(51);
 dcl ACM_METRIC_DRIVER_SUPPORT         fixed bin(31) value(100);
 dcl ACM_METRIC_DRIVER_PRIORITY        fixed bin(31) value(101);

 /*-------------------------------------------------------------------*/
 /*                                                                   */
 /* ACM Drivers                                                       */
 /*                                                                   */
 /*-------------------------------------------------------------------*/

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverEnum()                                                   */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 define alias ACMDRIVERENUMCB
        entry (type HACMDRIVERID,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmDriverEnum
     entry (type ACMDRIVERENUMCB,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverEnum');

 dcl ACM_DRIVERENUMF_NOLOCAL  type DWORD value('40000000'xu);
 dcl ACM_DRIVERENUMF_DISABLED type DWORD value('80000000'xu);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverID()                                                     */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmDriverID
     entry (type HACMOBJ,
            type HACMDRIVERID byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverID');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverAdd()                                                    */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmDriverAddA
     entry (type HACMDRIVERID byaddr,
            type HINSTANCE,
            type LPARAM,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverAddA');

 dcl acmDriverAddW
     entry (type HACMDRIVERID byaddr,
            type HINSTANCE,
            type LPARAM,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverAddW');

 %dcl acmDriverAdd char ext;
 %if UNICODE ^= '' %then %do;
   %acmDriverAdd='acmDriverAddW';
 %end; %else %do;
   %acmDriverAdd='acmDriverAddA';
 %end;

 dcl ACM_DRIVERADDF_FUNCTION   fixed bin(31) value('00000003'xn);
     /* lParam is a procedure */
 dcl ACM_DRIVERADDF_NOTIFYHWND fixed bin(31) value('00000004'xn);
     /* lParam is notify hwnd */
 dcl ACM_DRIVERADDF_TYPEMASK   fixed bin(31) value('00000007'xn);
     /* driver type mask */
 dcl ACM_DRIVERADDF_LOCAL      fixed bin(31) value('00000000'xn);
     /* is local to current task */
 dcl ACM_DRIVERADDF_GLOBAL     fixed bin(31) value('00000008'xn);
     /* is global */

 /* prototype for ACM driver procedures that are installed as         */
 /* _functions_ or _notifications_ instead of as a standalone         */
 /* installable driver.                                               */

 define alias ACMDRIVERPROC
        entry (type DWORD,
               type HACMDRIVERID,
               type UINT,
               type LPARAM,
               type LPARAM)
        returns (type LRESULT byvalue optional)
        limited CALLBACK;

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverRemove()                                                 */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmDriverRemove
     entry (type HACMDRIVERID,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverRemove');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverOpen()                                                   */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmDriverOpen
     entry (type HACMDRIVER byaddr,
            type HACMDRIVERID,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverOpen');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverClose()                                                  */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmDriverClose
     entry (type HACMDRIVER,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverClose');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverMessage()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmDriverMessage
     entry (type HACMDRIVER,
            type UINT,
            type LPARAM,
            type LPARAM)
     returns (type LRESULT byvalue optional)
     ACMAPI external('acmDriverMessage');


 dcl ACMDM_USER          fixed bin(31) value(DRV_USER+'0000'xn);
 dcl ACMDM_RESERVED_LOW  fixed bin(31) value(DRV_USER+'2000'xn);
 dcl ACMDM_RESERVED_HIGH fixed bin(31) value(DRV_USER+'2FFF'xn);
 dcl ACMDM_BASE          fixed bin(31) value(ACMDM_RESERVED_LOW);
 dcl ACMDM_DRIVER_ABOUT  fixed bin(31) value(ACMDM_BASE + 11);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverPriority                                                 */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmDriverPriority
     entry (type HACMDRIVERID,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverPriority');

 dcl ACM_DRIVERPRIORITYF_ENABLE    type DWORD value('00000001'xu);
 dcl ACM_DRIVERPRIORITYF_DISABLE   type DWORD value('00000002'xu);
 dcl ACM_DRIVERPRIORITYF_ABLEMASK  type DWORD value('00000003'xu);
 dcl ACM_DRIVERPRIORITYF_BEGIN     type DWORD value('00010000'xu);
 dcl ACM_DRIVERPRIORITYF_END       type DWORD value('00020000'xu);
 dcl ACM_DRIVERPRIORITYF_DEFERMASK type DWORD value('00030000'xu);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmDriverDetails()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 /* ACMDRIVERDETAILS                                                  */
 /*                                                                   */
 /* The ACMDRIVERDETAILS structure is used to get various             */
 /* capabilitiesfrom an ACM driver (codec, converter, filter).        */
 /*                                                                   */
 dcl ACMDRIVERDETAILS_SHORTNAME_CHARS fixed bin(31) value(32);
 dcl ACMDRIVERDETAILS_LONGNAME_CHARS  fixed bin(31) value(128);
 dcl ACMDRIVERDETAILS_COPYRIGHT_CHARS fixed bin(31) value(80);
 dcl ACMDRIVERDETAILS_LICENSING_CHARS fixed bin(31) value(128);
 dcl ACMDRIVERDETAILS_FEATURES_CHARS  fixed bin(31) value(512);

 define structure
   1 ACMDRIVERDETAILSA,
     2 cbStruct    type DWORD, /* number of valid bytes in structure */
     2 fccType     type FOURCC,/* compressor type 'audc' */
     2 fccComp     type FOURCC,/* sub-type (not used; reserved) */
     2 wMid        type WORD,  /* manufacturer id */
     2 wPid        type WORD,  /* product id */
     2 vdwACM      type DWORD, /* version of the ACM *compiled* for */
     2 vdwDriver   type DWORD, /* version of the driver */
     2 fdwSupport  type DWORD, /* misc. support flags */
     2 cFormatTags type DWORD, /* total unique format tags supported */
     2 cFilterTags type DWORD, /* total unique filter tags supported */
     2 hicon       type HICON, /* handle to custom icon */
     2 szShortName char(ACMDRIVERDETAILS_SHORTNAME_CHARS-1) varz,
     2 szLongName  char(ACMDRIVERDETAILS_LONGNAME_CHARS -1) varz,
     2 szCopyright char(ACMDRIVERDETAILS_COPYRIGHT_CHARS-1) varz,
     2 szLicensing char(ACMDRIVERDETAILS_LICENSING_CHARS-1) varz,
     2 szFeatures  char(ACMDRIVERDETAILS_FEATURES_CHARS -1) varz;
 define alias  PACMDRIVERDETAILSA handle ACMDRIVERDETAILSA;
 define alias LPACMDRIVERDETAILSA handle ACMDRIVERDETAILSA;

 define structure
   1 ACMDRIVERDETAILSW,
     2 cbStruct    type DWORD, /* number of valid bytes in structure */
     2 fccType     type FOURCC,/* compressor type 'audc' */
     2 fccComp     type FOURCC,/* sub-type (not used; reserved) */
     2 wMid        type WORD,  /* manufacturer id */
     2 wPid        type WORD,  /* product id */
     2 vdwACM      type DWORD, /* version of the ACM *compiled* for */
     2 vdwDriver   type DWORD, /* version of the driver */
     2 fdwSupport  type DWORD, /* misc. support flags */
     2 cFormatTags type DWORD, /* total unique format tags supported */
     2 cFilterTags type DWORD, /* total unique filter tags supported */
     2 hicon       type HICON, /* handle to custom icon */
     2 szShortName WCHAR(ACMDRIVERDETAILS_SHORTNAME_CHARS-1) varz,
     2 szLongName  WCHAR(ACMDRIVERDETAILS_LONGNAME_CHARS -1) varz,
     2 szCopyright WCHAR(ACMDRIVERDETAILS_COPYRIGHT_CHARS-1) varz,
     2 szLicensing WCHAR(ACMDRIVERDETAILS_LICENSING_CHARS-1) varz,
     2 szFeatures  WCHAR(ACMDRIVERDETAILS_FEATURES_CHARS -1) varz;
 define alias  PACMDRIVERDETAILSW handle ACMDRIVERDETAILSW;
 define alias LPACMDRIVERDETAILSW handle ACMDRIVERDETAILSW;

 %if UNICODE ^= '' %then %do;
 define alias   ACMDRIVERDETAILS  type  ACMDRIVERDETAILSW;
 define alias  PACMDRIVERDETAILS type  PACMDRIVERDETAILSW;
 define alias LPACMDRIVERDETAILS type LPACMDRIVERDETAILSW;
 %end; %else %do;
 define alias   ACMDRIVERDETAILS  type  ACMDRIVERDETAILSA;
 define alias  PACMDRIVERDETAILS type  PACMDRIVERDETAILSA;
 define alias LPACMDRIVERDETAILS type LPACMDRIVERDETAILSA;
 %end;

 /* ACMDRIVERDETAILS.fccType                                          */
 /*                                                                   */
 /* ACMDRIVERDETAILS_FCCTYPE_AUDIOCODEC: the FOURCC used in the       */
 /* fccType field of the ACMDRIVERDETAILS structure to specify that   */
 /* this is an ACM codec designed for audio.                          */
 /*                                                                   */
 /* ACMDRIVERDETAILS.fccComp                                          */
 /*                                                                   */
 /* ACMDRIVERDETAILS_FCCCOMP_UNDEFINED: the FOURCC used in the        */
 /* fccComp field of the ACMDRIVERDETAILS structure.                  */
 /* This is currently an unused field.                                */

 dcl ACMDRIVERDETAILS_FCCTYPE_AUDIOCODEC type FOURCC
     value(mmioFOURCC('a', 'u', 'd', 'c'));
 dcl ACMDRIVERDETAILS_FCCCOMP_UNDEFINED  type FOURCC
     value(mmioFOURCC('00'x, '00'x, '00'x, '00'x));

 /* The following flags are used to specify the type of conversion(s) */
 /* that the converter/codec/filter supports. These are placed in the */
 /* fdwSupport field of the ACMDRIVERDETAILS structure.               */
 /* Note that a converter can support one or more of these flags in   */
 /* any combination.                                                  */
 /*                                                                   */
 /* ACMDRIVERDETAILS_SUPPORTF_CODEC: this flag is set if the driver   */
 /* supports conversions from one format tag to another format tag.   */
 /* For example, if a converter compresses WAVE_FORMAT_PCM to         */
 /* WAVE_FORMAT_ADPCM, then this bit should be set.                   */
 /*                                                                   */
 /* ACMDRIVERDETAILS_SUPPORTF_CONVERTER: this flags is set if the     */
 /* driver supports conversions on the same format tag.               */
 /* As an example, the PCM converter that is built into the ACM sets  */
 /* this bit (and only this bit) because it converts only PCM formats */
 /* (bits, sample rate).                                              */
 /*                                                                   */
 /* ACMDRIVERDETAILS_SUPPORTF_FILTER: this flag is set if the driver  */
 /* supports transformations on a single format. For example, a       */
 /* converter that changed the 'volume' of PCM data would set this    */
 /* bit. 'echo' and 'reverb' are also filter types.                   */
 /*                                                                   */
 /* ACMDRIVERDETAILS_SUPPORTF_HARDWARE: this flag is set if the driver*/
 /* supports hardware input and/or output through a waveform device.  */
 /*                                                                   */
 /* ACMDRIVERDETAILS_SUPPORTF_ASYNC: this flag is set if the driver   */
 /* supports async conversions.                                       */
 /*                                                                   */
 /* ACMDRIVERDETAILS_SUPPORTF_LOCAL: this flag is set _by the ACM_    */
 /* if a driver has been installed local to the current task.         */
 /* This flag is also set in the fdwSupport argument to the           */
 /* enumeration callback function for drivers.                        */
 /*                                                                   */
 /* ACMDRIVERDETAILS_SUPPORTF_DISABLED: this flag is set _by the ACM_ */
 /* if a driver has been disabled. This flag is also passed set in    */
 /* the fdwSupport argument to the enumeration callback function for  */
 /* drivers.                                                          */

 dcl ( ACMDRIVERDETAILS_SUPPORTF_CODEC     value('00000001'xu)
     , ACMDRIVERDETAILS_SUPPORTF_CONVERTER value('00000002'xu)
     , ACMDRIVERDETAILS_SUPPORTF_FILTER    value('00000004'xu)
     , ACMDRIVERDETAILS_SUPPORTF_HARDWARE  value('00000008'xu)
     , ACMDRIVERDETAILS_SUPPORTF_ASYNC     value('00000010'xu)
     , ACMDRIVERDETAILS_SUPPORTF_LOCAL     value('40000000'xu)
     , ACMDRIVERDETAILS_SUPPORTF_DISABLED  value('80000000'xu)
     ) type DWORD;

 dcl acmDriverDetailsA
     entry (type HACMDRIVERID,
            type ACMDRIVERDETAILSA byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverDetailsA');

 dcl acmDriverDetailsW
     entry (type HACMDRIVERID,
            type LPACMDRIVERDETAILSW byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmDriverDetailsW');

 %dcl acmDriverDetails char ext;
 %if UNICODE ^= '' %then %do;
   %acmDriverDetails='acmDriverDetailsW';
 %end; %else %do;
   %acmDriverDetails='acmDriverDetailsA';
 %end;


 /*-------------------------------------------------------------------*/
 /*                                                                   */
 /* ACM Format Tags                                                   */
 /*                                                                   */
 /*-------------------------------------------------------------------*/

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFormatTagDetails()                                             */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl ACMFORMATTAGDETAILS_FORMATTAG_CHARS fixed bin(31) value(48);

 define structure
   1 ACMFORMATTAGDETAILSA,
     2 cbStruct         type DWORD,
     2 dwFormatTagIndex type DWORD,
     2 dwFormatTag      type DWORD,
     2 cbFormatSize     type DWORD,
     2 fdwSupport       type DWORD,
     2 cStandardFormats type DWORD,
     2 szFormatTag  char(ACMFORMATTAGDETAILS_FORMATTAG_CHARS-1) varz;
 define alias  PACMFORMATTAGDETAILSA handle ACMFORMATTAGDETAILSA;
 define alias LPACMFORMATTAGDETAILSA handle ACMFORMATTAGDETAILSA;

 define structure
   1 ACMFORMATTAGDETAILSW,
     2 cbStruct         type DWORD,
     2 dwFormatTagIndex type DWORD,
     2 dwFormatTag      type DWORD,
     2 cbFormatSize     type DWORD,
     2 fdwSupport       type DWORD,
     2 cStandardFormats type DWORD,
     2 szFormatTag WCHAR(ACMFORMATTAGDETAILS_FORMATTAG_CHARS-1) varz;
 define alias  PACMFORMATTAGDETAILSW handle ACMFORMATTAGDETAILSW;
 define alias LPACMFORMATTAGDETAILSW handle ACMFORMATTAGDETAILSW;


 %if UNICODE ^= '' %then %do;
 define alias   ACMFORMATTAGDETAILS type   ACMFORMATTAGDETAILSW;
 define alias  PACMFORMATTAGDETAILS type  PACMFORMATTAGDETAILSW;
 define alias LPACMFORMATTAGDETAILS type LPACMFORMATTAGDETAILSW;
 %end; %else %do;
 define alias   ACMFORMATTAGDETAILS type   ACMFORMATTAGDETAILSA;
 define alias  PACMFORMATTAGDETAILS type  PACMFORMATTAGDETAILSA;
 define alias LPACMFORMATTAGDETAILS type LPACMFORMATTAGDETAILSA;
 %end;

 dcl acmFormatTagDetailsA
     entry (type HACMDRIVER,
            type ACMFORMATTAGDETAILSA byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatTagDetailsA');

 dcl acmFormatTagDetailsW
     entry (type HACMDRIVER,
            type ACMFORMATTAGDETAILSW byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatTagDetailsW');

 %dcl acmFormatTagDetails char ext;
 %if UNICODE ^= '' %then %do;
 %acmFormatTagDetails='acmFormatTagDetailsW';
 %end; %else %do;
   %acmFormatTagDetails='acmFormatTagDetailsA';
 %end;

 dcl ( ACM_FORMATTAGDETAILSF_INDEX       value('00000000'xu)
     , ACM_FORMATTAGDETAILSF_FORMATTAG   value('00000001'xu)
     , ACM_FORMATTAGDETAILSF_LARGESTSIZE value('00000002'xu)
     , ACM_FORMATTAGDETAILSF_QUERYMASK   value('0000000F'xu)
     ) type DWORD;

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFormatTagEnum()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 define alias ACMFORMATTAGENUMCBA
        entry (type HACMDRIVERID,
               type ACMFORMATTAGDETAILSA byaddr,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmFormatTagEnumA
     entry (type HACMDRIVER,
            type ACMFORMATTAGDETAILSA byaddr,
            type ACMFORMATTAGENUMCBA,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatTagEnumA');

 define alias ACMFORMATTAGENUMCBW
        entry (type HACMDRIVERID,
               type ACMFORMATTAGDETAILSW byaddr,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmFormatTagEnumW
     entry (type HACMDRIVER,
            type ACMFORMATTAGDETAILSW byaddr,
            type ACMFORMATTAGENUMCBW,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatTagEnumW');


 %dcl acmFormatTagEnum char ext;
 %if UNICODE ^= '' %then %do;
 define alias ACMFORMATTAGENUMCB type ACMFORMATTAGENUMCBW;
 %acmFormatTagEnum='acmFormatTagEnumW';
 %end; %else %do;
 define alias ACMFORMATTAGENUMCB type ACMFORMATTAGENUMCBA;
 %acmFormatTagEnum='acmFormatTagEnumA';
 %end;

 /*-------------------------------------------------------------------*/
 /*                                                                   */
 /* ACM Formats                                                       */
 /*                                                                   */
 /*-------------------------------------------------------------------*/

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFormatDetails()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl ACMFORMATDETAILS_FORMAT_CHARS fixed bin(31) value(128);

 define structure
   1 ACMFORMATDETAILSA,
     2 cbStruct      type DWORD,
     2 dwFormatIndex type DWORD,
     2 dwFormatTag   type DWORD,
     2 fdwSupport    type DWORD,
     2 pwfx          type LPWAVEFORMATEX,
     2 cbwfx         type DWORD,
     2 szFormat      char(ACMFORMATDETAILS_FORMAT_CHARS-1) varz;
 define alias  PACMFORMATDETAILSA handle ACMFORMATDETAILSA;
 define alias LPACMFORMATDETAILSA handle ACMFORMATDETAILSA;

 define structure
   1 ACMFORMATDETAILSW,
     2 cbStruct      type DWORD,
     2 dwFormatIndex type DWORD,
     2 dwFormatTag   type DWORD,
     2 fdwSupport    type DWORD,
     2 pwfx          type LPWAVEFORMATEX,
     2 cbwfx         type DWORD,
     2 szFormat      WCHAR (ACMFORMATDETAILS_FORMAT_CHARS-1) varz;
 define alias  PACMFORMATDETAILSW handle ACMFORMATDETAILSW;
 define alias LPACMFORMATDETAILSW handle ACMFORMATDETAILSW;


 %if UNICODE ^= '' %then %do;
 define alias   ACMFORMATDETAILS type   ACMFORMATDETAILSW;
 define alias  PACMFORMATDETAILS type  PACMFORMATDETAILSW;
 define alias LPACMFORMATDETAILS type LPACMFORMATDETAILSW;
 %end; %else %do;
 define alias   ACMFORMATDETAILS type   ACMFORMATDETAILSA;
 define alias  PACMFORMATDETAILS type  PACMFORMATDETAILSA;
 define alias LPACMFORMATDETAILS type LPACMFORMATDETAILSA;
 %end;

 dcl acmFormatDetailsA
     entry (type HACMDRIVER,
            type ACMFORMATDETAILSA byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatDetailsA');

 dcl acmFormatDetailsW
     entry (type HACMDRIVER,
            type ACMFORMATDETAILSW byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatDetailsW');


 %dcl acmFormatDetails char ext;
 %if UNICODE ^= '' %then %do;
   %acmFormatDetails='acmFormatDetailsW';
 %end; %else %do;
   %acmFormatDetails='acmFormatDetailsA';
 %end;

 dcl ( ACM_FORMATDETAILSF_INDEX     value('00000000'xu)
     , ACM_FORMATDETAILSF_FORMAT    value('00000001'xu)
     , ACM_FORMATDETAILSF_QUERYMASK value('0000000F'xu)
     ) type DWORD;

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFormatEnum()                                                   */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 define alias ACMFORMATENUMCBA
        entry (type HACMDRIVERID,
               type ACMFORMATDETAILSA byaddr,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmFormatEnumA
     entry (type HACMDRIVER,
            type ACMFORMATDETAILSA byaddr,
            type ACMFORMATENUMCBA,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatEnumA');

 define alias  ACMFORMATENUMCBW
        entry (type HACMDRIVERID,
               type ACMFORMATDETAILSW byaddr,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmFormatEnumW
     entry (type HACMDRIVER,
            type ACMFORMATDETAILSW byaddr,
            type ACMFORMATENUMCBW,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatEnumW');


 %dcl acmFormatEnum char ext;
 %if UNICODE ^= '' %then %do;
   define alias ACMFORMATENUMCB type ACMFORMATENUMCBW;
   %acmFormatEnum='acmFormatEnumW';
 %end; %else %do;
   define alias ACMFORMATENUMCB type ACMFORMATENUMCBA;
   %acmFormatEnum='acmFormatEnumA';
 %end;

 dcl ( ACM_FORMATENUMF_WFORMATTAG     value('00010000'xu)
     , ACM_FORMATENUMF_NCHANNELS      value('00020000'xu)
     , ACM_FORMATENUMF_NSAMPLESPERSEC value('00040000'xu)
     , ACM_FORMATENUMF_WBITSPERSAMPLE value('00080000'xu)
     , ACM_FORMATENUMF_CONVERT        value('00100000'xu)
     , ACM_FORMATENUMF_SUGGEST        value('00200000'xu)
     , ACM_FORMATENUMF_HARDWARE       value('00400000'xu)
     , ACM_FORMATENUMF_INPUT          value('00800000'xu)
     , ACM_FORMATENUMF_OUTPUT         value('01000000'xu)
     ) type DWORD;

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFormatSuggest()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmFormatSuggest
     entry (type HACMDRIVER,
            type WAVEFORMATEX byaddr,
            type WAVEFORMATEX byaddr,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatSuggest');

 dcl ( ACM_FORMATSUGGESTF_WFORMATTAG     value('00010000'xu)
     , ACM_FORMATSUGGESTF_NCHANNELS      value('00020000'xu)
     , ACM_FORMATSUGGESTF_NSAMPLESPERSEC value('00040000'xu)
     , ACM_FORMATSUGGESTF_WBITSPERSAMPLE value('00080000'xu)
     , ACM_FORMATSUGGESTF_TYPEMASK       value('00FF0000'xu)
     ) type DWORD;

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFormatChoose()                                                 */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl ACMHELPMSGSTRINGA      char  value('acmchoose_help');
 dcl ACMHELPMSGSTRINGW      WCHAR value('acmchoose_help');
 dcl ACMHELPMSGCONTEXTMENUA char  value('acmchoose_contextmenu');
 dcl ACMHELPMSGCONTEXTMENUW WCHAR value('acmchoose_contextmenu');
 dcl ACMHELPMSGCONTEXTHELPA char  value('acmchoose_contexthelp');
 dcl ACMHELPMSGCONTEXTHELPW WCHAR value('acmchoose_contexthelp');

 %dcl ACMHELPMSGSTRING char ext;
 %dcl ACMHELPMSGCONTEXTMENU char ext;
 %dcl ACMHELPMSGCONTEXTHELP char ext;
 %if UNICODE ^= '' %then %do;
 %ACMHELPMSGSTRING     ='ACMHELPMSGSTRINGW';
 %ACMHELPMSGCONTEXTMENU='ACMHELPMSGCONTEXTMENUW';
 %ACMHELPMSGCONTEXTHELP='ACMHELPMSGCONTEXTHELPW';
 %end; %else %do;
 %ACMHELPMSGSTRING     ='ACMHELPMSGSTRINGA';
 %ACMHELPMSGCONTEXTMENU='ACMHELPMSGCONTEXTMENUA';
 %ACMHELPMSGCONTEXTHELP='ACMHELPMSGCONTEXTHELPA';
 %end;

 /* MM_ACM_FORMATCHOOSE is sent to hook callbacks */
 /* by the Format Chooser Dialog...               */

 dcl MM_ACM_FORMATCHOOSE fixed bin(31) value('8000'xn);

 dcl ( FORMATCHOOSE_MESSAGE          value(0)
     , FORMATCHOOSE_FORMATTAG_VERIFY value(FORMATCHOOSE_MESSAGE+0)
     , FORMATCHOOSE_FORMAT_VERIFY    value(FORMATCHOOSE_MESSAGE+1)
     , FORMATCHOOSE_CUSTOM_VERIFY    value(FORMATCHOOSE_MESSAGE+2)
     ) fixed bin(31);

 define alias ACMFORMATCHOOSEHOOKPROCA
        entry (type HWND,
               type UINT,
               type WPARAM,
               type LPARAM)
        returns (type UINT byvalue optional)
        limited CALLBACK;

 define alias ACMFORMATCHOOSEHOOKPROCW
        entry (type HWND,
               type UINT,
               type WPARAM,
               type LPARAM)
        returns (type UINT byvalue optional)
        limited CALLBACK;


 %if UNICODE ^= '' %then %do;
 define alias ACMFORMATCHOOSEHOOKPROC type ACMFORMATCHOOSEHOOKPROCW;
 %end; %else %do;
 define alias ACMFORMATCHOOSEHOOKPROC type ACMFORMATCHOOSEHOOKPROCA;
 %end;

 define structure
   1 ACMFORMATCHOOSEA,
     2 cbStruct    type DWORD,/* sizeof(ACMFORMATCHOOSE) */
     2 fdwStyle    type DWORD,/* chooser style flags */
     2 hwndOwner   type HWND, /* caller's window handle */
     2 pwfx        type LPWAVEFORMATEX,
                   /* ptr to wfx buf to receive choice */
     2 cbwfx       type DWORD, /* size of mem buf for pwfx */
     2 pszTitle    type LPCSTR,/* dialog box title bar */
     2 szFormatTag char(ACMFORMATTAGDETAILS_FORMATTAG_CHARS-1) varz,
     2 szFormat    char(ACMFORMATDETAILS_FORMAT_CHARS-1) varz,
     2 pszName     type LPSTR,/* custom name selection */
     2 cchName     type DWORD,/* size in chars of mem buf for pszName */
     2 fdwEnum     type DWORD,/* format enumeration restrictions */
     2 pwfxEnum    type LPWAVEFORMATEX,
                   /* format describing restrictions */
     2 hInstance   type HINSTANCE,
                   /* app instance containing dlg template */
     2 pszTemplateName type LPCSTR,/* custom template name */
     2 lCustData   type LPARAM,/* data passed to hook fn. */
     2 pfnHook     type ACMFORMATCHOOSEHOOKPROCA;
                   /* ptr to hook function */
 define alias  PACMFORMATCHOOSEA handle ACMFORMATCHOOSEA;
 define alias LPACMFORMATCHOOSEA handle ACMFORMATCHOOSEA;

 define structure
   1 ACMFORMATCHOOSEW,
     2 cbStruct    type DWORD,/* sizeof(ACMFORMATCHOOSE) */
     2 fdwStyle    type DWORD,/* chooser style flags */
     2 hwndOwner   type HWND, /* caller's window handle */
     2 pwfx        type LPWAVEFORMATEX,
                   /* ptr to wfx buf to receive choice */
     2 cbwfx       type DWORD,/* size of mem buf for pwfx */
     2 pszTitle    type LPCWSTR,/* dialog box title bar */
     2 szFormatTag WCHAR(ACMFORMATTAGDETAILS_FORMATTAG_CHARS-1) varz,
     2 szFormat    WCHAR(ACMFORMATDETAILS_FORMAT_CHARS-1) varz,
     2 pszName     type LPWSTR,/* custom name selection */
     2 cchName     type DWORD,/* size in chars of mem buf for pszName */
     2 fdwEnum     type DWORD,/* format enumeration restrictions */
     2 pwfxEnum    type LPWAVEFORMATEX,
                   /* format describing restrictions */
     2 hInstance   type HINSTANCE,
                   /* app instance containing dlg template */
     2 pszTemplateName type LPCWSTR,/* custom template name */
     2 lCustData   type LPARAM,/* data passed to hook fn. */
     2 pfnHook     type ACMFORMATCHOOSEHOOKPROCW;
                   /* ptr to hook function */
 define alias  PACMFORMATCHOOSEW handle ACMFORMATCHOOSEW;
 define alias LPACMFORMATCHOOSEW handle ACMFORMATCHOOSEW;

 %if UNICODE ^= '' %then %do;
 define alias   ACMFORMATCHOOSE type ACMFORMATCHOOSEW;
 define alias  PACMFORMATCHOOSE type PACMFORMATCHOOSEW;
 define alias LPACMFORMATCHOOSE type LPACMFORMATCHOOSEW;
 %end; %else %do;
 define alias   ACMFORMATCHOOSE type   ACMFORMATCHOOSEA;
 define alias  PACMFORMATCHOOSE type  PACMFORMATCHOOSEA;
 define alias LPACMFORMATCHOOSE type LPACMFORMATCHOOSEA;
 %end;

 /* ACMFORMATCHOOSE.fdwStyle */

 dcl ( ACMFORMATCHOOSE_STYLEF_SHOWHELP             value('00000004'xu)
     , ACMFORMATCHOOSE_STYLEF_ENABLEHOOK           value('00000008'xu)
     , ACMFORMATCHOOSE_STYLEF_ENABLETEMPLATE       value('00000010'xu)
     , ACMFORMATCHOOSE_STYLEF_ENABLETEMPLATEHANDLE value('00000020'xu)
     , ACMFORMATCHOOSE_STYLEF_INITTOWFXSTRUCT      value('00000040'xu)
     , ACMFORMATCHOOSE_STYLEF_CONTEXTHELP          value('00000080'xu)
     ) type DWORD;

 dcl acmFormatChooseA
     entry (type ACMFORMATCHOOSEA byaddr)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatChooseA');

 dcl acmFormatChooseW
     entry (type ACMFORMATCHOOSEW byaddr)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFormatChooseW');

 %dcl acmFormatChoose char ext;
 %if UNICODE ^= '' %then %do;
   %acmFormatChoose='acmFormatChooseW';
 %end; %else %do;
   %acmFormatChoose='acmFormatChooseA';
 %end;

 /*-------------------------------------------------------------------*/
 /*                                                                   */
 /*  ACM Filter Tags                                                  */
 /*                                                                   */
 /*-------------------------------------------------------------------*/

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFilterTagDetails()                                             */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl ACMFILTERTAGDETAILS_FILTERTAG_CHARS fixed bin(31) value(48);

 define structure
   1 ACMFILTERTAGDETAILSA,
    2 cbStruct         type DWORD,
    2 dwFilterTagIndex type DWORD,
    2 dwFilterTag      type DWORD,
    2 cbFilterSize     type DWORD,
    2 fdwSupport       type DWORD,
    2 cStandardFilters type DWORD,
    2 szFilterTag      char(ACMFILTERTAGDETAILS_FILTERTAG_CHARS-1) varz;
 define alias  PACMFILTERTAGDETAILSA handle ACMFILTERTAGDETAILSA;
 define alias LPACMFILTERTAGDETAILSA handle ACMFILTERTAGDETAILSA;

 define structure
   1 ACMFILTERTAGDETAILSW,
    2 cbStruct         type DWORD,
    2 dwFilterTagIndex type DWORD,
    2 dwFilterTag      type DWORD,
    2 cbFilterSize     type DWORD,
    2 fdwSupport       type DWORD,
    2 cStandardFilters type DWORD,
    2 szFilterTag     WCHAR(ACMFILTERTAGDETAILS_FILTERTAG_CHARS-1) varz;
 define alias  PACMFILTERTAGDETAILSW handle ACMFILTERTAGDETAILSW;
 define alias LPACMFILTERTAGDETAILSW handle ACMFILTERTAGDETAILSW;


 %if UNICODE ^= '' %then %do;
 define alias   ACMFILTERTAGDETAILS type   ACMFILTERTAGDETAILSW;
 define alias  PACMFILTERTAGDETAILS type  PACMFILTERTAGDETAILSW;
 define alias LPACMFILTERTAGDETAILS type LPACMFILTERTAGDETAILSW;
 %end; %else %do;
 define alias   ACMFILTERTAGDETAILS type   ACMFILTERTAGDETAILSA;
 define alias  PACMFILTERTAGDETAILS type  PACMFILTERTAGDETAILSA;
 define alias LPACMFILTERTAGDETAILS type LPACMFILTERTAGDETAILSA;
 %end;

 dcl acmFilterTagDetailsA
     entry (type HACMDRIVER,
            type ACMFILTERTAGDETAILSA byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterTagDetailsA');

 dcl acmFilterTagDetailsW
     entry (type HACMDRIVER,
            type ACMFILTERTAGDETAILSW byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterTagDetailsW');


 %dcl acmFilterTagDetails char ext;
 %if UNICODE ^= '' %then %do;
   %acmFilterTagDetails='acmFilterTagDetailsW';
 %end; %else %do;
   %acmFilterTagDetails='acmFilterTagDetailsA';
 %end;

 dcl ( ACM_FILTERTAGDETAILSF_INDEX       value('00000000'xu)
     , ACM_FILTERTAGDETAILSF_FILTERTAG   value('00000001'xu)
     , ACM_FILTERTAGDETAILSF_LARGESTSIZE value('00000002'xu)
     , ACM_FILTERTAGDETAILSF_QUERYMASK   value('0000000F'xu)
     ) type DWORD;

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFilterTagEnum()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 define alias ACMFILTERTAGENUMCBA
        entry (type HACMDRIVERID,
               type ACMFILTERTAGDETAILSA byaddr,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmFilterTagEnumA
     entry (type HACMDRIVER,
            type ACMFILTERTAGDETAILSA byaddr,
            type ACMFILTERTAGENUMCBA,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterTagEnumA');

 define alias ACMFILTERTAGENUMCBW
        entry (type HACMDRIVERID,
               type ACMFILTERTAGDETAILSW byaddr,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmFilterTagEnumW
     entry (type HACMDRIVER,
            type ACMFILTERTAGDETAILSW byaddr,
            type ACMFILTERTAGENUMCBW,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterTagEnumW');

 %dcl acmFilterTagEnum char ext;
 %if UNICODE ^= '' %then %do;
   define alias ACMFILTERTAGENUMCB type ACMFILTERTAGENUMCBW;
   %acmFilterTagEnum='acmFilterTagEnumW';
 %end; %else %do;
   define alias ACMFILTERTAGENUMCB type ACMFILTERTAGENUMCBA;
   %acmFilterTagEnum='acmFilterTagEnumA';
 %end;

 /*-------------------------------------------------------------------*/
 /*                                                                   */
 /* ACM Filters                                                       */
 /*                                                                   */
 /*-------------------------------------------------------------------*/

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFilterDetails()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl ACMFILTERDETAILS_FILTER_CHARS fixed bin(31) value(128);

 define structure
   1 ACMFILTERDETAILSA,
     2 cbStruct      type DWORD,
     2 dwFilterIndex type DWORD,
     2 dwFilterTag   type DWORD,
     2 fdwSupport    type DWORD,
     2 pwfltr        type LPWAVEFILTER,
     2 cbwfltr       type DWORD,
     2 szFilter      char(ACMFILTERDETAILS_FILTER_CHARS-1) varz;
 define alias  PACMFILTERDETAILSA handle ACMFILTERDETAILSA;
 define alias LPACMFILTERDETAILSA handle ACMFILTERDETAILSA;

 define structure
   1 ACMFILTERDETAILSW,
     2 cbStruct      type DWORD,
     2 dwFilterIndex type DWORD,
     2 dwFilterTag   type DWORD,
     2 fdwSupport    type DWORD,
     2 pwfltr        type LPWAVEFILTER,
     2 cbwfltr       type DWORD,
     2 szFilter      WCHAR (ACMFILTERDETAILS_FILTER_CHARS-1) varz;
 define alias  PACMFILTERDETAILSW handle ACMFILTERDETAILSW;
 define alias LPACMFILTERDETAILSW handle ACMFILTERDETAILSW;


 %if UNICODE ^= '' %then %do;
 define alias   ACMFILTERDETAILS type   ACMFILTERDETAILSW;
 define alias  PACMFILTERDETAILS type  PACMFILTERDETAILSW;
 define alias LPACMFILTERDETAILS type LPACMFILTERDETAILSW;
 %end; %else %do;
 define alias   ACMFILTERDETAILS type   ACMFILTERDETAILSA;
 define alias  PACMFILTERDETAILS type  PACMFILTERDETAILSA;
 define alias LPACMFILTERDETAILS type LPACMFILTERDETAILSA;
 %end;

 dcl acmFilterDetailsA
     entry (type HACMDRIVER,
            type ACMFILTERDETAILSA byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterDetailsA');

 dcl acmFilterDetailsW
     entry (type HACMDRIVER,
            type ACMFILTERDETAILSW byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterDetailsW');

 %dcl acmFilterDetails char ext;
 %if UNICODE ^= '' %then %do;
   %acmFilterDetails='acmFilterDetailsW';
 %end; %else %do;
   %acmFilterDetails='acmFilterDetailsA';
 %end;

 dcl ( ACM_FILTERDETAILSF_INDEX     value('00000000'xu)
     , ACM_FILTERDETAILSF_FILTER    value('00000001'xu)
     , ACM_FILTERDETAILSF_QUERYMASK value('0000000F'xu)
     ) type DWORD;

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFilterEnum()                                                   */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 define alias ACMFILTERENUMCBA
        entry (type HACMDRIVERID,
               type ACMFILTERDETAILSA byaddr,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmFilterEnumA
     entry (type HACMDRIVER,
            type ACMFILTERDETAILSA byaddr,
            type ACMFILTERENUMCBA,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterEnumA');

 define alias ACMFILTERENUMCBW
        entry (type HACMDRIVERID,
               type ACMFILTERDETAILSW byaddr,
               type DWORD,
               type DWORD)
        returns (type BOOL byvalue optional)
        limited CALLBACK;

 dcl acmFilterEnumW
     entry (type HACMDRIVER,
            type LPACMFILTERDETAILSW byaddr,
            type ACMFILTERENUMCBW,
            type DWORD,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterEnumW');


 %dcl acmFilterEnum char ext;
 %if UNICODE ^= '' %then %do;
   define alias ACMFILTERENUMCB type ACMFILTERENUMCBW;
   %acmFilterEnum='acmFilterEnumW';
 %end; %else %do;
   define alias ACMFILTERENUMCB type ACMFILTERENUMCBA;
   %acmFilterEnum='acmFilterEnumA';
 %end;

 dcl ACM_FILTERENUMF_DWFILTERTAG type DWORD value('00010000'xu);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmFilterChoose()                                                 */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 /* MM_ACM_FILTERCHOOSE is sent to hook callbacks */
 /* by the Filter Chooser Dialog...               */

 dcl MM_ACM_FILTERCHOOSE fixed bin(31) value('8000'xn);

 dcl ( FILTERCHOOSE_MESSAGE          value(0)
     , FILTERCHOOSE_FILTERTAG_VERIFY value(FILTERCHOOSE_MESSAGE+0)
     , FILTERCHOOSE_FILTER_VERIFY    value(FILTERCHOOSE_MESSAGE+1)
     , FILTERCHOOSE_CUSTOM_VERIFY    value(FILTERCHOOSE_MESSAGE+2)
     ) fixed bin(31);

 define alias ACMFILTERCHOOSEHOOKPROCA
        entry (type HWND,
               type UINT,
               type WPARAM,
               type LPARAM)
        returns (type UINT byvalue optional)
        limited CALLBACK;


 define alias ACMFILTERCHOOSEHOOKPROCW
        entry (type HWND,
               type UINT,
               type WPARAM,
               type LPARAM)
        returns (type UINT byvalue optional)
        limited CALLBACK;


 %if UNICODE ^= '' %then %do;
 define alias ACMFILTERCHOOSEHOOKPROC type ACMFILTERCHOOSEHOOKPROCW;
 %end; %else %do;
 define alias ACMFILTERCHOOSEHOOKPROC type ACMFILTERCHOOSEHOOKPROCA;
 %end;

 /* ACMFILTERCHOOSE */

 define structure
   1 ACMFILTERCHOOSEA,
     2 cbStruct    type DWORD,/* sizeof(ACMFILTERCHOOSE) */
     2 fdwStyle    type DWORD,/* chooser style flags */
     2 hwndOwner   type HWND,/* caller's window handle */
     2 pwfltr      type LPWAVEFILTER,
                   /* ptr to wfltr buf to receive choice */
     2 cbwfltr     type DWORD,/* size of mem buf for pwfltr */
     2 pszTitle    type LPCSTR,
     2 szFilterTag char(ACMFILTERTAGDETAILS_FILTERTAG_CHARS-1) varz,
     2 szFilter    char(ACMFILTERDETAILS_FILTER_CHARS-1) varz,
     2 pszName     type LPSTR,/* custom name selection */
     2 cchName     type DWORD,/* size in chars of mem buf for pszName */
     2 fdwEnum     type DWORD,/* filter enumeration restrictions */
     2 pwfltrEnum  type LPWAVEFILTER,/* filter describing restrictions*/
     2 hInstance   type HINSTANCE,
                   /* app instance containing dlg template */
     2 pszTemplateName type LPCSTR,/* custom template name */
     2 lCustData   type LPARAM,/* data passed to hook fn. */
     2 pfnHook     type ACMFILTERCHOOSEHOOKPROCA;/* ptr to hook func */
 define alias  PACMFILTERCHOOSEA handle ACMFILTERCHOOSEA;
 define alias LPACMFILTERCHOOSEA handle ACMFILTERCHOOSEA;

 define structure
   1 ACMFILTERCHOOSEW,
     2 cbStruct    type DWORD,/* sizeof(ACMFILTERCHOOSE) */
     2 fdwStyle    type DWORD,/* chooser style flags */
     2 hwndOwner   type HWND,/* caller's window handle */
     2 pwfltr      type LPWAVEFILTER,
                   /* ptr to wfltr buf to receive choice */
     2 cbwfltr     type DWORD,/* size of mem buf for pwfltr */
     2 pszTitle    type LPCWSTR,
     2 szFilterTag WCHAR(ACMFILTERTAGDETAILS_FILTERTAG_CHARS-1) varz,
     2 szFilter    WCHAR(ACMFILTERDETAILS_FILTER_CHARS-1) varz,
     2 pszName     type LPWSTR,/* custom name selection */
     2 cchName     type DWORD,/* size in chars of mem buf for pszName */
     2 fdwEnum     type DWORD,/* filter enumeration restrictions */
     2 pwfltrEnum  type LPWAVEFILTER,/* filter describing restrictions*/
     2 hInstance   type HINSTANCE,
                   /* app instance containing dlg template */
     2 pszTemplateName type LPCWSTR,/* custom template name */
     2 lCustData   type LPARAM,/* data passed to hook fn. */
     2 pfnHook type ACMFILTERCHOOSEHOOKPROCW;/* ptr to hook func */
 define alias  PACMFILTERCHOOSEW handle ACMFILTERCHOOSEW;
 define alias LPACMFILTERCHOOSEW handle ACMFILTERCHOOSEW;


 %if UNICODE ^= '' %then %do;
 define alias   ACMFILTERCHOOSE type   ACMFILTERCHOOSEW;
 define alias  PACMFILTERCHOOSE type  PACMFILTERCHOOSEW;
 define alias LPACMFILTERCHOOSE type LPACMFILTERCHOOSEW;
 %end; %else %do;
 define alias   ACMFILTERCHOOSE type   ACMFILTERCHOOSEA;
 define alias  PACMFILTERCHOOSE type  PACMFILTERCHOOSEA;
 define alias LPACMFILTERCHOOSE type LPACMFILTERCHOOSEA;
 %end;

 /*  ACMFILTERCHOOSE.fdwStyle */

 dcl ( ACMFILTERCHOOSE_STYLEF_SHOWHELP             value('00000004'xu)
     , ACMFILTERCHOOSE_STYLEF_ENABLEHOOK           value('00000008'xu)
     , ACMFILTERCHOOSE_STYLEF_ENABLETEMPLATE       value('00000010'xu)
     , ACMFILTERCHOOSE_STYLEF_ENABLETEMPLATEHANDLE value('00000020'xu)
     , ACMFILTERCHOOSE_STYLEF_INITTOFILTERSTRUCT   value('00000040'xu)
     , ACMFILTERCHOOSE_STYLEF_CONTEXTHELP          value('00000080'xu)
     ) type DWORD;

 dcl acmFilterChooseA
     entry (type ACMFILTERCHOOSEA byaddr)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterChooseA');

 dcl acmFilterChooseW
     entry (type ACMFILTERCHOOSEW byaddr)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmFilterChooseW');


 %dcl acmFilterChoose char ext;
 %if UNICODE ^= '' %then %do;
   %acmFilterChoose='acmFilterChooseW';
 %end; %else %do;
   %acmFilterChoose='acmFilterChooseA';
 %end;

 /*-------------------------------------------------------------------*/
 /*                                                                   */
 /* ACM Stream API's                                                  */
 /*                                                                   */
 /*-------------------------------------------------------------------*/

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmStreamOpen()                                                   */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 define structure
   1 ACMSTREAMHEADER,
     2 cbStruct         type DWORD,/* sizeof(ACMSTREAMHEADER) */
     2 fdwStatus        type DWORD,/* ACMSTREAMHEADER_STATUSF_* */
     2 dwUser           type DWORD,/* user instance data for hdr */
     2 pbSrc            type LPBYTE,
     2 cbSrcLength      type DWORD,
     2 cbSrcLengthUsed  type DWORD,
     2 dwSrcUser        type DWORD,/* user instance data for src */
     2 pbDst            type LPBYTE,
     2 cbDstLength      type DWORD,
     2 cbDstLengthUsed  type DWORD,
     2 dwDstUser        type DWORD,/* user instance data for dst */
     2 dwReservedDriver (10) type DWORD;/* driver reserved work space */

 define alias  PACMSTREAMHEADER handle ACMSTREAMHEADER;
 define alias LPACMSTREAMHEADER handle ACMSTREAMHEADER;

 /*  ACMSTREAMHEADER.fdwStatus                                     */
 /*                                                                */
 /*  ACMSTREAMHEADER_STATUSF_DONE: done bit for async conversions. */

 dcl ACMSTREAMHEADER_STATUSF_DONE     fixed bin(31) value('00010000'xn);
 dcl ACMSTREAMHEADER_STATUSF_PREPARED fixed bin(31) value('00020000'xn);
 dcl ACMSTREAMHEADER_STATUSF_INQUEUE  fixed bin(31) value('00100000'xn);

 dcl acmStreamOpen
     entry (type HACMSTREAM byaddr,  /* pointer to stream handle */
            type HACMDRIVER,         /* optional driver handle */
            type WAVEFORMATEX byaddr,/* source format to convert */
            type WAVEFORMATEX byaddr,/* required destination format */
            type WAVEFILTER byaddr optional, /* optional filter */
            type DWORD,              /* callback */
            type DWORD,              /* callback instance data */
            type DWORD)           /* ACM_STREAMOPENF_* and CALLBACK_* */
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmStreamOpen');

 dcl ACM_STREAMOPENF_QUERY       type DWORD value('00000001'xu);
 dcl ACM_STREAMOPENF_ASYNC       type DWORD value('00000002'xu);
 dcl ACM_STREAMOPENF_NONREALTIME type DWORD value('00000004'xu);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmStreamClose()                                                  */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmStreamClose
     entry (type HACMSTREAM,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmStreamClose');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmStreamSize()                                                   */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmStreamSize
     entry (type HACMSTREAM,
            type DWORD,
            type DWORD byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmStreamSize');

 dcl ACM_STREAMSIZEF_SOURCE      type DWORD value('00000000'xu);
 dcl ACM_STREAMSIZEF_DESTINATION type DWORD value('00000001'xu);
 dcl ACM_STREAMSIZEF_QUERYMASK   type DWORD value('0000000F'xu);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmStreamReset()                                                  */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmStreamReset
     entry (type HACMSTREAM,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmStreamReset');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmStreamMessage()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmStreamMessage
     entry (type HACMSTREAM,
            type UINT,
            type LPARAM,
            type LPARAM)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmStreamMessage');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmStreamConvert()                                                */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmStreamConvert
     entry (type HACMSTREAM,
            type ACMSTREAMHEADER byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmStreamConvert');

 dcl ACM_STREAMCONVERTF_BLOCKALIGN type DWORD value('00000004'xu);
 dcl ACM_STREAMCONVERTF_START      type DWORD value('00000010'xu);
 dcl ACM_STREAMCONVERTF_END        type DWORD value('00000020'xu);

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmStreamPrepareHeader()                                          */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmStreamPrepareHeader
     entry (type HACMSTREAM,
            type ACMSTREAMHEADER byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmStreamPrepareHeader');

 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
 /*                                                                   */
 /* acmStreamUnprepareHeader()                                        */
 /*                                                                   */
 /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

 dcl acmStreamUnprepareHeader
     entry (type HACMSTREAM,
            type ACMSTREAMHEADER byaddr,
            type DWORD)
     returns (type MMRESULT byvalue optional)
     ACMAPI external('acmStreamUnprepareHeader');

 %end; /* _INC_ACM */
