 /*********************************************************************/
 /* Winsock2.cpy - definitions to be used with the WinSock 2 DLL and  */
 /*                WinSock 2 applications.                            */
 /*                                                                   */
 /* This header file corresponds to version 2.2.x of the WinSock API  */
 /* specification.                                                    */
 /*                                                                   */
 /* This file includes parts which are Copyright (c) 1982-1986        */
 /* Regents of the University of California.  All rights reserved.    */
 /* The Berkeley Software License Agreement specifies the terms and   */
 /* conditions for redistribution.                                    */
 /*********************************************************************/

 %dcl #WINSOCK2API_ char ext;
 %dcl #WINSOCKAPI_ char ext;
 %if #WINSOCK2API_ = '' %then %do;
 %#WINSOCK2API_ = 'Y';
 %#WINSOCKAPI_ = 'Y';
 /* Prevent inclusion of winsock.cpy in windows.cpy */

 /* Default: include function prototypes, */
 /* don't include functiontype aliases    */

 %dcl INCL_WINSOCK_API_PROTOTYPES char ext;
 %if INCL_WINSOCK_API_PROTOTYPES = '' %then %do;
    %INCL_WINSOCK_API_PROTOTYPES = 'Y'; /* 0 and 1 won't work */
 %end;
 %dcl INCL_WINSOCK_API_TYPEDEFS char ext;
 %if INCL_WINSOCK_API_TYPEDEFS = '' %then %do;
    %INCL_WINSOCK_API_TYPEDEFS = 'N'; /* 0 and 1 won't work */
 %end;

 /* Pull in WINBASE.CPY if necessary */
 %dcl #WINBASE_ char ext;
 %if #WINBASE_ = '' %then %do;
   %include winbase;
 %end;

 %dcl UNICODE char ext;

 /* Basic system type definitions,       */
 /* taken from the BSD file sys/types.h. */
 define alias u_char  unsigned fixed bin(8);
 define alias u_short unsigned fixed bin(16);
 define alias u_int   unsigned fixed bin(32);
 define alias u_long  unsigned fixed bin(32);

 /* The new type to be used in all instances which refer to sockets. */
 define alias SOCKET type u_int;

 /* Select uses arrays of SOCKETs.  These procedures manipulate such  */
 /* arrays. FD_SETSIZE may be defined by the user before including    */
 /* this file, but the default here should be >= 64.                  */
 /*                                                                   */
 /* CAVEAT IMPLEMENTOR and USER: THESE MACROS AND TYPES MUST BE       */
 /* INCLUDED IN WINSOCK2.CPY EXACTLY AS SHOWN HERE.                   */
 %dcl FD_SETSIZE fixed ext;
 %if FD_SETSIZE = 0 %then %do;
   %FD_SETSIZE = 64;
 %end; /* FD_SETSIZE */

 define structure
   1 fd_set,
     2 fd_count fixed bin (31),           /* how many are SET? */
     2 fd_array (FD_SETSIZE) type SOCKET; /* array of SOCKETs  */

 dcl __WSAFDIsSet entry (
        type SOCKET,
        handle fd_set)
          returns (fixed bin(31) byvalue)
          options (byvalue linkage(stdcall) nodescriptor)
          external ('__WSAFDIsSet');

 FD_CLR: proc (fd, set)
         options (inline reorder) internal;
   dcl fd  type SOCKET byvalue,
       set handle fd_set byvalue;
   dcl i fixed bin (31);

   do i = 1 to set => fd_count
      while (set => fd_array (i) ^= fd);
   end;
   if i <= set => fd_count then
   do; /* found, so remove */
     do i = i to set => fd_count - 1;
       set => fd_array (i) = set => fd_array (i + 1);
     end;
     set => fd_count -= 1;
   end;
 end FD_CLR;

 FD_SET#: proc (fd, set)
          options (inline reorder) internal;
   dcl fd  type SOCKET byvalue,
       set handle fd_set byvalue;
   dcl i fixed bin (31);

   do i = 1 to set => fd_count
      while (set => fd_array (i) ^= fd);
   end;
   if i > set => fd_count then
   do; /* not found, ... */
     if set => fd_count < FD_SETSIZE then
     do; /* ... so append */
       set => fd_count += 1;
       set => fd_array (set => fd_count) = fd;
     end;
   end;
 end FD_SET#;

 FD_ZERO: proc (set)
          options (inline reorder) internal;
   dcl set handle fd_set byvalue;
   set => fd_count = 0;
 end FD_ZERO;

 FD_ISSET: proc (fd, set) returns (fixed bin (31) byvalue)
           options (inline reorder) internal;
   dcl fd  type SOCKET byvalue,
       set handle fd_set byvalue;
   return (__WSAFDIsSet (fd, set));
 end FD_ISSET;

 /* Structure used in select(), taken from the BSD file sys/time.h */

 define structure
   1 timeval,
     2 tv_sec  fixed bin(31), /* seconds */
     2 tv_usec fixed bin(31); /* and microseconds */

 TIMERISSET: proc (TVP) returns (bit(1) byvalue)
             options(inline reorder) internal;
    dcl TVP handle timeval byvalue;
    return (TVP => TV_SEC ^= 0 | TVP => TV_USEC ^= 0);
 end TIMERISSET;

 %dcl TIMERCMP entry;
 %TIMERCMP: proc (T1, T2, OP) returns (char);
   /* op may not be '>=' or '<=' */
   DCL (T1, T2, OP) char;

    return ('((' || T1 || ' => TV_SEC ' || OP || ' ' || T2 ||
            ' => TV_SEC) | ((' || T1 || ' => TV_SEC = ' || T2 ||
            ' => TV_SEC) & (' || T1 || ' => TV_USEC ' || OP ||
            ' ' || T2 || ' => TV_USEC)))');
 %end TIMERCMP;

 TIMERCLEAR: proc (TVP)
             options(inline reorder) internal;
    dcl TVP handle timeval byvalue;
    TVP => TV_SEC  = 0;
    TVP => TV_USEC = 0;
 end TIMERCLEAR;


 /* Commands for ioctlsocket(),  taken from the BSD file fcntl.h. */
 /*                                                               */
 /* Ioctl's have the command encoded in the lower word,           */
 /* and the size of any in or out parameters in the upper         */
 /* word. The high 2 bits of the upper word are used              */
 /* to encode the in/out status of the parameter; for now         */
 /* we restrict parameters to at most 128 bytes.                  */
 dcl IOCPARM_MASK fixed bin (31) value('7f'xn);
 /* parameters must be < 128 bytes */
 dcl IOC_VOID  fixed bin(31) value('20000000'xn),  /* no parameters  */
     IOC_OUT   fixed bin(31) value('40000000'xn),  /* copy out parms */
     IOC_IN    fixed bin(31) value('80000000'xn),  /* copy in parms  */
     IOC_INOUT fixed bin(31) value (ior(IOC_IN, IOC_OUT));
     /* 0x20000000 distinguishes new & old ioctl's */

 %#IO: proc (x, y) returns (char);
   dcl (x, y) char;
   return ('ior(IOC_VOID, isll (rank (' || x || '), 8), ' || y || ')');
 %end;

 %#IOR: proc (x, y, t) returns (char);
   dcl (x, y, t) char;
   return ('ior(IOC_OUT, isll (iand (size (:' || t ||
           ':), IOCPARM_MASK), 16), isll (rank (' || x || '), 8), ' ||
           y || ')');
 %end;

 %#IOW: proc (x, y, t) returns (char);
   dcl (x, y, t) char;
   return ('ior(IOC_IN, isll (iand (size (:' || t ||
           ':), IOCPARM_MASK), 16), isll (rank (' || x || '), 8), ' ||
           y || ')');
 %end;
 %dcl (#IO, #IOR, #IOW) entry;


 dcl FIONREAD fixed bin (31) value (#IOR ('f', 127, u_long)),
              /* get # bytes to read */
     FIONBIO  fixed bin (31) value (#IOW ('f', 126, u_long)),
              /* set/clear non-blocking i/o */
     FIOASYNC fixed bin (31) value (#IOW ('f', 125, u_long));
              /* set/clear async i/o */

 /* Socket I/O Controls */
 dcl SIOCSHIWAT fixed bin (31) value (#IOW ('s',  0, u_long)),
                /* set high watermark */
     SIOCGHIWAT fixed bin (31) value (#IOR ('s',  1, u_long)),
                /* get high watermark */
     SIOCSLOWAT fixed bin (31) value (#IOW ('s',  2, u_long)),
                /* set low watermark */
     SIOCGLOWAT fixed bin (31) value (#IOR ('s',  3, u_long)),
                /* get low watermark */
     SIOCATMARK fixed bin (31) value (#IOR ('s',  7, u_long));
                /* at oob mark? */

 /* Structures returned by network data base library, taken from the  */
 /* BSD file netdb.h.  All addresses are supplied in host order, and  */
 /* returned in network order (suitable for use in system calls).     */

 /* Many of the structures contain a pointer to a list of pointers    */
 /* terminated by a sysnull().                                        */
 /* Whereas C allows a pointer to be treated as an array (p[x]), in   */
 /* PL/I one needs an actual array declaration. Note: Lower bound: 1. */
 dcl $addr_list (256) pointer based; /* list, longer than necessary   */

 define structure
   1 hostent,
     2 h_name                     /* official name of host   */
                   pointer, /* to char (*) varz */
     2 h_aliases                  /* alias list              */
                   pointer, /* to (*) ptr to char (*) varz */
     2 h_addrtype                 /* host address type       */
                   fixed bin(15),
     2 h_length                   /* length of address */
                   fixed bin(15),
     2 h_addr_list                /* list of addr from namsrv*/
                   pointer; /* to (*) fixed bin (31) */
 %dcl h_addr char;
 %h_addr = 'h_addr_list -> $addr_list(1)';

 /* * It is assumed here that a network number fits in 32 bits. */
 define structure
   1 netent,
     2 n_name     pointer,       /* official name of net   */
     2 n_aliases  pointer,       /* alias list             */
     2 n_addrtype fixed bin(15), /* net address type       */
     2 n_net      type u_long;   /* network #              */

 define structure
   1 servent,
     2 s_name     pointer,       /* official service name  */
     2 s_aliases  pointer,       /* alias list             */
     2 s_port     fixed bin(15), /* port #                 */
     2 s_proto    pointer;       /* protocol to use        */

 define structure
   1 protoent,
     2 p_name     pointer,       /* official protocol name */
     2 p_aliases  pointer,       /* alias list             */
     2 p_proto    fixed bin(15); /* protocol #             */


 /* Constants and structures defined by the internet system,       */
 /* Per RFC 790, September 1981, taken from BSD file netinet/in.h. */

 /* Protocols */
 dcl
 ( IPPROTO_IP   value (  0) /* dummy for IP */
 , IPPROTO_ICMP value (  1) /* control message protocol */
 , IPPROTO_IGMP value (  2) /* internet group management protocol */
 , IPPROTO_GGP  value (  3) /* gateway^2 (deprecated) */
 , IPPROTO_TCP  value (  6) /* tcp */
 , IPPROTO_PUP  value ( 12) /* pup */
 , IPPROTO_UDP  value ( 17) /* user datagram protocol */
 , IPPROTO_IDP  value ( 22) /* xns idp */
 , IPPROTO_ND   value ( 77) /* UNOFFICIAL net disk proto */
 , IPPROTO_RAW  value (255) /* raw IP packet */
 , IPPROTO_MAX  value (256)
 ) fixed bin (31);

 /* Port/socket numbers: network standard functions */
 dcl
 ( IPPORT_ECHO        value(7)
 , IPPORT_DISCARD     value(9)
 , IPPORT_SYSTAT      value(11)
 , IPPORT_DAYTIME     value(13)
 , IPPORT_NETSTAT     value(15)
 , IPPORT_FTP         value(21)
 , IPPORT_TELNET      value(23)
 , IPPORT_SMTP        value(25)
 , IPPORT_TIMESERVER  value(37)
 , IPPORT_NAMESERVER  value(42)
 , IPPORT_WHOIS       value(43)
 , IPPORT_MTP         value(57)

 /* Port/socket numbers: host specific functions */
 , IPPORT_TFTP        value(69)
 , IPPORT_RJE         value(77)
 , IPPORT_FINGER      value(79)
 , IPPORT_TTYLINK     value(87)
 , IPPORT_SUPDUP      value(95)

 /* UNIX TCP sockets */
 , IPPORT_EXECSERVER  value(512)
 , IPPORT_LOGINSERVER value(513)
 , IPPORT_CMDSERVER   value(514)
 , IPPORT_EFSSERVER   value(520)

 /* UNIX UDP sockets */
 , IPPORT_BIFFUDP     value(512)
 , IPPORT_WHOSERVER   value(513)
 , IPPORT_ROUTESERVER value(520)
 /* 520+1 also used */

 /* Ports < IPPORT_RESERVED are reserved for */
 /* privileged processes (e.g. root).        */
 , IPPORT_RESERVED    value(1024)
 ) fixed bin(31);

 /* Link numbers */
 dcl IMPLINK_IP         fixed bin(31) value(155),
     IMPLINK_LOWEXPER   fixed bin(31) value(156),
     IMPLINK_HIGHEXPER  fixed bin(31) value(158);

 /* Internet address (old style... should be updated) */

 define structure
   1 in_addr,
     2 S_un union,
       3 SS_un_b,
         4 (s_b1,s_b2,s_b3,s_b4) type u_char,
       3 S_un_w,
         4 (s_w1,s_w2) type u_short,
       3 S_addr type u_long;

 %dcl h_addr char;
 %h_addr = 'h_addr_list -> $addr_list(1)';

 %dcl s_host  char;
 %s_host  = 'S_un.S_un_b.s_b2'; /* host on imp */
 %dcl s_net   char;
 %s_net   = 'S_un.S_un_b.s_b1'; /* network */
 %dcl s_imp   char;
 %s_imp   = 'S_un.S_un_w.s_w2'; /* imp */
 %dcl s_impno char;
 %s_impno = 'S_un.S_un_b.s_b4'; /* imp # */
 %dcl s_lh    char;
 %s_lh    = 'S_un.S_un_b.s_b3'; /* logical host */

 /* Definitions of bits in internet address integers.                 */
 /* On subnets, the decomposition of addresses to host and net parts  */
 /* is done according to subnet mask, not the masks here.             */

 IN_CLASSA: proc (i) returns (bit (1) aligned byvalue)
            options (inline reorder) internal;
   dcl i fixed bin (31) byvalue;
   dcl iand builtin;
   return (iand (i, '80000000'xn) = 0);
 end IN_CLASSA;

 dcl IN_CLASSA_NET       fixed bin(31) value('ff000000'xn),
     IN_CLASSA_NSHIFT    fixed bin(31) value(24),
     IN_CLASSA_HOST      fixed bin(31) value('00ffffff'xn),
     IN_CLASSA_MAX       fixed bin(31) value(128);

 IN_CLASSB: proc (i) returns (bit (1) aligned byvalue)
            options (inline reorder) internal;
   dcl i fixed bin (31) byvalue;
   dcl iand builtin;
   return (iand (i, 'C0000000'xn) = '80000000'xn);
 end IN_CLASSB;

 dcl IN_CLASSB_NET       fixed bin(31) value('ffff0000'xn),
     IN_CLASSB_NSHIFT    fixed bin(31) value(16),
     IN_CLASSB_HOST      fixed bin(31) value('0000ffff'xn),
     IN_CLASSB_MAX       fixed bin(31) value(65536);

 IN_CLASSC: proc (i) returns (bit (1) aligned byvalue)
            options (inline reorder) internal;
   dcl i fixed bin (31) byvalue;
   dcl iand builtin;
   return (iand (i, 'E0000000'xn) = 'C0000000'xn);
 end IN_CLASSC;

 dcl IN_CLASSC_NET       fixed bin(31) value('ffffff00'xn),
     IN_CLASSC_NSHIFT    fixed bin(31) value(8),
     IN_CLASSC_HOST      fixed bin(31) value('000000ff'xn);

 IN_CLASSD: proc (i) returns (bit (1) aligned byvalue)
            options (inline reorder) internal;
   dcl i fixed bin (31) byvalue;
   dcl iand builtin;
   return (iand (i, 'F0000000'xn) = 'E0000000'xn);
 end IN_CLASSD;

 dcl IN_CLASSD_NET       fixed bin(31) value('f0000000'xn),
     IN_CLASSD_NSHIFT    fixed bin(31) value(28),
     IN_CLASSD_HOST      fixed bin(31) value('0fffffff'xn);

 IN_MULTICAST: proc (i) returns (bit (1) aligned byvalue)
               options (inline reorder) internal;
   dcl i fixed bin (31) byvalue;
   return (IN_CLASSD (i));
 end IN_MULTICAST;

 dcl INADDR_ANY             fixed bin(31) value(0),
     INADDR_LOOPBACK        fixed bin(31) value('7f000001'xn),
     INADDR_BROADCAST       fixed bin(31) value(-1),
     INADDR_NONE            fixed bin(31) value(-1),
     ADDR_ANY               fixed bin(31) value(INADDR_ANY);


 /* Socket address, internet style. */
 define structure
   1 sockaddr_in,
     2 sin_family fixed bin(15),
     2 sin_port   type u_short,
     2 sin_addr   type in_addr,
     2 sin_zero   char(8);


 dcl WSADESCRIPTION_LEN fixed bin(31) value(256);
 dcl WSASYS_STATUS_LEN  fixed bin(31) value(128);

 define structure
   1 WSAData,
     2 wVersion       type WORD,
     2 wHighVersion   type WORD,
     2 szDescription  char(WSADESCRIPTION_LEN) varz,
     2 szSystemStatus char(WSASYS_STATUS_LEN) varz,
     2 iMaxSockets    unsigned fixed bin(16),
     2 iMaxUdpDg      unsigned fixed bin(16),
     2 lpVendorInfo   pointer;

 define alias LPWSADATA handle WSAData;

 /* MAKEWORD is declared in the normal Windows Headers */

 /* Definitions related to sockets: types, address families, options, */
 /* taken from the BSD file sys/socket.h.                             */

 /* This is used instead of -1, since the SOCKET type is unsigned.    */
 dcl INVALID_SOCKET type SOCKET value ('ffffffff'xu);
 dcl SOCKET_ERROR   fixed bin (31) value (-1);

 /* The  following  may  be used in place of the address family,     */
 /* socket type, or protocol in a call to WSASocket to indicate      */
 /* that the corresponding value should  be taken from the supplied  */
 /* WSAPROTOCOL_INFO structure instead of the parameter itself.      */
 dcl FROM_PROTOCOL_INFO fixed bin (31) value (-1);

 dcl /* Types */
 ( SOCK_STREAM    value(1) /* stream socket */
 , SOCK_DGRAM     value(2) /* datagram socket */
 , SOCK_RAW       value(3) /* raw-protocol interface */
 , SOCK_RDM       value(4) /* reliably-delivered message */
 , SOCK_SEQPACKET value(5) /* sequenced packet stream */
 ) fixed bin (31);

 dcl /* Option flags per-socket */
 ( SO_DEBUG       value('0001'xn) /* turn on debugging info recording */
 , SO_ACCEPTCONN  value('0002'xn) /* socket has had listen() */
 , SO_REUSEADDR   value('0004'xn) /* allow local address reuse */
 , SO_KEEPALIVE   value('0008'xn) /* keep connections alive */
 , SO_DONTROUTE   value('0010'xn) /* just use interface addresses */
 , SO_BROADCAST   value('0020'xn) /* permit sending of broadcast msgs */
 , SO_USELOOPBACK value('0040'xn) /* bypass hardware when possible */
 , SO_LINGER      value('0080'xn) /* linger on close if data present */
 , SO_OOBINLINE   value('0100'xn) /* leave received OOB data in line */
 , SO_DONTLINGER  value(inot(SO_LINGER))
 ) fixed bin (15);


 dcl /* Additional options, not kept in so_options */
 ( SO_SNDBUF      value('1001'xn) /* send buffer size */
 , SO_RCVBUF      value('1002'xn) /* receive buffer size */
 , SO_SNDLOWAT    value('1003'xn) /* send low-water mark */
 , SO_RCVLOWAT    value('1004'xn) /* receive low-water mark */
 , SO_SNDTIMEO    value('1005'xn) /* send timeout */
 , SO_RCVTIMEO    value('1006'xn) /* receive timeout */
 , SO_ERROR       value('1007'xn) /* get error status and clear */
 , SO_TYPE        value('1008'xn) /* get socket type */
 ) fixed bin (15);



 dcl /* WinSock 2 extension -- new options */
 ( SO_GROUP_ID       value('2001'xn) /* ID of a socket group */
 , SO_GROUP_PRIORITY value('2002'xn) /* rel priority within a group */
 , SO_MAX_MSG_SIZE   value('2003'xn) /* maximum message size */
 , SO_PROTOCOL_INFOA value('2004'xn) /* WSAPROTOCOL_INFOA structure */
 , SO_PROTOCOL_INFOW value('2005'xn) /* WSAPROTOCOL_INFOW structure */
 %if UNICODE ^= '' %then %do;
 , SO_PROTOCOL_INFO  value (SO_PROTOCOL_INFOW)
 %end; %else %do;
 , SO_PROTOCOL_INFO  value (SO_PROTOCOL_INFOW)
 %end; /* UNICODE */
 , PVD_CONFIG        value('3001'xn) /* config info, service provider */
 ) fixed bin (15);

 /* TCP options. */
 dcl TCP_NODELAY fixed bin(15) value('0001'xn);

 /* Address families. */
 /* Although  AF_UNSPEC  is  defined for backwards compatibility,     */
 /* using AF_UNSPEC for the "af" parameter when creating a socket is  */
 /* STRONGLY DISCOURAGED. The interpretation of the "protocol"        */
 /* parameter depends on the actual address family chosen. As         */
 /* environments grow to include more and more address families that  */
 /* use overlapping protocol values there is more and more chance     */
 /* of choosing an undesired address family when AF_UNSPEC is used.   */
 dcl
 ( AF_UNSPEC      value( 0) /* unspecified */
 , AF_UNIX        value( 1) /* local to host (pipes, portals) */
 , AF_INET        value( 2) /* internetwork: UDP, TCP, etc. */
 , AF_IMPLINK     value( 3) /* arpanet imp addresses */
 , AF_PUP         value( 4) /* pup protocols: e.g. BSP */
 , AF_CHAOS       value( 5) /* mit CHAOS protocols */
 , AF_NS          value( 6) /* XEROX NS protocols */
 , AF_IPX         value(AF_NS) /* IPX protocols: IPX, SPX, etc. */
 , AF_ISO         value( 7) /* ISO protocols */
 , AF_OSI         value(AF_ISO) /* OSI is ISO */
 , AF_ECMA        value( 8) /* european computer manufacturers */
 , AF_DATAKIT     value( 9) /* datakit protocols */
 , AF_CCITT       value(10) /* CCITT protocols, X.25 etc */
 , AF_SNA         value(11) /* IBM SNA */
 , AF_DECnet      value(12) /* DECnet */
 , AF_DLI         value(13) /* Direct data link interface */
 , AF_LAT         value(14) /* LAT */
 , AF_HYLINK      value(15) /* NSC Hyperchannel */
 , AF_APPLETALK   value(16) /* Apple Talk */
 , AF_NETBIOS     value(17) /* NetBios-style addresses */
 , AF_VOICEVIEW   value(18) /* VoiceView */
 , AF_FIREFOX     value(19) /* Protocols from Firefox */
 , AF_UNKNOWN1    value(20) /* Somebody is using this! */
 , AF_BAN         value(21) /* Banyan */
 , AF_ATM         value(22) /* Native ATM Services */
 , AF_INET6       value(23) /* Internetwork Version 6 */

 , AF_MAX         value(24)
 ) fixed bin (16) unsigned;


 /* Structure used by kernel to store most addresses. */
 define structure
   1 sockaddr,
     2 sa_family type u_short, /* address family */
     2 sa_data   char(14);     /* up to 14 bytes of direct address */


 /* Structure used by kernel to pass protocol information */
 /* in raw sockets.                                       */
 define structure
   1 sockproto,
     2 sp_family   type u_short, /* address family */
     2 sp_protocol type u_short; /* protocol */


 /* Protocol families, same as address families for now */
 dcl
 ( PF_UNSPEC      value (AF_UNSPEC)
 , PF_UNIX        value (AF_UNIX)
 , PF_INET        value (AF_INET)
 , PF_IMPLINK     value (AF_IMPLINK)
 , PF_PUP         value (AF_PUP)
 , PF_CHAOS       value (AF_CHAOS)
 , PF_NS          value (AF_NS)
 , PF_IPX         value (AF_IPX)
 , PF_ISO         value (AF_ISO)
 , PF_OSI         value (AF_OSI)
 , PF_ECMA        value (AF_ECMA)
 , PF_DATAKIT     value (AF_DATAKIT)
 , PF_CCITT       value (AF_CCITT)
 , PF_SNA         value (AF_SNA)
 , PF_DECnet      value (AF_DECnet)
 , PF_DLI         value (AF_DLI)
 , PF_LAT         value (AF_LAT)
 , PF_HYLINK      value (AF_HYLINK)
 , PF_APPLETALK   value (AF_APPLETALK)
 , PF_NETBIOS     value (AF_NETBIOS)
 , PF_VOICEVIEW   value (AF_VOICEVIEW)
 , PF_FIREFOX     value (AF_FIREFOX)
 , PF_UNKNOWN1    value (AF_UNKNOWN1)
 , PF_BAN         value (AF_BAN)
 , PF_ATM         value (AF_ATM)
 , PF_INET6       value (AF_INET6)
 , PF_MAX         value (AF_MAX)
 ) fixed bin (16) unsigned;

 /* Structure used for manipulating linger option. */
 define structure
   1 linger,
     2 l_onoff  type u_short, /* option on/off */
     2 l_linger type u_short; /* linger time */


 /* Level number for (get/set)sockopt() to apply to socket itself. */
 dcl SOL_SOCKET       fixed bin(31) value('ffff'xn);
     /* options for socket level */

 /* Maximum queue length specifiable by listen. */
 dcl SOMAXCONN        fixed bin(31) value('7fffffff'xn);

 dcl MSG_OOB          fixed bin(31) value('1'xn);
     /* process out-of-band data */
 dcl MSG_PEEK         fixed bin(31) value('2'xn);
     /* peek at incoming message */
 dcl MSG_DONTROUTE    fixed bin(31) value('4'xn);
     /* send without using routing tables */

 dcl MSG_PARTIAL      fixed bin(31) value('8000'xn);
     /* partial send or recv for message xport */

 /* WinSock 2 extension -- new flags for WSASend(), WSASendTo(), */
 /* WSARecv() and WSARecvFrom()                                  */
 dcl MSG_INTERRUPT    fixed bin(31) value('10'xn);
 /* send/recv in the interrupt context */

 dcl MSG_MAXIOVLEN    fixed bin(31) value(16);

 /* Define constant based on rfc883, used by gethostbyxxxx() calls. */
 dcl MAXGETHOSTSTRUCT fixed bin(31) value(1024);

 /* WinSock 2 extension -- bit values and indices for FD_XXX network */
 /* events                                                           */
 dcl FD_READ_BIT      fixed bin(31) value(0),
     FD_READ          fixed bin(31) value(raise2(1,FD_READ_BIT));

 dcl FD_WRITE_BIT     fixed bin(31) value(1),
     FD_WRITE         fixed bin(31) value(raise2(1,FD_WRITE_BIT));

 dcl FD_OOB_BIT       fixed bin(31) value(2),
     FD_OOB           fixed bin (31) value(raise2(1,FD_OOB_BIT));

 dcl FD_ACCEPT_BIT    fixed bin(31) value(3),
     FD_ACCEPT        fixed bin (31) value(raise2(1,FD_ACCEPT_BIT));

 dcl FD_CONNECT_BIT   fixed bin(31) value(4),
     FD_CONNECT       fixed bin (31) value(raise2(1,FD_CONNECT_BIT));

 dcl FD_CLOSE_BIT     fixed bin(31) value(5),
     FD_CLOSE         fixed bin (31) value(raise2(1,FD_CLOSE_BIT));

 dcl FD_QOS_BIT       fixed bin(31) value(6),
     FD_QOS           fixed bin (31) value(raise2(1,FD_QOS_BIT));

 dcl FD_GROUP_QOS_BIT fixed bin(31) value(7),
     FD_GROUP_QOS     fixed bin (31) value(raise2(1,FD_GROUP_QOS_BIT));

 dcl FD_MAX_EVENTS    fixed bin(31) value(8),
     FD_ALL_EVENTS    fixed bin(31) value(raise2(1,FD_MAX_EVENTS) - 1);


 /* All Windows Sockets error constants are biased by WSABASEERR */
 /* from the "normal"                                            */
 %dcl #WSABASEERR fixed ext;
 %if #WSABASEERR = 0 %then %do;
 %#WSABASEERR = 10000;
 dcl WSABASEERR           fixed bin(31) value(10000);

 /* Windows Sockets definitions of regular Microsoft C error constants*/
 dcl
 ( WSAEINTR               value (WSABASEERR+4)
 , WSAEBADF               value (WSABASEERR+9)
 , WSAEACCES              value (WSABASEERR+13)
 , WSAEFAULT              value (WSABASEERR+14)
 , WSAEINVAL              value (WSABASEERR+22)
 , WSAEMFILE              value (WSABASEERR+24)

 /* Windows Sockets definitions of regular Berkeley error constants */
 , WSAEWOULDBLOCK         value (WSABASEERR+35)
 , WSAEINPROGRESS         value (WSABASEERR+36)
 , WSAEALREADY            value (WSABASEERR+37)
 , WSAENOTSOCK            value (WSABASEERR+38)
 , WSAEDESTADDRREQ        value (WSABASEERR+39)
 , WSAEMSGSIZE            value (WSABASEERR+40)
 , WSAEPROTOTYPE          value (WSABASEERR+41)
 , WSAENOPROTOOPT         value (WSABASEERR+42)
 , WSAEPROTONOSUPPORT     value (WSABASEERR+43)
 , WSAESOCKTNOSUPPORT     value (WSABASEERR+44)
 , WSAEOPNOTSUPP          value (WSABASEERR+45)
 , WSAEPFNOSUPPORT        value (WSABASEERR+46)
 , WSAEAFNOSUPPORT        value (WSABASEERR+47)
 , WSAEADDRINUSE          value (WSABASEERR+48)
 , WSAEADDRNOTAVAIL       value (WSABASEERR+49)
 , WSAENETDOWN            value (WSABASEERR+50)
 , WSAENETUNREACH         value (WSABASEERR+51)
 , WSAENETRESET           value (WSABASEERR+52)
 , WSAECONNABORTED        value (WSABASEERR+53)
 , WSAECONNRESET          value (WSABASEERR+54)
 , WSAENOBUFS             value (WSABASEERR+55)
 , WSAEISCONN             value (WSABASEERR+56)
 , WSAENOTCONN            value (WSABASEERR+57)
 , WSAESHUTDOWN           value (WSABASEERR+58)
 , WSAETOOMANYREFS        value (WSABASEERR+59)
 , WSAETIMEDOUT           value (WSABASEERR+60)
 , WSAECONNREFUSED        value (WSABASEERR+61)
 , WSAELOOP               value (WSABASEERR+62)
 , WSAENAMETOOLONG        value (WSABASEERR+63)
 , WSAEHOSTDOWN           value (WSABASEERR+64)
 , WSAEHOSTUNREACH        value (WSABASEERR+65)
 , WSAENOTEMPTY           value (WSABASEERR+66)
 , WSAEPROCLIM            value (WSABASEERR+67)
 , WSAEUSERS              value (WSABASEERR+68)
 , WSAEDQUOT              value (WSABASEERR+69)
 , WSAESTALE              value (WSABASEERR+70)
 , WSAEREMOTE             value (WSABASEERR+71)

 /* Extended Windows Sockets error constant definitions */
 , WSASYSNOTREADY         value (WSABASEERR+91)
 , WSAVERNOTSUPPORTED     value (WSABASEERR+92)
 , WSANOTINITIALISED      value (WSABASEERR+93)
 , WSAEDISCON             value (WSABASEERR+101)
 , WSAENOMORE             value (WSABASEERR+102)
 , WSAECANCELLED          value (WSABASEERR+103)
 , WSAEINVALIDPROCTABLE   value (WSABASEERR+104)
 , WSAEINVALIDPROVIDER    value (WSABASEERR+105)
 , WSAEPROVIDERFAILEDINIT value (WSABASEERR+106)
 , WSASYSCALLFAILURE      value (WSABASEERR+107)
 , WSASERVICE_NOT_FOUND   value (WSABASEERR+108)
 , WSATYPE_NOT_FOUND      value (WSABASEERR+109)
 , WSA_E_NO_MORE          value (WSABASEERR+110)
 , WSA_E_CANCELLED        value (WSABASEERR+111)
 , WSAEREFUSED            value (WSABASEERR+112)
 ) fixed bin (31);

 /* Error return codes from gethostbyname() and gethostbyaddr() */
 /* (when using the resolver). Note that these errors are       */
 /* retrieved via WSAGetLastError() and must therefore follow   */
 /* the rules for avoiding clashes with error numbers from      */
 /* specific implementations or language run-time systems.      */
 /* For this reason the codes are based at WSABASEERR+1001.     */
 /* Note also that [WSA]NO_ADDRESS is defined only for          */
 /* compatibility purposes.                                     */

 /* Authoritative Answer: Host not found */
 dcl WSAHOST_NOT_FOUND          fixed bin (31) value(WSABASEERR+1001);

 /* Non-Authoritative: Host not found, or SERVERFAIL */
 dcl WSATRY_AGAIN               fixed bin (31) value(WSABASEERR+1002);

 /* Non-recoverable errors, FORMERR, REFUSED, NOTIMP */
 dcl WSANO_RECOVERY             fixed bin (31) value(WSABASEERR+1003);

 /* Valid name, no data record of requested type */
 dcl WSANO_DATA                 fixed bin (31) value(WSABASEERR+1004);

 /* Windows Sockets errors redefined as regular Berkeley error       */
 /* constants. These are commented out in Windows NT to avoid        */
 /* conflicts with errno.h. Use the WSA constants instead.           */

 dcl WSA_QOS_RECEIVERS          fixed bin (31) value(WSABASEERR+1005);
     /* at least one Reserve has arrived */
 dcl WSA_QOS_SENDERS            fixed bin (31) value(WSABASEERR+1006);
     /* at least one Path has arrived */
 dcl WSA_QOS_NO_SENDERS         fixed bin (31) value(WSABASEERR+1007);
     /* there are no senders */
 dcl WSA_QOS_NO_RECEIVERS       fixed bin (31) value(WSABASEERR+1008);
     /* there are no receivers */
 dcl WSA_QOS_REQUEST_CONFIRMED  fixed bin (31) value(WSABASEERR+1009);
     /* Reserve has been confirmed */
 dcl WSA_QOS_ADMISSION_FAILURE  fixed bin (31) value(WSABASEERR+1010);
     /* error due to lack of resources */
 dcl WSA_QOS_POLICY_FAILURE     fixed bin (31) value(WSABASEERR+1011);
     /* rejected for administrative reasons - bad credentials */
 dcl WSA_QOS_BAD_STYLE          fixed bin (31) value(WSABASEERR+1012);
     /* unknown or conflicting style */
 dcl WSA_QOS_BAD_OBJECT         fixed bin (31) value(WSABASEERR+1013);
     /* problem with some part of the filterspec or providerspecific */
     /* buffer in general */
 dcl WSA_QOS_TRAFFIC_CTRL_ERROR fixed bin (31) value(WSABASEERR+1014);
     /* problem with some part of the flowspec */
 dcl WSA_QOS_GENERIC_ERROR      fixed bin (31) value(WSABASEERR+1015);
     /* general error */

 /* WinSock error codes are also defined in winerror.cpy. */
 /* Hence the %if                                         */

 %end; /* #WSABASEERR */


 /* Compatibility macros. */
 %dcl h_errno char ext;
 %h_errno = 'WSAGetLastError()';
 dcl HOST_NOT_FOUND             fixed bin (31) value(WSAHOST_NOT_FOUND);
 dcl TRY_AGAIN                  fixed bin (31) value(WSATRY_AGAIN);
 dcl NO_RECOVERY                fixed bin (31) value(WSANO_RECOVERY);
 dcl NO_DATA                    fixed bin (31) value(WSANO_DATA);
 /* no address, look for MX record */
 dcl WSANO_ADDRESS              fixed bin (31) value(WSANO_DATA);
 dcl NO_ADDRESS                 fixed bin (31) value(WSANO_ADDRESS);


 %dcl WSAAPI char ext;
 %WSAAPI = 'options (byvalue linkage(stdcall) nodescriptor)';

 define alias   WSAEVENT      type   HANDLE;
 define alias LPWSAEVENT      type LPHANDLE;
 define alias   WSAOVERLAPPED type OVERLAPPED;
 define alias LPWSAOVERLAPPED handle WSAOVERLAPPED;

 dcl ( WSA_IO_PENDING         value (ERROR_IO_PENDING)
     , WSA_IO_INCOMPLETE      value (ERROR_IO_INCOMPLETE)
     , WSA_INVALID_HANDLE     value (ERROR_INVALID_HANDLE)
     , WSA_INVALID_PARAMETER  value (ERROR_INVALID_PARAMETER)
     , WSA_NOT_ENOUGH_MEMORY  value (ERROR_NOT_ENOUGH_MEMORY)
     , WSA_OPERATION_ABORTED  value (ERROR_OPERATION_ABORTED)
     ) fixed bin (31);

 dcl WSA_INVALID_EVENT         type HANDLE value (sysnull());

 dcl ( WSA_MAXIMUM_WAIT_EVENTS value (MAXIMUM_WAIT_OBJECTS)
     , WSA_WAIT_FAILED         value (iunsigned(bin(-1,31)))
     , WSA_WAIT_EVENT_0        value (WAIT_OBJECT_0)
     , WSA_WAIT_IO_COMPLETION  value (WAIT_IO_COMPLETION)
     , WSA_WAIT_TIMEOUT        value (WAIT_TIMEOUT)
     , WSA_INFINITE            value (INFINITE)
     ) type DWORD;

 /* WinSock 2 extension -- WSABUF and QOS struct, include qos.cpy */
 /* to pull in FLOWSPEC and related definitions                   */
 define structure
   1 WSABUF,
     2 len type u_long,/* the length of the buffer */
     2 buf pointer;    /* the pointer to the buffer */
 define alias LPWSABUF handle WSABUF;

 %include qos;

 define structure
   1 #QualityOfService,
     2 SendingFlowspec   type FLOWSPEC,
        /* the flow spec for data sending */
     2 ReceivingFlowspec type FLOWSPEC,
        /* the flow spec for data receiving */
     2 ProviderSpecific  type WSABUF;
        /* additional provider specific stuff */

 define alias   QOS type   #QualityOfService;
 define alias LPQOS handle #QualityOfService;

 /* WinSock 2 extension -- manifest constants for return values of */
 /* the condition function                                         */
 dcl CF_ACCEPT  fixed bin(15) value('0000'xn);
 dcl CF_REJECT  fixed bin(15) value('0001'xn);
 dcl CF_DEFER   fixed bin(15) value('0002'xn);

 /* WinSock 2 extension -- manifest constants for shutdown() */
 dcl SD_RECEIVE fixed bin(31) value('00'xn);
 dcl SD_SEND    fixed bin(31) value('01'xn);
 dcl SD_BOTH    fixed bin(31) value('02'xn);

 /* WinSock 2 extension -- data type and manifest constants for */
 /* socket groups                                               */
 define alias GROUP unsigned fixed bin(32);

 dcl SG_UNCONSTRAINED_GROUP fixed bin(32) value('01'xu);
 dcl SG_CONSTRAINED_GROUP   fixed bin(32) value('02'xu);

 /* WinSock 2 extension -- data type for WSAEnumNetworkEvents() */
 define structure
   1 WSANETWORKEVENTS,
     2 lNetworkEvents fixed bin(31),
     2 iErrorCode (FD_MAX_EVENTS) fixed bin(31);

 define alias LPWSANETWORKEVENTS handle WSANETWORKEVENTS;

 /* WinSock 2 extension -- WSAPROTOCOL_INFO structure and associated  */
 /* manifest constants                                                */

 %dcl GUID_DEFINED char ext;
 %if GUID_DEFINED = '' %then %do;
 %GUID_DEFINED = 'Y';
 define structure
   1 GUID,
     2 Data1 unsigned fixed bin(32),
     2 Data2 unsigned fixed bin(16),
     2 Data3 unsigned fixed bin(16),
     2 Data4 (8) char(1);
 %end; /* GUID_DEFINED */

 %dcl #_LPGUID_DEFINED__ char ext;
 %if #_LPGUID_DEFINED__ = '' %then %do;
 %#_LPGUID_DEFINED__ = 'Y';
 define alias LPGUID handle GUID;
 %end;

 dcl MAX_PROTOCOL_CHAIN fixed bin(31) value(7);

 dcl BASE_PROTOCOL fixed bin(31) value(1);
 dcl LAYERED_PROTOCOL fixed bin(31) value(0);

 define structure
   1 WSAPROTOCOLCHAIN,
     2 ChainLen fixed bin(31),/* the length of the chain, */
       /* length = 0 means layered protocol, */
       /* length = 1 means base protocol, */
       /* length > 1 means protocol chain */
     2 ChainEntries (MAX_PROTOCOL_CHAIN) type DWORD;
       /* a list of dwCatalogEntryIds */

 define alias LPWSAPROTOCOLCHAIN handle WSAPROTOCOLCHAIN;

 dcl WSAPROTOCOL_LEN fixed bin(31) value(255);

 define structure
   1 WSAPROTOCOL_INFOA,
     2 dwServiceFlags1    type DWORD,
     2 dwServiceFlags2    type DWORD,
     2 dwServiceFlags3    type DWORD,
     2 dwServiceFlags4    type DWORD,
     2 dwProviderFlags    type DWORD,
     2 ProviderId         type GUID,
     2 dwCatalogEntryId   type DWORD,
     2 ProtocolChain      type WSAPROTOCOLCHAIN,
     2 iVersion           fixed bin(31),
     2 iAddressFamily     fixed bin(31),
     2 iMaxSockAddr       fixed bin(31),
     2 iMinSockAddr       fixed bin(31),
     2 iSocketType        fixed bin(31),
     2 iProtocol          fixed bin(31),
     2 iProtocolMaxOffset fixed bin(31),
     2 iNetworkByteOrder  fixed bin(31),
     2 iSecurityScheme    fixed bin(31),
     2 dwMessageSize      type DWORD,
     2 dwProviderReserved type DWORD,
     2 szProtocol         char (WSAPROTOCOL_LEN) varz;
 define alias LPWSAPROTOCOL_INFOA handle WSAPROTOCOL_INFOA;

 define structure
   1 WSAPROTOCOL_INFOW,
     2 dwServiceFlags1    type DWORD,
     2 dwServiceFlags2    type DWORD,
     2 dwServiceFlags3    type DWORD,
     2 dwServiceFlags4    type DWORD,
     2 dwProviderFlags    type DWORD,
     2 ProviderId         type GUID,
     2 dwCatalogEntryId   type DWORD,
     2 ProtocolChain      type WSAPROTOCOLCHAIN,
     2 iVersion           fixed bin(31),
     2 iAddressFamily     fixed bin(31),
     2 iMaxSockAddr       fixed bin(31),
     2 iMinSockAddr       fixed bin(31),
     2 iSocketType        fixed bin(31),
     2 iProtocol          fixed bin(31),
     2 iProtocolMaxOffset fixed bin(31),
     2 iNetworkByteOrder  fixed bin(31),
     2 iSecurityScheme    fixed bin(31),
     2 dwMessageSize      type DWORD,
     2 dwProviderReserved type DWORD,
     2 szProtocol         WCHAR (WSAPROTOCOL_LEN) varz;
 define alias LPWSAPROTOCOL_INFOW handle WSAPROTOCOL_INFOW;

 %if UNICODE ^= '' %then %do;
 define alias WSAPROTOCOL_INFO    type WSAPROTOCOL_INFOW;
 define alias LPWSAPROTOCOL_INFO  type LPWSAPROTOCOL_INFOW;
 %end; %else %do;
 define alias WSAPROTOCOL_INFO    type WSAPROTOCOL_INFOA;
 define alias LPWSAPROTOCOL_INFO  type LPWSAPROTOCOL_INFOA;
 %end; /* UNICODE */

 /* Flag bit definitions for dwProviderFlags */
 dcl PFL_MULTIPLE_PROTO_ENTRIES   fixed bin(31) value('00000001'xn),
     PFL_RECOMMENDED_PROTO_ENTRY  fixed bin(31) value('00000002'xn),
     PFL_HIDDEN                   fixed bin(31) value('00000004'xn),
     PFL_MATCHES_PROTOCOL_ZERO    fixed bin(31) value('00000008'xn);

 /* Flag bit definitions for dwServiceFlags1 */
 dcl XP1_CONNECTIONLESS           fixed bin(31) value('00000001'xn),
     XP1_GUARANTEED_DELIVERY      fixed bin(31) value('00000002'xn),
     XP1_GUARANTEED_ORDER         fixed bin(31) value('00000004'xn),
     XP1_MESSAGE_ORIENTED         fixed bin(31) value('00000008'xn),
     XP1_PSEUDO_STREAM            fixed bin(31) value('00000010'xn),
     XP1_GRACEFUL_CLOSE           fixed bin(31) value('00000020'xn),
     XP1_EXPEDITED_DATA           fixed bin(31) value('00000040'xn),
     XP1_CONNECT_DATA             fixed bin(31) value('00000080'xn),
     XP1_DISCONNECT_DATA          fixed bin(31) value('00000100'xn),
     XP1_SUPPORT_BROADCAST        fixed bin(31) value('00000200'xn),
     XP1_SUPPORT_MULTIPOINT       fixed bin(31) value('00000400'xn),
     XP1_MULTIPOINT_CONTROL_PLANE fixed bin(31) value('00000800'xn),
     XP1_MULTIPOINT_DATA_PLANE    fixed bin(31) value('00001000'xn),
     XP1_QOS_SUPPORTED            fixed bin(31) value('00002000'xn),
     XP1_INTERRUPT                fixed bin(31) value('00004000'xn),
     XP1_UNI_SEND                 fixed bin(31) value('00008000'xn),
     XP1_UNI_RECV                 fixed bin(31) value('00010000'xn),
     XP1_IFS_HANDLES              fixed bin(31) value('00020000'xn),
     XP1_PARTIAL_MESSAGE          fixed bin(31) value('00040000'xn);

 dcl BIGENDIAN    fixed bin(15) value('0000'xn),
     LITTLEENDIAN fixed bin(15) value('0001'xn);

 dcl SECURITY_PROTOCOL_NONE fixed bin(15) value('0000'xn);

 /* WinSock 2 extension -- manifest constants for WSAJoinLeaf() */
 dcl JL_SENDER_ONLY             fixed bin(15) value('01'xn),
     JL_RECEIVER_ONLY           fixed bin(15) value('02'xn),
     JL_BOTH                    fixed bin(15) value('04'xn);

 /* WinSock 2 extension -- manifest constants for WSASocket() */
 dcl WSA_FLAG_OVERLAPPED        fixed bin(15) value('01'xn),
     WSA_FLAG_MULTIPOINT_C_ROOT fixed bin(15) value('02'xn),
     WSA_FLAG_MULTIPOINT_C_LEAF fixed bin(15) value('04'xn),
     WSA_FLAG_MULTIPOINT_D_ROOT fixed bin(15) value('08'xn),
     WSA_FLAG_MULTIPOINT_D_LEAF fixed bin(15) value('10'xn);

 /* WinSock 2 extension -- manifest constants for WSAIoctl() */
 dcl IOC_UNIX     fixed bin(31) value('00000000'xn);
 dcl IOC_WS2      fixed bin(31) value('08000000'xn);
 dcl IOC_PROTOCOL fixed bin(31) value('10000000'xn);
 dcl IOC_VENDOR   fixed bin(31) value('18000000'xn);

 %#WSAIO: proc (x, y) returns (char);
   dcl (x,y) char;
   return ('ior(IOC_VOID, ' || x || ', ' || y || ')');
 %end;
 %#WSAIOR: proc (x, y) returns (char);
   dcl (x,y) char;
   return ('ior(IOC_OUT,' || x || ', ' || y || ')');
 %end;
 %#WSAIOW: proc (x, y) returns (char);
   dcl (x,y) char;
   return ('ior(IOC_IN, ' || x || ', ' || y || ')');
 %end;
 %#WSAIORW: proc (x, y) returns (char);
   dcl (x,y) char;
   return ('ior(IOC_INOUT, ' || x || ', ' || y || ')');
 %end;
 %dcl (#WSAIO, #WSAIOR, #WSAIOW, #WSAIORW) entry;

 dcl
 ( SIO_ASSOCIATE_HANDLE               value (#WSAIOW (IOC_WS2, 1))
 , SIO_ENABLE_CIRCULAR_QUEUEING       value (#WSAIO  (IOC_WS2, 2))
 , SIO_FIND_ROUTE                     value (#WSAIOR (IOC_WS2, 3))
 , SIO_FLUSH                          value (#WSAIO  (IOC_WS2, 4))
 , SIO_GET_BROADCAST_ADDRESS          value (#WSAIOR (IOC_WS2, 5))
 , SIO_GET_EXTENSION_FUNCTION_PTR     value (#WSAIORW(IOC_WS2, 6))
 , SIO_GET_QOS                        value (#WSAIORW(IOC_WS2, 7))
 , SIO_GET_GROUP_QOS                  value (#WSAIORW(IOC_WS2, 8))
 , SIO_MULTIPOINT_LOOPBACK            value (#WSAIOW (IOC_WS2, 9))
 , SIO_MULTICAST_SCOPE                value (#WSAIOW (IOC_WS2,10))
 , SIO_SET_QOS                        value (#WSAIOW (IOC_WS2,11))
 , SIO_SET_GROUP_QOS                  value (#WSAIOW (IOC_WS2,12))
 , SIO_TRANSLATE_HANDLE               value (#WSAIORW(IOC_WS2,13))
 ) fixed bin (31);

 /* WinSock 2 extension -- manifest constants for */
 /* SIO_TRANSLATE_HANDLE ioctl                    */
 dcl TH_NETDEV fixed bin(31) value('00000001'xn);
 dcl TH_TAPI   fixed bin(31) value('00000002'xn);

 /* Microsoft Windows Extended data types required for the functions  */
 /* to convert back and forth between binary and string forms of      */
 /* addresses.                                                        */

 define alias  PSOCKADDR pointer; /* not HANDLE, as too messy to use */
 define alias LPSOCKADDR pointer; /* not HANDLE, as too messy to use */

 /* Manifest constants and type definitions related to */
 /* name resolution and registration (RNR) API         */

 %dcl #tagBLOB_DEFINED char ext;
 %if #tagBLOB_DEFINED = '' %then %do;
 %#tagBLOB_DEFINED = 'Y';
 %dcl #BLOB_DEFINED char ext;
 %#BLOB_DEFINED = 'Y';
 %dcl #LPBLOB_DEFINED char ext;
 %#LPBLOB_DEFINED = 'Y';
 define structure
   1 BLOB,
     2 cbSize    type ULONG,
     2 pBlobData pointer;

 define alias LPBLOB handle BLOB;
 %end;

 /* Service Install Flags */

 dcl SERVICE_MULTIPLE fixed bin(31) value('00000001'xn);

 /* Name Spaces */

 dcl NS_ALL         fixed bin(31) value(0),

     NS_SAP         fixed bin(31) value(1),
     NS_NDS         fixed bin(31) value(2),
     NS_PEER_BROWSE fixed bin(31) value(3),

     NS_TCPIP_LOCAL fixed bin(31) value(10),
     NS_TCPIP_HOSTS fixed bin(31) value(11),
     NS_DNS         fixed bin(31) value(12),
     NS_NETBT       fixed bin(31) value(13),
     NS_WINS        fixed bin(31) value(14),

     NS_NBP         fixed bin(31) value(20),

     NS_MS          fixed bin(31) value(30),
     NS_STDA        fixed bin(31) value(31),
     NS_NTDS        fixed bin(31) value(32),

     NS_X500        fixed bin(31) value(40),
     NS_NIS         fixed bin(31) value(41),
     NS_NISPLUS     fixed bin(31) value(42),

     NS_WRQ         fixed bin(31) value(50),
     NS_NETDES      fixed bin(31) value(60);
                    /* Network Designers Limited */

 /* Resolution flags for WSAGetAddressByName().               */
 /* Note these are also used by the 1.1 API GetAddressByName, */
 /* so leave them around.                                     */
 dcl RES_UNUSED_1    fixed bin(31) value('00000001'xn),
     RES_FLUSH_CACHE fixed bin(31) value('00000002'xn),
     RES_SERVICE     fixed bin(31) value('00000004'xn);

 /* Well known value names for Service Types */

 dcl
 ( SERVICE_TYPE_VALUE_IPXPORTA  init to(varz) ('IpxSocket')
 /* , SERVICE_TYPE_VALUE_IPXPORTW init to(varz) ('IpxSocket'L) */
 , SERVICE_TYPE_VALUE_SAPIDA    init to(varz) ('SapId')
 /* , SERVICE_TYPE_VALUE_SAPIDW init to(varz) ('SapId'L)       */

 , SERVICE_TYPE_VALUE_TCPPORTA  init to(varz) ('TcpPort')
 /* , SERVICE_TYPE_VALUE_TCPPORTW init to(varz) ('TcpPort'L)   */

 , SERVICE_TYPE_VALUE_UDPPORTA  init to(varz) ('UdpPort')
 /* , SERVICE_TYPE_VALUE_UDPPORTW init to(varz) ('UdpPort'L)   */

 , SERVICE_TYPE_VALUE_OBJECTIDA init to(varz) ('ObjectId')
 /* , SERVICE_TYPE_VALUE_OBJECTIDW init to(varz) ('ObjectId'L) */
 ) static pointer;

 %dcl SERVICE_TYPE_VALUE_SAPID    char ext;
 %dcl SERVICE_TYPE_VALUE_TCPPORT  char ext;
 %dcl SERVICE_TYPE_VALUE_UDPPORT  char ext;
 %dcl SERVICE_TYPE_VALUE_OBJECTID char ext;
 %if UNICODE ^= '' %then %do; /* to be sorted out */
  %SERVICE_TYPE_VALUE_SAPID    = 'SERVICE_TYPE_VALUE_SAPIDW';
  %SERVICE_TYPE_VALUE_TCPPORT  = 'SERVICE_TYPE_VALUE_TCPPORTW';
  %SERVICE_TYPE_VALUE_UDPPORT  = 'SERVICE_TYPE_VALUE_UDPPORTW';
  %SERVICE_TYPE_VALUE_OBJECTID = 'SERVICE_TYPE_VALUE_OBJECTIDW';
 %end; %else %do;
  %SERVICE_TYPE_VALUE_SAPID    = 'SERVICE_TYPE_VALUE_SAPIDA';
  %SERVICE_TYPE_VALUE_TCPPORT  = 'SERVICE_TYPE_VALUE_TCPPORTA';
  %SERVICE_TYPE_VALUE_UDPPORT  = 'SERVICE_TYPE_VALUE_UDPPORTA';
  %SERVICE_TYPE_VALUE_OBJECTID = 'SERVICE_TYPE_VALUE_OBJECTIDA';
 %end;

 %dcl #_CSADDR_DEFINED__ char ext;
 %if #_CSADDR_DEFINED__ = '' %then %do;
 %#_CSADDR_DEFINED__ = 'Y';

 /* SockAddr Information */
 define structure
   1 SOCKET_ADDRESS,
     2 lpSockaddr      type LPSOCKADDR,
     2 iSockaddrLength type INT;

 define alias  PSOCKET_ADDRESS handle SOCKET_ADDRESS;
 define alias LPSOCKET_ADDRESS handle SOCKET_ADDRESS;

 /* CSAddr Information */
 define structure
   1 CSADDR_INFO,
     2 LocalAddr   type SOCKET_ADDRESS,
     2 RemoteAddr  type SOCKET_ADDRESS,
     2 iSocketType type INT,
     2 iProtocol   type INT;

 define alias  PCSADDR_INFO handle CSADDR_INFO;
 define alias LPCSADDR_INFO handle CSADDR_INFO;
 %end; /* __CSADDR_DEFINED__ */

 /*  Address Family/Protocol Tuples */
 define structure
   1 AFPROTOCOLS,
     2 iAddressFamily type INT,
     2 iProtocol      type INT;

 define alias  PAFPROTOCOLS handle AFPROTOCOLS;
 define alias LPAFPROTOCOLS handle AFPROTOCOLS;

 /* Client Query API Typedefs */

 /* The comparators */
 define ordinal
      WSAEcomparator
         ( COMP_EQUAL,
           COMP_NOTLESS
         ) prec(31);
 define alias  PWSAECOMPARATOR pointer;
 define alias LPWSAECOMPARATOR pointer;

 define structure
   1 WSAVersion,
     2 dwVersion type DWORD,
     2 ecHow     type WSAECOMPARATOR;

 define alias  PWSAVERSION handle WSAVersion;
 define alias LPWSAVERSION handle WSAVersion;

 define structure
   1 WSAQuerySetA,
     2 dwSize                  type DWORD,
     2 lpszServiceInstanceName type LPSTR,
     2 lpServiceClassId        type LPGUID,
     2 lpVersion               type LPWSAVERSION,
     2 lpszComment             type LPSTR,
     2 dwNameSpace             type DWORD,
     2 lpNSProviderId          type LPGUID,
     2 lpszContext             type LPSTR,
     2 dwNumberOfProtocols     type DWORD,
     2 lpafpProtocols          type LPAFPROTOCOLS,
     2 lpszQueryString         type LPSTR,
     2 dwNumberOfCsAddrs       type DWORD,
     2 lpcsaBuffer             type LPCSADDR_INFO,
     2 dwOutputFlags           type DWORD,
     2 lpBlob                  type LPBLOB;

 define alias  PWSAQUERYSETA handle WSAQuerySetA;
 define alias LPWSAQUERYSETA handle WSAQuerySetA;
 define structure
   1 WSAQuerySetW,
     2 dwSize                  type DWORD,
     2 lpszServiceInstanceName type LPWSTR,
     2 lpServiceClassId        type LPGUID,
     2 lpVersion               type LPWSAVERSION,
     2 lpszComment             type LPWSTR,
     2 dwNameSpace             type DWORD,
     2 lpNSProviderId          type LPGUID,
     2 lpszContext             type LPWSTR,
     2 dwNumberOfProtocols     type DWORD,
     2 lpafpProtocols          type LPAFPROTOCOLS,
     2 lpszQueryString         type LPWSTR,
     2 dwNumberOfCsAddrs       type DWORD,
     2 lpcsaBuffer             type LPCSADDR_INFO,
     2 dwOutputFlags           type DWORD,
     2 lpBlob                  type LPBLOB;

 define alias  PWSAQUERYSETW handle WSAQuerySetW;
 define alias LPWSAQUERYSETW handle WSAQuerySetW;
 %if UNICODE ^= '' %then %do;
 define alias   WSAQUERYSET  type   WSAQUERYSETW;
 define alias  PWSAQUERYSET  type  PWSAQUERYSETW;
 define alias LPWSAQUERYSET  type LPWSAQUERYSETW;
 %end; %else %do;
 define alias   WSAQUERYSET  type   WSAQUERYSETA;
 define alias  PWSAQUERYSET  type  PWSAQUERYSETA;
 define alias LPWSAQUERYSET  type LPWSAQUERYSETA;
 %end; /* UNICODE */

 dcl LUP_DEEP                fixed bin(31) value('0001'xn),
     LUP_CONTAINERS          fixed bin(31) value('0002'xn),
     LUP_NOCONTAINERS        fixed bin(31) value('0004'xn),
     LUP_NEAREST             fixed bin(31) value('0008'xn),
     LUP_RETURN_NAME         fixed bin(31) value('0010'xn),
     LUP_RETURN_TYPE         fixed bin(31) value('0020'xn),
     LUP_RETURN_VERSION      fixed bin(31) value('0040'xn),
     LUP_RETURN_COMMENT      fixed bin(31) value('0080'xn),
     LUP_RETURN_ADDR         fixed bin(31) value('0100'xn),
     LUP_RETURN_BLOB         fixed bin(31) value('0200'xn),
     LUP_RETURN_ALIASES      fixed bin(31) value('0400'xn),
     LUP_RETURN_QUERY_STRING fixed bin(31) value('0800'xn),
     LUP_RETURN_ALL          fixed bin(31) value('0FF0'xn),
     LUP_RES_SERVICE         fixed bin(31) value('8000'xn),

     LUP_FLUSHCACHE          fixed bin(31) value('1000'xn),
     LUP_FLUSHPREVIOUS       fixed bin(31) value('2000'xn);


 /* Return flags */

 dcl RESULT_IS_ALIAS         fixed bin(31) value('0001'xn);

 /* Service Address Registration and Deregistration Data Types. */

 define ordinal
      WSAESETSERVICEOP
         ( RNRSERVICE_REGISTER,
           RNRSERVICE_DEREGISTER,
           RNRSERVICE_DELETE
         ) prec(31);
 define alias  PWSAESETSERVICEOP pointer;
 define alias LPWSAESETSERVICEOP pointer;

 /* Service Installation/Removal Data Types. */

 define structure
   1 WSANSClassInfoA,
     2 lpszName    type LPSTR,
     2 dwNameSpace type DWORD,
     2 dwValueType type DWORD,
     2 dwValueSize type DWORD,
     2 lpValue     type LPVOID;

 define alias  PWSANSCLASSINFOA handle WSANSClassInfoA;
 define alias LPWSANSCLASSINFOA handle WSANSClassInfoA;
 define structure
   1 WSANSClassInfoW,
     2 lpszName    type LPWSTR,
     2 dwNameSpace type DWORD,
     2 dwValueType type DWORD,
     2 dwValueSize type DWORD,
     2 lpValue     type LPVOID;

 define alias  PWSANSCLASSINFOW handle WSANSClassInfoW;
 define alias LPWSANSCLASSINFOW handle WSANSClassInfoW;
 %if UNICODE ^= '' %then %do;
 define alias   WSANSCLASSINFO type   WSANSCLASSINFOW;
 define alias  PWSANSCLASSINFO type  PWSANSCLASSINFOW;
 define alias LPWSANSCLASSINFO type LPWSANSCLASSINFOW;
 %end; %else %do;
 define alias   WSANSCLASSINFO type   WSANSCLASSINFOA;
 define alias  PWSANSCLASSINFO type  PWSANSCLASSINFOA;
 define alias LPWSANSCLASSINFO type LPWSANSCLASSINFOA;
 %end; /* UNICODE */

 define structure
   1 WSAServiceClassInfoA,
     2 lpServiceClassId     type LPGUID,
     2 lpszServiceClassName type LPSTR,
     2 dwCount              type DWORD,
     2 lpClassInfos         type LPWSANSCLASSINFOA;
 define alias  PWSASERVICECLASSINFOA handle WSAServiceClassInfoA;
 define alias LPWSASERVICECLASSINFOA handle WSAServiceClassInfoA;

 define structure
   1 WSAServiceClassInfoW,
     2 lpServiceClassId     type LPGUID,
     2 lpszServiceClassName type LPWSTR,
     2 dwCount              type DWORD,
     2 lpClassInfos         type LPWSANSCLASSINFOW;
 define alias  PWSASERVICECLASSINFOW handle WSAServiceClassInfoW;
 define alias LPWSASERVICECLASSINFOW handle WSAServiceClassInfoW;

 %if UNICODE ^= '' %then %do;
 define alias   WSASERVICECLASSINFO type   WSASERVICECLASSINFOW;
 define alias  PWSASERVICECLASSINFO type  PWSASERVICECLASSINFOW;
 define alias LPWSASERVICECLASSINFO type LPWSASERVICECLASSINFOW;
 %end; %else %do;
 define alias   WSASERVICECLASSINFO type   WSASERVICECLASSINFOA;
 define alias  PWSASERVICECLASSINFO type  PWSASERVICECLASSINFOA;
 define alias LPWSASERVICECLASSINFO type LPWSASERVICECLASSINFOA;
 %end; /* UNICODE */

 define structure
   1 WSANAMESPACE_INFOA,
     2 NSProviderId   type GUID,
     2 dwNameSpace    type DWORD,
     2 fActive        type BOOL,
     2 dwVersion      type DWORD,
     2 lpszIdentifier type LPSTR;

 define alias  PWSANAMESPACE_INFOA handle WSANAMESPACE_INFOA;
 define alias LPWSANAMESPACE_INFOA handle WSANAMESPACE_INFOA;
 define structure
   1 WSANAMESPACE_INFOW,
     2 NSProviderId   type GUID,
     2 dwNameSpace    type DWORD,
     2 fActive        type BOOL,
     2 dwVersion      type DWORD,
     2 lpszIdentifier type LPWSTR;

 define alias  PWSANAMESPACE_INFOW handle WSANAMESPACE_INFOW;
 define alias LPWSANAMESPACE_INFOW handle WSANAMESPACE_INFOW;
 %if UNICODE ^= '' %then %do;
 define alias   WSANAMESPACE_INFO type   WSANAMESPACE_INFOW;
 define alias  PWSANAMESPACE_INFO type  PWSANAMESPACE_INFOW;
 define alias LPWSANAMESPACE_INFO type LPWSANAMESPACE_INFOW;
 %end; %else %do;
 define alias   WSANAMESPACE_INFO type   WSANAMESPACE_INFOA;
 define alias  PWSANAMESPACE_INFO type  PWSANAMESPACE_INFOA;
 define alias LPWSANAMESPACE_INFO type LPWSANAMESPACE_INFOA;
 %end; /* UNICODE */

 /* Socket function prototypes */

 %if INCL_WINSOCK_API_PROTOTYPES ^= 'N' %then %do;
 dcl accept
     entry (type SOCKET,
            type LPSOCKADDR,          /* to appropriate sockaddr */
            fixed bin (31) byaddr)
     returns (byvalue optional type SOCKET )
     WSAAPI external ('accept');

 dcl bind
     entry (type SOCKET,
            type LPSOCKADDR,          /* to appropriate sockaddr */
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('bind');

 dcl closesocket
     entry (type SOCKET )
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('closesocket');

 dcl connect
     entry (type SOCKET,
            type LPSOCKADDR,          /* to appropriate sockaddr */
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('connect');

 dcl ioctlsocket
     entry (type SOCKET,
            fixed bin(31),
            pointer)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('ioctlsocket');

 dcl getpeername
     entry (type SOCKET,
            type LPSOCKADDR,          /* to appropriate sockaddr */
            fixed bin (31) byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('getpeername');

 dcl getsockname
     entry (type SOCKET,
            type LPSOCKADDR,          /* to appropriate sockaddr */
            fixed bin (31) byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('getsockname');

 dcl getsockopt
     entry (type SOCKET,
            fixed bin(31),
            fixed bin(31),
            pointer,
            fixed bin (31) byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('getsockopt');

 dcl htonl
     entry (type u_long)
     returns (byvalue type u_long )
     WSAAPI external ('htonl');

 dcl htons
     entry (type u_short)
     returns (byvalue type u_short )
     WSAAPI external ('htons');

 dcl inet_addr
     entry (char (*) varz byaddr)
     returns (byvalue type u_long )
     WSAAPI external ('inet_addr');

 dcl inet_ntoa
     entry (type in_addr)
     returns(pointer byvalue) /* use pli_p2z for result */
     WSAAPI external ('inet_ntoa');

 dcl listen
     entry (type SOCKET,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('listen');

 dcl ntohl
     entry (type u_long)
     returns (byvalue type u_long )
     WSAAPI external ('ntohl');

 dcl ntohs
     entry (type u_short)
     returns (byvalue type u_short )
     WSAAPI external ('ntohs');

 dcl recv
     entry (type SOCKET,
            pointer,
            fixed bin(31),
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('recv');

 dcl recvfrom
     entry (type SOCKET,
            pointer,
            fixed bin(31),
            fixed bin(31),
            type LPSOCKADDR,          /* to appropriate sockaddr */
            fixed bin (31) byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('recvfrom');

 dcl select
     entry (fixed bin(31),
            handle fd_set,
            handle fd_set,
            handle fd_set,
            handle timeval)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('select');

 dcl send
     entry (type SOCKET,
            pointer,
            fixed bin(31),
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('send');

 dcl sendto
     entry (type SOCKET,
            pointer,
            fixed bin(31),
            fixed bin(31),
            type LPSOCKADDR,       /* to appropriate sockaddr */
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('sendto');

 dcl setsockopt
     entry (type SOCKET,
            fixed bin(31),
            fixed bin(31),
            pointer,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('setsockopt');

 dcl shutdown
     entry (type SOCKET,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('shutdown');

 dcl socket
     entry (fixed bin(31),
            fixed bin(31),
            fixed bin(31))
     returns (byvalue optional type SOCKET )
     WSAAPI external ('socket');

 /* Database function prototypes */

 dcl gethostbyaddr
     entry (pointer,         /* to host address in network byte order */
            fixed bin (31),
            fixed bin (31))
     returns (handle hostent byvalue optional)
     WSAAPI external ('gethostbyaddr');

 dcl gethostbyname
     entry (char (*) varz byaddr)
     returns(handle hostent byvalue optional)
     WSAAPI external ('gethostbyname');

 dcl gethostname
     entry (char (*) varz byaddr,
            fixed bin(31))
     returns (fixed bin(31) byvalue )
     WSAAPI external ('gethostname');

 dcl getservbyport
     entry (fixed bin (31),
            char (*) varz byaddr optional)
     returns (handle servent byvalue optional)
     WSAAPI external ('getservbyport');

 dcl getservbyname
     entry (char (*) varz byaddr,
            char (*) varz byaddr optional)
     returns (handle servent byvalue optional)
     WSAAPI external ('getservbyname');

 dcl getprotobynumber
     entry (fixed bin (31))
     returns (handle protoent byvalue optional)
     WSAAPI external ('getprotobynumber');

 dcl getprotobyname
     entry (char (*) varz byaddr)
     returns (handle protoent byvalue optional)
     WSAAPI external ('getprotobyname');

 /* Microsoft Windows Extension function prototypes */

 dcl WSAStartup
     entry (type WORD,
            type WSAData byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAStartup');


 dcl WSACleanup
     entry ()
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSACleanup');


 dcl WSASetLastError
     entry (fixed bin(31))
     WSAAPI external ('WSASetLastError');


 dcl WSAGetLastError
     entry ()
     returns (fixed bin(31) byvalue)
     WSAAPI external ('WSAGetLastError');

 /* WSAIsBlocking()          is obsolete and consequently absent */
 /* WSAUnhookBlockingHook()  is obsolete and consequently absent */
 /* WSASetBlockingHook()     is obsolete and consequently absent */
 /* WSACancelBlockingCall () is obsolete and consequently absent */

 dcl WSAAsyncGetServByName
     entry (type HWND,
            type u_int,
            char (*) varz byaddr nonasgn,
            char (*) varz byaddr nonasgn,
            char (*) varz byaddr,
            fixed bin(31))
     returns (byvalue optional type HANDLE)
     WSAAPI external ('WSAAsyncGetServByName');

 dcl WSAAsyncGetServByPort
     entry (type HWND,
            type u_int,
            fixed bin(31),
            char (*) varz byaddr nonasgn,
            char (*) varz byaddr,
            fixed bin(31))
     returns (byvalue optional type HANDLE)
     WSAAPI external ('WSAAsyncGetServByPort');


 dcl WSAAsyncGetProtoByName
     entry (type HWND,
            type u_int,
            char (*) varz byaddr nonasgn,
            char (*) varz byaddr,
            fixed bin(31))
     returns (byvalue optional type HANDLE)
     WSAAPI external ('WSAAsyncGetProtoByName');


 dcl WSAAsyncGetProtoByNumber
     entry (type HWND,
            type u_int,
            fixed bin(31),
            char (*) varz byaddr,
            fixed bin(31))
     returns (byvalue optional type HANDLE)
     WSAAPI external ('WSAAsyncGetProtoByNumber');


 dcl WSAAsyncGetHostByName
     entry (type HWND,
            type u_int,
            char (*) varz byaddr nonasgn,
            char (*) varz byaddr,
            fixed bin(31))
     returns (byvalue optional type HANDLE)
     WSAAPI external ('WSAAsyncGetHostByName');


 dcl WSAAsyncGetHostByAddr
     entry (type HWND,
            type u_int,
            pointer,         /* to host address in network byte order */
            fixed bin(31),
            fixed bin(31),
            char (*) varz byaddr,
            fixed bin(31))
     returns (byvalue optional type HANDLE )
     WSAAPI external ('WSAAsyncGetHostByAddr');

 dcl WSACancelAsyncRequest
     entry (type HANDLE)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSACancelAsyncRequest');


 dcl WSAAsyncSelect
     entry (type SOCKET,
            type HWND,
            type u_int,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAAsyncSelect');
 %end; /* INCL_WINSOCK_API_PROTOTYPES */

 %if INCL_WINSOCK_API_TYPEDEFS ^= 'N' %then %do;

 define alias LPFN_accept
        entry (type SOCKET,
               type LPSOCKADDR,          /* to appropriate sockaddr */
               fixed bin (31) byaddr)
        returns (byvalue optional type SOCKET )
        WSAAPI limited;

 define alias LPFN_bind
        entry (type SOCKET,
               type LPSOCKADDR,          /* to appropriate sockaddr */
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_closesocket
        entry (type SOCKET)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_connect
        entry (type SOCKET,
               type LPSOCKADDR,          /* to appropriate sockaddr */
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_ioctlsocket
        entry (type SOCKET,
               fixed bin(31),
               pointer)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_getpeername
        entry (type SOCKET,
               type LPSOCKADDR,
               fixed bin (31) byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_getsockname
        entry (type SOCKET,
               type LPSOCKADDR,          /* to appropriate sockaddr */
               fixed bin (31) byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_getsockopt
        entry (type SOCKET,
               fixed bin(31),
               fixed bin(31),
               pointer,
               fixed bin (31) byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_htonl
        entry (type u_long)
        returns (byvalue type u_long )
        WSAAPI limited;

 define alias LPFN_htons
        entry (type u_short)
        returns (byvalue type u_short )
        WSAAPI limited;

 define alias LPFN_inet_addr
        entry (char (*) varz byaddr)
        returns (byvalue type u_long )
        WSAAPI limited;

 define alias LPFN_inet_ntoa
        entry (type in_addr)
        returns(pointer byvalue) /* use pli_p2z for result */
        WSAAPI limited;

 define alias LPFN_listen
        entry (type SOCKET,
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_ntohl
        entry (type u_long)
        returns (byvalue type u_long )
        WSAAPI limited;

 define alias LPFN_ntohs
        entry (type u_short)
        returns (byvalue type u_short )
        WSAAPI limited;

 define alias LPFN_recv
        entry (type SOCKET,
               pointer,
               fixed bin(31),
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_recvfrom
        entry (type SOCKET,
               pointer,
               fixed bin(31),
               fixed bin(31),
               type LPSOCKADDR,          /* to appropriate sockaddr */
               fixed bin (31) byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_select
        entry (fixed bin(31),
               handle fd_set,
               handle fd_set,
               handle fd_set,
               handle timeval)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_send
        entry (type SOCKET,
               pointer,
               fixed bin(31),
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_sendto
        entry (type SOCKET,
               pointer,
               fixed bin(31),
               fixed bin(31),
               type LPSOCKADDR,       /* to appropriate sockaddr */
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_setsockopt
        entry (type SOCKET,
               fixed bin(31),
               fixed bin(31),
               pointer,
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_shutdown
        entry (type SOCKET,
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_socket
        entry (fixed bin(31),
               fixed bin(31),
               fixed bin(31))
        returns (byvalue optional type SOCKET )
        WSAAPI limited;

 /* Database function prototypes */

 define alias LPFN_gethostbyaddr
        entry (pointer,      /* to host address in network byte order */
               fixed bin (31),
               fixed bin (31))
        returns(handle hostent byvalue optional)
        WSAAPI limited;

 define alias LPFN_gethostbyname
        entry (char (*) varz byaddr)
        returns(handle hostent byvalue optional)
        WSAAPI limited;

 define alias LPFN_gethostname
        entry (char (*) varz byaddr,
               fixed bin(31))
        returns (fixed bin(31) byvalue )
        WSAAPI limited;

 define alias LPFN_getservbyport
        entry (fixed bin (31),
               char (*) varz byaddr optional)
        returns (handle servent byvalue optional)
        WSAAPI limited;

 define alias LPFN_getservbyname
        entry (char (*) varz byaddr,
               char (*) varz byaddr optional)
        returns (handle servent byvalue optional)
        WSAAPI limited;

 define alias LPFN_getprotobynumber
        entry (fixed bin (31))
        returns (handle protoent byvalue optional)
        WSAAPI limited;

 define alias LPFN_getprotobyname
        entry (char (*) varz byaddr)
        returns (handle protoent byvalue optional)
        WSAAPI limited;

 /* Microsoft Windows Extension function prototypes */

 define alias LPFN_WSAStartup
        entry (type WORD,
               type WSAData byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSACleanup
        entry ()
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSASetLastError
        entry (fixed bin(31))
        WSAAPI limited;

 define alias LPFN_WSAGetLastError
        entry ()
        returns (fixed bin(31) byvalue)
        WSAAPI limited;

 /* WSAIsBlocking()          is obsolete and consequently absent */
 /* WSAUnhookBlockingHook()  is obsolete and consequently absent */
 /* WSASetBlockingHook()     is obsolete and consequently absent */
 /* WSACancelBlockingCall () is obsolete and consequently absent */

 define alias LPFN_WSAAsyncGetServByName
        entry (type HWND,
               type u_int,
               char (*) varz byaddr nonasgn,
               char (*) varz byaddr nonasgn,
               char (*) varz byaddr,
               fixed bin(31))
        returns (byvalue optional type HANDLE )
        WSAAPI limited;

 define alias LPFN_WSAAsyncGetServByPort
        entry (type HWND,
               type u_int,
               fixed bin(31),
               char (*) varz byaddr nonasgn,
               char (*) varz byaddr,
               fixed bin(31))
        returns (byvalue optional type HANDLE )
        WSAAPI limited;

 define alias LPFN_WSAAsyncGetProtoByName
        entry (type HWND,
               type u_int,
               char (*) varz byaddr nonasgn,
               char (*) varz byaddr,
               fixed bin(31))
        returns (byvalue optional type HANDLE )
        WSAAPI limited;

 define alias LPFN_WSAAsyncGetProtoByNumber
        entry (type HWND,
               type u_int,
               fixed bin(31),
               char (*) varz byaddr,
               fixed bin(31))
        returns (byvalue optional type HANDLE )
        WSAAPI limited;

 define alias LPFN_WSAAsyncGetHostByName
        entry (type HWND,
               type u_int,
               char (*) varz byaddr nonasgn,
               char (*) varz byaddr,
               fixed bin(31))
        returns (byvalue optional type HANDLE )
        WSAAPI limited;

 define alias LPFN_WSAAsyncGetHostByAddr
        entry (type HWND,
               type u_int,
               pointer,      /* to host address in network byte order */
               fixed bin(31),
               fixed bin(31),
               char (*) varz byaddr,
               fixed bin(31))
        returns (byvalue optional type HANDLE )
        WSAAPI limited;

 define alias LPFN_WSACancelAsyncRequest
        entry (type HANDLE)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSAAsyncSelect
        entry (type SOCKET,
               type HWND,
               type u_int,
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;
 %end;

 /*********************************************************************/
 /* WinSock 2 extensions -- data types for the condition function     */
 /* in WSAAccept() and overlapped I/O completion routine.             */
 /*********************************************************************/

  define alias LPCONDITIONPROC
         entry (type WSABUF byaddr,
                type WSABUF byaddr,
                type QOS byaddr,
                type QOS byaddr,
                type WSABUF byaddr,
                type WSABUF byaddr,
                pointer,
                type DWORD)
         returns (type int optional byvalue)
         limited CALLBACK;

  define alias LPWSAOVERLAPPED_COMPLETION_ROUTINE
         entry (type DWORD,
                type DWORD,
                type OVERLAPPED byaddr,
                type DWORD)
         limited CALLBACK;


 /* WinSock 2 API new function prototypes */

 %if INCL_WINSOCK_API_PROTOTYPES ^= 'N' %then %do;
 dcl WSAAccept
     entry (type SOCKET,
            type LPSOCKADDR,          /* to appropriate sockaddr */
            fixed bin (31) byaddr optional,
            type LPCONDITIONPROC optional,
            type DWORD)
     returns (byvalue optional type SOCKET )
     WSAAPI external ('WSAAccept');

 dcl WSACloseEvent
     entry (type WSAEVENT)
     returns (byvalue optional type BOOL )
     WSAAPI external ('WSACloseEvent');

 dcl WSAConnect
     entry (type SOCKET,
            type LPSOCKADDR,       /* to appropriate sockaddr */
            fixed bin(31),
            type WSABUF byaddr,
            type WSABUF byaddr,
            type QOS byaddr,
            type QOS byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAConnect');

 dcl WSACreateEvent
     entry ()
     returns (byvalue optional type WSAEVENT )
     WSAAPI external ('WSACreateEvent');

 dcl WSADuplicateSocketA
     entry (type SOCKET,
            type DWORD,
           type WSAPROTOCOL_INFOA byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSADuplicateSocketA');

 dcl WSADuplicateSocketW
     entry (type SOCKET,
            type DWORD,
            type WSAPROTOCOL_INFOW byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSADuplicateSocketW');
 %dcl WSADuplicateSocket char ext;
 %if UNICODE ^= '' %then %do;
 %WSADuplicateSocket = 'WSADuplicateSocketW';
 %end; %else %do;
 %WSADuplicateSocket = 'WSADuplicateSocketA';
 %end; /* !UNICODE */

 dcl WSAEnumNetworkEvents
     entry (type SOCKET,
            type WSAEVENT,
            type WSANETWORKEVENTS byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAEnumNetworkEvents');

 dcl WSAEnumProtocolsA
     entry (fixed bin (31) byaddr,
            type WSAPROTOCOL_INFOA byaddr,
            fixed bin (31) byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAEnumProtocolsA');

 dcl WSAEnumProtocolsW
     entry (fixed bin (31) byaddr,
            type WSAPROTOCOL_INFOW byaddr,
            fixed bin (31) byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAEnumProtocolsW');
 %dcl WSAEnumProtocols char ext;
 %if UNICODE ^= '' %then %do;
 %WSAEnumProtocols = 'WSAEnumProtocolsW';
 %end; %else %do;
 %WSAEnumProtocols = 'WSAEnumProtocolsA';
 %end; /* !UNICODE */

 dcl WSAEventSelect
     entry (type SOCKET,
            type WSAEVENT,
            fixed bin(31))
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAEventSelect');

 dcl WSAGetOverlappedResult
     entry (type SOCKET,
            type WSAOVERLAPPED byaddr,
            fixed bin (31) byaddr,
            type BOOL,
            fixed bin (31) byaddr)
     returns (byvalue optional type BOOL )
     WSAAPI external ('WSAGetOverlappedResult');

 dcl WSAGetQOSByName
     entry (type SOCKET,
            type WSABUF byaddr,
            type QOS byaddr)
     returns (byvalue optional type BOOL )
     WSAAPI external ('WSAGetQOSByName');

 dcl WSAHtonl
     entry (type SOCKET,
            type u_long,
           type u_long byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAHtonl');

 dcl WSAHtons
     entry (type SOCKET,
            type u_short,
            type u_short byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAHtons');

 dcl WSAIoctl
     entry (type SOCKET,
            type DWORD,
            type LPVOID,
            type DWORD,
            type LPVOID,
            type DWORD,
            fixed bin (31) byaddr,
            type WSAOVERLAPPED byaddr optional,
            type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSAIoctl');

 dcl WSAJoinLeaf
     entry (type SOCKET,
            type LPSOCKADDR,       /* to appropriate sockaddr */
            fixed bin(31),
            type WSABUF byaddr,
            type WSABUF byaddr,
            type QOS byaddr,
            type QOS byaddr,
            type DWORD)
     returns (byvalue optional type SOCKET )
     WSAAPI external ('WSAJoinLeaf');

 dcl WSANtohl
     entry (type SOCKET,
            type u_long,
            type u_long byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSANtohl');

 dcl WSANtohs
     entry (type SOCKET,
            type u_short,
            type u_short byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSANtohs');

 dcl WSARecv
     entry (type SOCKET,
            type WSABUF byaddr,
            type DWORD,
            fixed bin (31) byaddr,
            fixed bin (31) byaddr,
            type WSAOVERLAPPED byaddr optional,
            type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSARecv');

 dcl WSARecvDisconnect
     entry (type SOCKET,
            type WSABUF byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSARecvDisconnect');

 dcl WSARecvFrom
     entry (type SOCKET,
            type WSABUF byaddr,
            type DWORD,
            fixed bin (31) byaddr,
            fixed bin (31) byaddr,
            type LPSOCKADDR,
            fixed bin (31) byaddr,
            type WSAOVERLAPPED byaddr optional,
            type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSARecvFrom');

 dcl WSAResetEvent
     entry (type WSAEVENT)
     returns (byvalue optional type BOOL )
     WSAAPI external ('WSAResetEvent');

 dcl WSASend
     entry (type SOCKET,
            type WSABUF byaddr,
            type DWORD,
            fixed bin (31) byaddr,
            type DWORD,
            type WSAOVERLAPPED byaddr optional,
            type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSASend');

 dcl WSASendDisconnect
     entry (type SOCKET,
            type WSABUF byaddr)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSASendDisconnect');

 dcl WSASendTo
     entry (type SOCKET,
            type WSABUF byaddr,
            type DWORD,
            fixed bin (31) byaddr,
            type DWORD,
            type LPSOCKADDR,       /* to appropriate sockaddr */
            fixed bin(31),
            type WSAOVERLAPPED byaddr optional,
            type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
     returns (fixed bin(31) byvalue optional)
     WSAAPI external ('WSASendTo');

 dcl WSASetEvent
     entry (type WSAEVENT)
     returns (byvalue optional type BOOL )
     WSAAPI external ('WSASetEvent');

 dcl WSASocketA
     entry (fixed bin(31),
            fixed bin(31),
            fixed bin(31),
            type WSAPROTOCOL_INFOA byaddr,
            type GROUP,
            type DWORD)
     returns (byvalue optional type SOCKET )
     WSAAPI external ('WSASocketA');

 dcl WSASocketW
     entry (fixed bin(31),
            fixed bin(31),
            fixed bin(31),
            type WSAPROTOCOL_INFOW byaddr,
            type GROUP,
            type DWORD)
     returns (byvalue optional type SOCKET )
     WSAAPI external ('WSASocketW');
 %dcl WSASocket char ext;
 %if UNICODE ^= '' %then %do;
 %WSASocket='WSASocketW';
 %end; %else %do;
 %WSASocket='WSASocketA';
 %end; /* !UNICODE */

 dcl WSAWaitForMultipleEvents
     entry (type DWORD,
            (*) type WSAEVENT byaddr nonasgn,
            type BOOL,
            type DWORD,
            type BOOL)
     returns (byvalue optional type DWORD )
     WSAAPI external ('WSAWaitForMultipleEvents');

 dcl WSAAddressToStringA
     entry (type LPSOCKADDR,          /* to appropriate sockaddr */
            type DWORD,
            type WSAPROTOCOL_INFOA byaddr,
            char (*) varz byaddr,
            fixed bin (31) byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAAddressToStringA');

 dcl WSAAddressToStringW
     entry (type LPSOCKADDR,          /* to appropriate sockaddr */
            type DWORD,
            type WSAPROTOCOL_INFOW byaddr,
            WCHAR(*) varz byaddr,
            fixed bin (31) byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAAddressToStringW');
 %dcl WSAAddressToString char ext;
 %if UNICODE ^= '' %then %do;
 %WSAAddressToString='WSAAddressToStringW';
 %end; %else %do;
 %WSAAddressToString='WSAAddressToStringA';
 %end; /* !UNICODE */

 dcl WSAStringToAddressA
     entry (char (*) varz byaddr,
            type INT,
            type WSAPROTOCOL_INFOA byaddr,
            type LPSOCKADDR,
            fixed bin (31) byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAStringToAddressA');

 dcl WSAStringToAddressW
     entry (type LPSTR,
            type INT,
            type WSAPROTOCOL_INFOW byaddr,
            type LPSOCKADDR,
            fixed bin (31) byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAStringToAddressW');
 %dcl WSAStringToAddress char ext;
 %if UNICODE ^= '' %then %do;
 %WSAStringToAddress='WSAStringToAddressW';
 %end; %else %do;
 %WSAStringToAddress='WSAStringToAddressA';
 %end; /* !UNICODE */

 /* Registration and Name Resolution API functions */

 dcl WSALookupServiceBeginA
     entry (type WSAQUERYSETA byaddr,
            type DWORD,
            type HANDLE byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSALookupServiceBeginA');

 dcl WSALookupServiceBeginW
     entry (type WSAQUERYSETW byaddr,
            type DWORD,
            type HANDLE byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSALookupServiceBeginW');

 %dcl WSALookupServiceBegin char ext;
 %if UNICODE ^= '' %then %do;
 %WSALookupServiceBegin='WSALookupServiceBeginW';
 %end; %else %do;
 %WSALookupServiceBegin='WSALookupServiceBeginA';
 %end; /* !UNICODE */

 dcl WSALookupServiceNextA
     entry (type HANDLE,
            type DWORD,
            fixed bin (31) byaddr,
            type WSAQUERYSETA byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSALookupServiceNextA');

 dcl WSALookupServiceNextW
     entry (type HANDLE,
            type DWORD,
            fixed bin (31) byaddr,
            type WSAQUERYSETW byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSALookupServiceNextW');

 %dcl WSALookupServiceNext char ext;
 %if UNICODE ^= '' %then %do;
 %WSALookupServiceNext='WSALookupServiceNextW';
 %end; %else %do;
 %WSALookupServiceNext='WSALookupServiceNextA';
 %end; /* !UNICODE */

 dcl WSALookupServiceEnd
     entry (type HANDLE)
     returns (byvalue optional type INT)
     WSAAPI external ('WSALookupServiceEnd');

 dcl WSAInstallServiceClassA
     entry (type WSASERVICECLASSINFOA byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAInstallServiceClassA');

 dcl WSAInstallServiceClassW
     entry (type WSASERVICECLASSINFOW byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAInstallServiceClassW');
 %dcl WSAInstallServiceClass char ext;
 %if UNICODE ^= '' %then %do;
 %WSAInstallServiceClass='WSAInstallServiceClassW';
 %end; %else %do;
 %WSAInstallServiceClass='WSAInstallServiceClassA';
 %end; /* !UNICODE */

 dcl WSARemoveServiceClass
     entry (type GUID byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSARemoveServiceClass');

 dcl WSAGetServiceClassInfoA
     entry (type GUID byaddr,
            type GUID byaddr,
            fixed bin (31) byaddr,
            type WSASERVICECLASSINFOA byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAGetServiceClassInfoA');

 dcl WSAGetServiceClassInfoW
     entry (type GUID byaddr,
            type GUID byaddr,
            fixed bin (31) byaddr,
            type WSASERVICECLASSINFOW byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAGetServiceClassInfoW');
 %dcl WSAGetServiceClassInfo char ext;
 %if UNICODE ^= '' %then %do;
 %WSAGetServiceClassInfo='WSAGetServiceClassInfoW';
 %end; %else %do;
 %WSAGetServiceClassInfo='WSAGetServiceClassInfoA';
 %end; /* !UNICODE */

 dcl WSAEnumNameSpaceProvidersA
     entry (fixed bin (31) byaddr,
            type WSANAMESPACE_INFOA byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAEnumNameSpaceProvidersA');

 dcl WSAEnumNameSpaceProvidersW
     entry (fixed bin (31) byaddr,
            type WSANAMESPACE_INFOW byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAEnumNameSpaceProvidersW');
 %dcl WSAEnumNameSpaceProviders char ext;
 %if UNICODE ^= '' %then %do;
 %WSAEnumNameSpaceProviders='WSAEnumNameSpaceProvidersW';
 %end; %else %do;
 %WSAEnumNameSpaceProviders='WSAEnumNameSpaceProvidersA';
 %end; /* !UNICODE */

 dcl WSAGetServiceClassNameByClassIdA
     entry (type GUID byaddr,
           char (*) varz byaddr,
            fixed bin (31) byaddr)
     returns (byvalue optional type INT)
     WSAAPI external ('WSAGetServiceClassNameByClassIdA');

 dcl WSAGetServiceClassNameByClassIdW
     entry (type GUID byaddr,
            WCHAR(*) varz byaddr,
            fixed bin (31) byaddr)
     returns (byvalue optional type INT)
     WSAAPI external('WSAGetServiceClassNameByClassIdW');

 %dcl WSAGetSvcClassNameByClassId char ext;
 %if UNICODE ^= '' %then %do;
 %WSAGetSvcClassNameByClassId='WSAGetSvcClassNameByClassIdW';
 %end; %else %do;
 %WSAGetSvcClassNameByClassId='WSAGetSvcClassNameByClassIdA';
 %end; /* !UNICODE */

 dcl WSASetServiceA
     entry (type WSAQUERYSETA byaddr,
            type WSAESETSERVICEOP,
            type DWORD)
     returns (byvalue optional type INT)
     WSAAPI external ('WSASetServiceA');

 dcl WSASetServiceW
     entry (type WSAQUERYSETW byaddr,
            type WSAESETSERVICEOP,
            type DWORD)
     returns (byvalue optional type INT)
     WSAAPI external ('WSASetServiceW');

 %dcl WSASetService char ext;
 %if UNICODE ^= '' %then %do;
 %WSASetService = 'WSASetServiceW';
 %end; %else %do;
 %WSASetService = 'WSASetServiceA';
 %end; /* !UNICODE */
 %end; /* INCL_WINSOCK_API_PROTOTYPES */

 %if INCL_WINSOCK_API_TYPEDEFS ^= 'N' %then %do;
 define alias LPFN_WSAAccept
        entry (type SOCKET,
               type LPSOCKADDR,            /* to appropriate sockaddr */
               fixed bin (31) byaddr optional,
               type LPCONDITIONPROC optional,
               type DWORD)
        returns (byvalue optional type SOCKET )
        WSAAPI limited;

 define alias LPFN_WSACloseEvent
        entry (type WSAEVENT)
        returns (byvalue optional type BOOL )
        WSAAPI limited;

 define alias LPFN_WSAConnect
        entry (type SOCKET,
               type LPSOCKADDR,       /* to appropriate sockaddr */
               fixed bin(31),
               type WSABUF byaddr,
               type WSABUF byaddr,
               type QOS byaddr,
               type QOS byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSACreateEvent
        entry ()
        returns (byvalue optional type WSAEVENT )
        WSAAPI limited;

 define alias LPFN_WSADuplicateSocketA
        entry (type SOCKET,
               type DWORD,
               type WSAPROTOCOL_INFO byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSADuplicateSocketW
        entry (type SOCKET,
               type DWORD,
               type WSAPROTOCOL_INFOW byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSADuplicateSocket type LPFN_WSADuplicateSocketW;
 %end; %else %do;
 define alias LPFN_WSADuplicateSocket type LPFN_WSADuplicateSocketA;
 %end; /* !UNICODE */

 define alias LPFN_WSAEnumNetworkEvents
        entry (type SOCKET,
               type WSAEVENT,
               type WSANETWORKEVENTS byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSAEnumProtocolsA
        entry (fixed bin (31) byaddr,
               type WSAPROTOCOL_INFOA byaddr,
               fixed bin (31) byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSAEnumProtocolsW
        entry (fixed bin (31) byaddr,
               type WSAPROTOCOL_INFOW byaddr,
               fixed bin (31) byaddr)
          returns (fixed bin(31) byvalue optional)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSAEnumProtocols type LPFN_WSAEnumProtocolsW;
 %end; %else %do;
 define alias LPFN_WSAEnumProtocols type LPFN_WSAEnumProtocolsA;
 %end; /* !UNICODE */

 define alias LPFN_WSAEventSelect
        entry (type SOCKET,
               type WSAEVENT,
               fixed bin(31))
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSAGetOverlappedResult
        entry (type SOCKET,
               type WSAOVERLAPPED byaddr optional,
               fixed bin (31) byaddr,
               type BOOL,
               fixed bin (31) byaddr)
        returns (byvalue optional type BOOL )
        WSAAPI limited;

 define alias LPFN_WSAGetQOSByName
        entry (type SOCKET,
               type WSABUF byaddr,
               type QOS byaddr)
        returns (byvalue optional type BOOL )
        WSAAPI limited;

 define alias LPFN_WSAHtonl
        entry (type SOCKET,
               type u_long,
               type u_long byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSAHtons
        entry (type SOCKET,
               type u_short,
               type u_short byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSAIoctl
        entry (type SOCKET,
               type DWORD,
               type LPVOID,
               type DWORD,
               type LPVOID,
               type DWORD,
               fixed bin (31) byaddr,
               type WSAOVERLAPPED byaddr optional,
               type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSAJoinLeaf
        entry (type SOCKET,
               type LPSOCKADDR,       /* to appropriate sockaddr */
               fixed bin(31),
               type WSABUF byaddr,
               type WSABUF byaddr,
               type QOS byaddr,
               type QOS byaddr,
               type DWORD)
        returns (byvalue optional type SOCKET )
        WSAAPI limited;

 define alias LPFN_WSANtohl
        entry (type SOCKET,
               type u_long,
               type u_long byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSANtohs
        entry (type SOCKET,
               type u_short,
               type u_short byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSARecv
        entry (type SOCKET,
               type WSABUF byaddr,
               type DWORD,
               fixed bin (31) byaddr,
               fixed bin (31) byaddr,
               type WSAOVERLAPPED byaddr optional,
               type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSARecvDisconnect
        entry (type SOCKET,
               type WSABUF byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSARecvFrom
        entry (type SOCKET,
               type WSABUF byaddr,
               type DWORD,
               fixed bin (31) byaddr,
               fixed bin (31) byaddr,
               type LPSOCKADDR,
               fixed bin (31) byaddr,
               type WSAOVERLAPPED byaddr optional,
               type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSAResetEvent
        entry (type WSAEVENT)
        returns (byvalue optional type BOOL )
        WSAAPI limited;

 define alias LPFN_WSASend
        entry (type SOCKET,
               type WSABUF byaddr,
               type DWORD,
               fixed bin (31) byaddr,
               type DWORD,
               type WSAOVERLAPPED byaddr optional,
               type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSASendDisconnect
        entry (type SOCKET,
               type WSABUF byaddr)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSASendTo
        entry (type SOCKET,
               type WSABUF byaddr,
               type DWORD,
               fixed bin (31) byaddr,
               type DWORD,
               type LPSOCKADDR,       /* to appropriate sockaddr */
               fixed bin(31),
               type WSAOVERLAPPED byaddr optional,
               type LPWSAOVERLAPPED_COMPLETION_ROUTINE optional)
        returns (fixed bin(31) byvalue optional)
        WSAAPI limited;

 define alias LPFN_WSASetEvent
        entry (type WSAEVENT)
        returns (byvalue optional type BOOL )
        WSAAPI limited;

 define alias LPFN_WSASocketA
        entry (fixed bin(31),
               fixed bin(31),
               fixed bin(31),
               type WSAPROTOCOL_INFOA byaddr,
               type GROUP,
               type DWORD)
        returns (byvalue optional type SOCKET )
        WSAAPI limited;

 define alias LPFN_WSASocketW
        entry (fixed bin(31),
               fixed bin(31),
               fixed bin(31),
               type WSAPROTOCOL_INFOW byaddr,
               type GROUP,
               type DWORD)
        returns (byvalue optional type SOCKET )
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSASocket type LPFN_WSASocketW;
 %end; %else %do;
 define alias LPFN_WSASocket type LPFN_WSASocketA;
 %end; /* !UNICODE */

 define alias LPFN_WSAWaitForMultipleEvents
        entry (type DWORD,
               (*) type WSAEVENT byaddr nonasgn,
               type BOOL,
               type DWORD,
               type BOOL)
        returns (byvalue optional type DWORD )
        WSAAPI limited;

 define alias LPFN_WSAAddressToStringA
        entry (type LPSOCKADDR,
               type DWORD,
               type WSAPROTOCOL_INFOA byaddr,
               char (*) varz byaddr,
               fixed bin (31) byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSAAddressToStringW
        entry (type LPSOCKADDR,
               type DWORD,
               type WSAPROTOCOL_INFOW byaddr,
               WCHAR (*) varz byaddr,
               fixed bin (31) byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSAAddressToString type LPFN_WSAAddressToStringW;
 %end; %else %do;
 define alias LPFN_WSAAddressToString type LPFN_WSAAddressToStringA;
 %end; /* !UNICODE */

 define alias LPFN_WSAStringToAddressA
        entry (char (*) varz byaddr,
               type INT,
               type WSAPROTOCOL_INFOA byaddr,
               type LPSOCKADDR,
               fixed bin (31) byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSAStringToAddressW
        entry (WCHAR (*) varz byaddr,
               type INT,
               type WSAPROTOCOL_INFOW byaddr,
               type LPSOCKADDR,
               fixed bin (31) byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSAStringToAddress type LPFN_WSAStringToAddressW;
 %end; %else %do;
 define alias LPFN_WSAStringToAddress type LPFN_WSAStringToAddressA;
 %end; /* !UNICODE */

 /* Registration and Name Resolution API functions */

 define alias LPFN_WSALookupServiceBeginA
        entry (type WSAQUERYSETA byaddr,
               type DWORD,
               type HANDLE byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSALookupServiceBeginW
        entry (type WSAQUERYSETA byaddr,
               type DWORD,
               type HANDLE byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSALookupServiceBegin
         type LPFN_WSALookupServiceBeginW;
 %end; %else %do;
 define alias LPFN_WSALookupServiceBegin
         type LPFN_WSALookupServiceBeginA;
 %end; /* !UNICODE */

 define alias LPFN_WSALookupServiceNextA
        entry (type HANDLE,
               type DWORD,
               fixed bin (31) byaddr,
               type WSAQUERYSETA byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSALookupServiceNextW
        entry (type HANDLE,
               type DWORD,
               fixed bin (31) byaddr,
               type WSAQUERYSETW byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSALookupServiceNext type LPFN_WSALookupServiceNextW;
 %end; %else %do;
 define alias LPFN_WSALookupServiceNext type LPFN_WSALookupServiceNextA;
 %end; /* !UNICODE */

 define alias LPFN_WSALookupServiceEnd
        entry (type HANDLE)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSAInstallServiceClassA
        entry (type WSASERVICECLASSINFOA byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSAInstallServiceClassW
        entry (type WSASERVICECLASSINFOW byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias
   LPFN_WSAInstallServiceClass type LPFN_WSAInstallServiceClassW;
 %end; %else %do;
 define alias
   LPFN_WSAInstallServiceClass type LPFN_WSAInstallServiceClassA;
 %end; /* !UNICODE */

 define alias LPFN_WSARemoveServiceClass
        entry (type GUID byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSAGetServiceClassInfoA
        entry (type GUID byaddr,
               type GUID byaddr,
               fixed bin (31) byaddr,
               type WSASERVICECLASSINFOA byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSAGetServiceClassInfoW
        entry (type GUID byaddr,
               type GUID byaddr,
               fixed bin (31) byaddr,
               type WSASERVICECLASSINFOW byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias
   LPFN_WSAGetServiceClassInfo type LPFN_WSAGetServiceClassInfoW;
 %end; %else %do;
 define alias
   LPFN_WSAGetServiceClassInfo type LPFN_WSAGetServiceClassInfoA;
 %end; /* !UNICODE */

 define alias LPFN_WSAEnumNameSpaceProvidersA
        entry (fixed bin (31) byaddr,
               type WSANAMESPACE_INFOA byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSAEnumNameSpaceProvidersW
        entry (fixed bin (31) byaddr,
               type WSANAMESPACE_INFOW byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias
   LPFN_WSAEnumNameSpaceProviders type LPFN_WSAEnumNameSpaceProvidersW;
 %end; %else %do;
 define alias
   LPFN_WSAEnumNameSpaceProviders type LPFN_WSAEnumNameSpaceProvidersA;
 %end; /* !UNICODE */

 define alias LPFN_WSAGetServiceClasssNameByClassIdA
        entry (type GUID byaddr,
               char (*) varz byaddr,
               fixed bin (31) byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSAGetServiceClasssNameByClassIdW
        entry (type GUID byaddr,
               WCHAR(*) varz byaddr,
               fixed bin (31) byaddr)
        returns (byvalue optional type INT)
        WSAAPI limited;

 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSAGetServiceClasssNameByClassId
              type LPFN_WSAGetServiceClasssNameByClassIdW;
 %end; %else %do;
 define alias LPFN_WSAGetServiceClasssNameByClassId
              type LPFN_WSAGetServiceClasssNameByClassIdA;
 %end; /* !UNICODE */

 define alias LPFN_WSASetServiceA
        entry (type WSAQUERYSETA byaddr,
               type WSAESETSERVICEOP,
               type DWORD)
        returns (byvalue optional type INT)
        WSAAPI limited;

 define alias LPFN_WSASetServiceW
        entry (type WSAQUERYSETW byaddr,
               type WSAESETSERVICEOP,
               type DWORD)
        returns (byvalue optional type INT)
        WSAAPI limited;
 %if UNICODE ^= '' %then %do;
 define alias LPFN_WSASetService type LPFN_WSASetServiceW;
 %end; %else %do;
 define alias LPFN_WSASetService type LPFN_WSASetServiceA;
 %end; /* !UNICODE */
 %end; /* INCL_WINSOCK_API_TYPEDEFS */


 /* Microsoft Windows Extended data types */
 define alias  PSOCKADDR_IN handle sockaddr_in;
 define alias LPSOCKADDR_IN handle sockaddr_in;
 define alias  PLINGER      handle linger;
 define alias LPLINGER      handle linger;
 define alias  PIN_ADDR     handle in_addr;
 define alias LPIN_ADDR     handle in_addr;
 define alias  PFD_SET      handle fd_set;
 define alias LPFD_SET      handle fd_set;
 define alias  PHOSTENT     handle hostent;
 define alias LPHOSTENT     handle hostent;
 define alias  PSERVENT     handle servent;
 define alias LPSERVENT     handle servent;
 define alias  PPROTOENT    handle protoent;
 define alias LPPROTOENT    handle protoent;
 define alias  PTIMEVAL     handle timeval;
 define alias LPTIMEVAL     handle timeval;

 /* Windows message parameter composition and decomposition macros.   */

 /* WSAMAKEASYNCREPLY is intended for use by the Windows Sockets      */
 /* implementationwhen constructing the response to a                 */
 /* WSAAsyncGetXByY() routine.                                        */
 %WSAMAKEASYNCREPLY: proc (buflen, error) returns (char);
   dcl (buflen, error) char;
   return ('MAKELONG(' || buflen || ',' || error || ')');
 %end;
 %dcl WSAMAKEASYNCREPLY entry;

 /* WSAMAKESELECTREPLY is intended for use by the Windows Sockets     */
 /* implementation when constructing the response to WSAAsyncSelect().*/
 %WSAMAKESELECTREPLY: proc (event, error) returns (char);
   dcl (event, error) char;
   return ('MAKELONG(' || event || ',' || error || ')');
 %end;
 %dcl WSAMAKESELECTREPLY entry;

 /* WSAGETASYNCBUFLEN is intended for use by the Windows Sockets      */
 /* application to extract the buffer length from the lParam in the   */
 %WSAGETASYNCBUFLEN: proc (lParam) returns (char);
   dcl lParam char;
   return ('LOWORD(' || lParam || ')');
 %end;
 %dcl WSAGETASYNCBUFLEN entry;

 /* WSAGETASYNCERROR is intended for use by the Windows Sockets       */
 /* application to extract the error code from the lParam in the      */
 /* response to a WSAGetXByY().                                       */
 %WSAGETASYNCERROR: proc (lParam) returns (char);
   dcl lParam char;
   return ('HIWORD(' || lParam || ')');
 %end;
 %dcl WSAGETASYNCERROR entry;

 /* WSAGETSELECTEVENT is intended for use by the Windows Sockets      */
 /* application to extract the event code from the lParam in the      */
 /* response to a WSAAsyncSelect().                                   */
 %WSAGETSELECTEVENT: proc (lParam) returns (char);
   dcl lParam char;
   return ('LOWORD(' || lParam || ')');
 %end;
 %dcl WSAGETSELECTEVENT entry;

 /* WSAGETSELECTERROR is intended for use by the Windows Sockets      */
 /* application to extract the error code from the lParam in the      */
 /* response to a WSAAsyncSelect().                                   */
 %WSAGETSELECTERROR: proc (lParam) returns (char);
   dcl lParam char;
   return ('HIWORD(' || lParam || ')');
 %end;
 %dcl WSAGETSELECTERROR entry;

 %end; /* #WINSOCK2API_ */
